@@ -53,6 +53,8 @@
             ["mayManageFeatureStore", "Manage Feature Store"],
             ["mayCreateDataCollections", "Create Data Collections"],
             ["mayPublishToDataCollections", "Publish to Data Collections"],
+            ["mayManageEnterpriseAssetLibrary", "Manage Enterprise Asset Library"],
+            ["mayCreateEnterpriseAssetCollections", "Create Enterprise Asset Collections"],
         ];
 
         const usersSortFns = {
