@@ -256,7 +256,7 @@ app.directive('codeEnvResources', ['DataikuAPI', 'Dialogs', 'CreateModalFromTemp
             canUpdateCodeEnv: '=',
             deploymentMode: '='
         },
-        controller: function($scope) {
+        controller: function($scope, $rootScope, $q, PathUtils) {
             $scope.showResourcesSize = false;
             $scope.resourcesCanSelectFn = () => true;
             $scope.resourcesBrowserPath = "/";
@@ -339,7 +339,66 @@ app.directive('codeEnvResources', ['DataikuAPI', 'Dialogs', 'CreateModalFromTemp
             }
 
             $scope.canBeUpdated = function() {
-                return $scope.canUpdateCodeEnv && ['DESIGN_MANAGED', 'PLUGIN_MANAGED', 'AUTOMATION_SINGLE', 'AUTOMATION_VERSIONED'].includes($scope.deploymentMode);
+                return $scope.canUpdateCodeEnv && (($scope.deploymentMode === 'DSS_INTERNAL' && $rootScope.appConfig.admin) || ['DESIGN_MANAGED', 'PLUGIN_MANAGED', 'AUTOMATION_SINGLE', 'AUTOMATION_VERSIONED'].includes($scope.deploymentMode));
+            };
+
+            $scope.openRenameResourceModal = resource => {
+                return Dialogs.prompt($scope, "Rename " + resource.name, 'New name', resource.name, {
+                    // eslint-disable-next-line no-control-regex
+                    pattern: /^(?!\.{1,2}$)[^<>"/\\|?*\x00-\x1F]+$/
+                }).then(function(newName) {
+                    return DataikuAPI.admin.codeenvs[$scope.nodeType.toLowerCase()].renameResource("PYTHON", $scope.envName, resource.fullPath, newName, $scope.codeEnv.versionId).then(function({ data }){
+                        return data;
+                    }).catch(setErrorInScope.bind($scope));
+                });
+            };
+
+            $scope.openMoveResourceModal = resource => {
+                const deferred = $q.defer();
+                CreateModalFromTemplate("/templates/admin/code-envs/common/code-env-move-resource-modal.html", $scope, undefined, function(newScope) {
+                    newScope.resourceToMove = resource;
+                    newScope.sourceParent = $scope.resourcesBrowserPath;
+                    newScope.destParent = $scope.resourcesBrowserPath;
+
+                    newScope.canBrowse = (currentResource) => {
+                        return currentResource.directory && currentResource.fullPath !== resource.fullPath;
+                    };
+
+                    newScope.canSelect = () => false;
+
+                    newScope.cancel = () => {
+                        deferred.resolve();
+                        newScope.dismiss();
+                    };
+
+                    newScope.isSamePath = () => {
+                        return newScope.destParent === newScope.sourceParent;
+                    };
+
+                    newScope.confirmMove = () => {
+                        const destination = PathUtils.absPathFromUserPath(newScope.destParent, newScope.sourceParent);
+                        const sourcePath = resource.fullPath;
+                        const destPath = PathUtils.concatLNT(destination, resource.name);
+                        DataikuAPI.admin.codeenvs[$scope.nodeType.toLowerCase()].moveResource("PYTHON", $scope.envName, sourcePath, destPath, $scope.codeEnv.versionId)
+                            .then(function({ data }){
+                                newScope.dismiss();
+                                deferred.resolve(data);
+                                newScope.dismiss();
+                            }).catch(function(data) {
+                                setErrorInScope.bind($scope)(data);
+                            });
+                    };
+                });
+
+                return deferred.promise;
+            };
+
+            $scope.openDeleteResourceModal = resource => {
+                return Dialogs.confirm($scope, `File deletion (${resource.name})`, `Are you sure you want to delete this ${resource.directory ? 'folder and all its contents' : 'file'}?`).then(function() {
+                    return DataikuAPI.admin.codeenvs[$scope.nodeType.toLowerCase()].deleteResource("PYTHON", $scope.envName, resource.fullPath, $scope.codeEnv.versionId).then(function({ data }){
+                        return data;
+                    }).catch(setErrorInScope.bind($scope));
+                });
             };
 
             DataikuAPI.codeenvs.getUserAccessibleCodeEnvSettings().then(({data}) => {
@@ -363,14 +422,14 @@ app.controller("AdminCodeEnvsUploadResourcesController", ['$scope', 'Assert', 'D
             case "DESIGN":
                 DataikuAPI.admin.codeenvs.design.uploadResources("PYTHON", parentScope.envName, parentScope.resourcesBrowserPath, $scope.resourcesToUpload, $scope.overwrite).then(() => {
                     $scope.dismiss();
-                }).catch(setErrorInScope.bind($scope)).finally(() => {
+                }).catch(setErrorInScope2.bind($scope)).finally(() => {
                     parentScope.executeResourcesBrowse(parentScope.resourcesBrowserPath); // to refresh browser
                 });
                 break;
             case "AUTOMATION":
                 DataikuAPI.admin.codeenvs.automation.uploadResources("PYTHON", parentScope.envName, parentScope.codeEnv.versionId, parentScope.resourcesBrowserPath, $scope.resourcesToUpload, $scope.overwrite).then(() => {
                     $scope.dismiss();
-                }).catch(setErrorInScope.bind($scope)).finally(() => {
+                }).catch(setErrorInScope2.bind($scope)).finally(() => {
                     parentScope.executeResourcesBrowse(parentScope.resourcesBrowserPath); // to refresh browser
                 });
                 break;
@@ -432,12 +491,17 @@ app.component('dssInternalCodeEnv', {
 
                     if ($ctrl.internalCodeEnvType === "PII_DETECTION_CODE_ENV") {
                         DataikuAPI.codeenvs.checkInternalCodeEnvUsedForPII().then(function({data}) {
-                            $ctrl.showPiiDetectionCodeEnvWarning = !data;
+                            $ctrl.showDefaultCodeEnvIsNotUsedWarning = !data;
                         }).catch(setErrorInScope.bind($scope));
                     }
                     if ($ctrl.internalCodeEnvType === "RAG_CODE_ENV") {
                         DataikuAPI.codeenvs.checkInternalCodeEnvUsedForRAG().then(function({data}) {
-                            $ctrl.showRagCodeEnvWarning = !data;
+                            $ctrl.showDefaultCodeEnvIsNotUsedWarning = !data;
+                        }).catch(setErrorInScope.bind($scope));
+                    }
+                    if ($ctrl.internalCodeEnvType === "DOCUMENT_EXTRACTION_CODE_ENV") {
+                        DataikuAPI.codeenvs.checkInternalCodeEnvUsedForDocumentExtraction().then(function({data}) {
+                            $ctrl.showDefaultCodeEnvIsNotUsedWarning = !data;
                         }).catch(setErrorInScope.bind($scope));
                     }
                     if ($ctrl.internalCodeEnvType === "HUGGINGFACE_LOCAL_CODE_ENV") {
