@@ -88,7 +88,7 @@ app.controller("_CMLModelBaseController", function($scope, $controller, DataikuA
     }
 
     $scope.mlTasksContext.exportPredictedData = function() {
-        ExportModelDatasetService.exportPredictedData($scope, $scope.modelData.fullModelId, $scope.table.totalRows);
+        ExportModelDatasetService.exportPredictedData($scope, $scope.modelData.fullModelId);
     };
 
     $scope.mlTasksContext.exportPredictedDataForbiddenReason = function() {
@@ -152,7 +152,8 @@ app.controller("AnalysisClusteringDeployClusterRecipeController", function($scop
             specificSettings: {
                 formatOptionId: $scope.newOutputDataset.formatOptionId,
                 overrideSQLCatalog: $scope.newOutputDataset.overrideSQLCatalog,
-                overrideSQLSchema: $scope.newOutputDataset.overrideSQLSchema
+                overrideSQLSchema: $scope.newOutputDataset.overrideSQLSchema,
+                overrideIcebergNamespace: $scope.newOutputDataset.overrideIcebergNamespace
             },
             partitioningOptionId: $scope.newOutputDataset.partitioningOption || 'NP'
         };
@@ -164,13 +165,9 @@ app.controller("AnalysisClusteringDeployClusterRecipeController", function($scop
         ).success(function(data) {
             WT1.event("analysis-cml-deploy-cluster-recipe-to-the-flow", { from: $scope.deploymentOrigin })
             $scope.dismiss();
-            $state.go("projects.project.flow");
+            $state.go("projects.project.flow", { id: "dataset_" + $stateParams.projectKey + "." + $scope.newOutputDataset.name });
         }).error(setErrorInScope.bind($scope));
     }
-
-    DatasetUtils.listDatasetsUsabilityInAndOut($stateParams.projectKey, "clustering_cluster").then(function(data){
-        $scope.availableInputDatasets = data[0];
-    });
 });
 
 
@@ -217,7 +214,7 @@ app.controller("AnalysisClusteringDeployController", function($scope, $controlle
             ).success(function(data){
                 WT1.event("analysis-cml-deploy-model-to-the-flow", { from: $scope.deploymentOrigin })
                 $scope.dismiss();
-                $state.go("projects.project.flow");
+                $state.go("projects.project.flow", { id: "savedmodel_" + $stateParams.projectKey + "." + data.second });
             }).error(setErrorInScope.bind($scope));
         }
     };
@@ -234,8 +231,9 @@ app.controller("AnalysisClusteringDeployController", function($scope, $controlle
                 $scope.formData.redeployTrainRecipeName,
                 $scope.formData.redeployTrainActivate
             ).success(function(data){
+                const updatedSavedModelId = $scope.redeployableTrains.find(r => r.recipeName === $scope.formData.redeployTrainRecipeName).savedModel.id
                 WT1.event("analysis-cml-redeploy-model-to-the-flow", { from: $scope.deploymentOrigin })
-                var go = $state.go.bind($state, "projects.project.flow");
+                var go = $state.go.bind($state, "projects.project.flow", { id: "savedmodel_" + $stateParams.projectKey + "." + updatedSavedModelId });
                 var parentScope = $scope.$parent;
                 $scope.dismiss();
                 if (data.schemaChanged) {
@@ -263,31 +261,6 @@ app.controller("AnalysisClusteringDeployController", function($scope, $controlle
         fetchManagedDatasetConnections($scope, DataikuAPI);
         $scope.uiState.selectedMode = 'cluster';
         $scope.io = {};
-
-        $scope.deployCluster = function() {
-            Assert.inScope($scope, "newDataset");
-
-            // build the 'settings' object, from the bits the template for the new dataset creation puts in place
-            $scope.newDataset.settings = {
-                connectionId:$scope.newDataset.connectionOption.id,
-                specificSettings : {
-                    formatOptionId:$scope.newDataset.formatOptionId,
-                    overrideSQLCatalog : $scope.newDataset.overrideSQLCatalog,
-                    overrideSQLSchema : $scope.newDataset.overrideSQLSchema
-                },
-                partitioningOptionId:$scope.newDataset.partitioningOption || 'NP'
-            }
-
-            DataikuAPI.analysis.cml.deployCluster($stateParams.fullModelId,
-                $scope.io.mainInputDataset,
-                $scope.newDataset.name,
-                $scope.newDataset.settings
-            ).success(function(data) {
-                WT1.event("analysis-cml-deploy-cluster-recipe-to-the-flow", { from: $scope.deploymentOrigin })
-                $scope.dismiss();
-                $state.go("projects.project.flow");
-            }).error(setErrorInScope.bind($scope));
-        };
     };
 
     $scope.onSelectRedeployCluster = function() {
@@ -309,7 +282,8 @@ app.controller("AnalysisClusteringDeployController", function($scope, $controlle
                         "you may want to check the output schema of this clustering recipe."
                     ).then($state.go.bind($state, "projects.project.recipes.recipe", {recipeName: $scope.formData.redeployClusterRecipeName}));
                 } else {
-                    $state.go("projects.project.flow");
+                    const updatedDatasetName = $scope.redeployableClusters.find(r => r.recipeName === $scope.formData.redeployClusterRecipeName).outputDatasetName
+                    $state.go("projects.project.flow", { id: "dataset_" + $stateParams.projectKey + "." + updatedDatasetName });
                 }
             }).error(setErrorInScope.bind($scope));
         };
