@@ -4,7 +4,7 @@
 const app = angular.module('dataiku.analysis.mlcore');
 
 app.controller("MLTaskFeaturesController", function($scope, $controller, $timeout, $stateParams, $rootScope, Assert, ColumnAnalysesService,
-    DataikuAPI, Fn, Dialogs, PMLSettings, $q, VisualMlCodeEnvCompatibility, CreateModalFromTemplate, Logger, DatasetUtils) {
+    DataikuAPI, Fn, Dialogs, PMLSettings, $q, VisualMlCodeEnvCompatibility, CreateModalFromTemplate, Logger, DatasetUtils, MLFeaturesService) {
     Assert.inScope($scope, "analysisCoreParams");
     $scope.columnAnalysesService = ColumnAnalysesService.init($scope.analysisCoreParams, setErrorInScope.bind($scope));
 
@@ -286,7 +286,7 @@ app.controller("MLTaskFeaturesController", function($scope, $controller, $timeou
         if (nv) {
             $scope.columnAnalysesService.fetchColumnAnalysisIfNeeded(nv._name);
             $scope.fixupFeatureConfiguration(nv, ov);
-            if (ov === null || nv.$idx !== ov.$idx){
+            if (ov === null || nv?.$idx !== ov?.$idx){
                 // reinitialize possible modes, only when we change from one feature to another or select for the 1st time a feature
                 $scope.setMissingValuesModes(nv);
             }
@@ -496,26 +496,17 @@ app.controller("MLTaskFeaturesController", function($scope, $controller, $timeou
         }
     };
 
-    var datasetColumns = [];
-    var datasetLoc = DatasetUtils.getLocFromSmart($stateParams.projectKey, $scope.analysisCoreParams.inputDatasetSmartName);
-    DataikuAPI.datasets.get(datasetLoc.projectKey, datasetLoc.name, $stateParams.projectKey)
-        .success(function(dataset) {
-            datasetColumns = dataset.schema.columns.map(Fn.prop('name'));
-            addDataSetColumnId();
-        }).error(setErrorInScope.bind($scope));
-    var addDataSetColumnId = function() {
-        if (datasetColumns.length === 0) {
-            return;
+    MLFeaturesService.getDatasetColumns($scope.analysisCoreParams).then(function(datasetColumns) {
+        function addDataSetColumnId() {
+            MLFeaturesService.addDataSetColumnId($scope.mlTaskDesign.preprocessing.per_feature, datasetColumns);
+            // Make sure `filteredMultiSelect` directive is aware of change in way the list of features is sorted
+            if ($scope.updateSorted) {
+                $scope.updateSorted();
+            }
         }
-        angular.forEach($scope.mlTaskDesign.preprocessing.per_feature, function(feature) {
-            feature.datasetColumnId = datasetColumns.indexOf(feature._name);
-        });
-        // Make sure `filteredMultiSelect` directive is aware of change in way the list of features is sorted
-        if ($scope.updateSorted) {
-            $scope.updateSorted();
-        }
-    }
-    $scope.$watch('mlTaskDesign.preprocessing.per_feature', addDataSetColumnId);
+        addDataSetColumnId();
+        $scope.$watch('mlTaskDesign.preprocessing.per_feature', addDataSetColumnId);
+    }).catch(setErrorInScope.bind($scope));
 
     $scope.acceptDSSChange = function(feature) {
         Assert.trueish(feature.state.dssWantsToSet, "unexpected call to acceptDSSChange");
@@ -792,7 +783,106 @@ app.controller("MLTaskFeaturesController", function($scope, $controller, $timeou
             feature.datetime_cyclical_periods.push(period);
         }
     };
+    // default ordering to be kept in sync with the featureSortingMethod constant in analysis/prediction/timeseries/settings.js
     $scope.selection = {orderQuery: 'datasetColumnId'};
+
+    Object.defineProperty($scope, 'roleIsSelected', {
+        get: function() {
+            return $scope.feature.role === 'INPUT' || $scope.feature.role === 'INPUT_PAST_ONLY';
+        },
+        set: function(value) {
+            $scope.feature.role = value ? 'INPUT' : 'REJECT';
+        }
+    });
+
+    $scope.setFeatureRole = function(featureRole) {
+        const feature = $scope.selection.filteredObjects.filter(o => o._name === featureRole._name)[0];
+        let state;
+        if (['INPUT', 'INPUT_PAST_ONLY'].includes(featureRole.role)) {
+            state = 'REJECT';
+        } else {
+            state = 'INPUT';
+        }
+        if (feature) {
+            feature.role = state;
+        }
+    };
+
+
+    $scope.hasShiftFromHorizon = function(feature) {
+        return $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name] !== undefined
+            && $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon
+            && $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon.length > 0;
+    };
+
+    $scope.handleChangeInputMode = function (feature, newRole) {
+        if (feature.role === newRole) {
+            return;
+        }
+        feature.role = newRole;
+        if (feature.role === "REJECT") {
+            $scope.selection.selectedObject = undefined;
+            $scope.selection.selectedObjects = [];
+            return;
+        } else {
+            $scope.selection.selectedObject = feature;
+            $scope.selection.selectedObjects = [feature];
+        }
+
+        const oldFeature = angular.copy(feature)
+
+        $scope.fixupFeatureConfiguration(feature, oldFeature);
+        if ($scope.hasShiftFromHorizon(feature)) {
+            CreateModalFromTemplate("/templates/analysis/mlcommon/settings/time-series-reset-shift-modal.html", $scope, "PMLChangeBasicParamsModal", function(newScope) {
+                newScope.feature = feature;
+            });
+        }
+    };
+
+    $scope.handleMultipleInputChange = function (features, newRole) {
+
+        if ($scope.mlTaskDesign.predictionType !== 'TIMESERIES_FORECAST') {
+            $scope.groupSet({role:newRole})
+            return;
+        }
+
+        let shouldOpenModal = false;
+        for (let feature of features.filter(feature => feature.role !== newRole)) {
+            const oldFeature = angular.copy(feature)
+            feature.role = newRole;
+            $scope.fixupFeatureConfiguration(feature, oldFeature);
+            if ($scope.hasShiftFromHorizon(feature)) {
+                shouldOpenModal = true;
+            }
+        }
+        if (shouldOpenModal) {
+            CreateModalFromTemplate("/templates/analysis/mlcommon/settings/time-series-reset-shift-modal.html", $scope, "PMLChangeBasicParamsModal", function(newScope) {
+                newScope.features = features;
+            });
+        }
+    };
+
+    $scope.handleMultipleTypeChange = function (features, targetType) {
+
+        if ($scope.mlTaskDesign.predictionType !== 'TIMESERIES_FORECAST') {
+            $scope.groupSet({type:targetType})
+            return;
+        }
+
+        let shouldOpenModal = false;
+        for (let feature of features.filter(feature => feature.type !== targetType)) {
+            feature.type = targetType;
+            if ($scope.hasShiftFromHorizon(feature)) {
+                shouldOpenModal = true;
+            }
+        }
+        if (shouldOpenModal) {
+            CreateModalFromTemplate("/templates/analysis/mlcommon/settings/time-series-reset-shift-modal.html", $scope, "PMLChangeBasicParamsModal", function(newScope) {
+                newScope.features = features;
+            });
+        }
+    };
+
 });
 
 app.controller("MLTaskOneFeatureController", function($scope, PMLSettings, $stateParams, $filter, DataikuAPI, VisualMlCodeEnvCompatibility, Dialogs) {
@@ -813,6 +903,10 @@ app.controller("MLTaskOneFeatureController", function($scope, PMLSettings, $stat
         if (!$scope.missingValuesModes.some(mode => mode[0] === featureData.missing_handling)) {
             featureData.missing_handling = $scope.missingValuesModes[0][0];
         }
+
+        if (!$scope.featureSupportsAutoShifts(featureData)) {
+            $scope.mlTaskDesign.preprocessing.feature_generation.shifts[featureData._name].from_horizon_mode = 'FIXED';
+        }
     };
 
     $scope.onVariableRoleChange = function() {
@@ -1101,6 +1195,490 @@ app.component("tooManyDroppedRowsWarning", {
     }
 });
 
+app.service("TimeseriesFeatureGenerationService", function($timeout) {
+        this.cleanUpSettings = function(mlTaskDesign) {
+            /*
+            Cleans the timeseries shift values of an ML Task for time-aware feature engineering.
+            Beware, this function is used in the wide prediction context, not only for timeseries ML tasks.
+            */
+            if (mlTaskDesign.predictionType !== "TIMESERIES_FORECAST") {
+                return;
+            }
+
+            for (var featureName in mlTaskDesign.preprocessing.feature_generation.shifts) {
+                const toCheckFromForecast = mlTaskDesign.preprocessing.feature_generation.shifts[featureName].from_forecast;
+                mlTaskDesign.preprocessing.feature_generation.shifts[featureName].from_forecast = this.parseArrayFromStringIfNeeded(toCheckFromForecast);
+
+                const toCheckFromHorizon = mlTaskDesign.preprocessing.feature_generation.shifts[featureName].from_horizon;
+                mlTaskDesign.preprocessing.feature_generation.shifts[featureName].from_horizon = this.parseArrayFromStringIfNeeded(toCheckFromHorizon);
+            }
+        };
+
+        this.parseArrayFromStringIfNeeded = function (input) {
+            if (typeof input === "string") {
+                return [...new Set(Array.from(input.split(",")).map(x => parseInt(x)).filter(x => !isNaN(x)))];
+            }
+            return input;
+        }
+
+        this.isMassEditMode = function(scope, selection) {
+            let newState =  selection.selectedObjects && selection.selectedObjects.length > 0;
+
+            // when going from the mass edit to the per-feature mode, the table can break
+            // to prevent this, we force a rerender
+            if(scope._isMassEditMode && !newState) {
+                if (scope.shouldRerender) {
+                    $timeout(function () {
+                        scope.$broadcast("redrawFatTable");
+                    }, 0);
+                    scope.shouldRerender = false;
+                }
+            }
+
+            scope._isMassEditMode = newState;
+            return newState;
+        };
+
+        this.getTableSize = function (numberOfElements, rowHeight) {
+           return Math.min(numberOfElements, 10) * rowHeight + 10;
+        }
+    });
+
+app.controller('FeatureGenerationController', function($scope, CreateModalFromTemplate) {
+    $scope.openWindowsSettingsModal = function() {
+        CreateModalFromTemplate("/templates/analysis/mlcommon/settings/time-series-windows-settings-modal.html", $scope, null, function(newScope) {
+            newScope.windows_rescale_numericals = $scope.mlTaskDesign.preprocessing.feature_generation.windows_rescale_numericals;
+            newScope.windows_max_categories = $scope.mlTaskDesign.preprocessing.feature_generation.windows_max_categories;
+
+            newScope.save = () => {
+                $scope.mlTaskDesign.preprocessing.feature_generation.windows_rescale_numericals = newScope.windows_rescale_numericals;
+                $scope.mlTaskDesign.preprocessing.feature_generation.windows_max_categories = newScope.windows_max_categories;
+                newScope.dismiss();
+            }
+        }, false, false, true)
+    };
+})
+
+app.controller('FeatureGenerationShiftController', function($scope, CreateModalFromTemplate, TimeseriesFeatureGenerationService) {
+        // ================= Mass operations
+        $scope._isMassEditMode = false; // used by TimeseriesFeatureGenerationService
+
+        $scope.massEditInputIsDirty = false; // whether or not the user interacted with the mass edit inputs
+
+        $scope.$watch('selection', function() {
+            // erase the dirty state when the user deselects everything
+            if ($scope.selection.selectedObjects && $scope.selection.selectedObjects.length === 0) {
+                $scope.massEditInputIsDirty = false;
+            }
+            $scope.initMassEditValues();
+        }, true);
+
+        $scope.setMassEditDirty = function () {
+            $scope.massEditInputIsDirty = true;
+        };
+
+        let redrawFn = function() {
+            $scope.shouldRerender = true;
+        };
+
+        $scope.setMassEditTargetShiftMode = function (target) {
+            $scope.massEditTargetShiftMode = target;
+        };
+
+        $scope.applyMassEditShifts = function () {
+            $scope.setSelectedFeaturesShifts();
+            if ($scope.massEditTargetShiftMode === "AUTO") {
+                $scope.setSelectedFeaturesToAutoShift();
+            }
+            if ($scope.massEditTargetShiftMode === "FIXED") {
+                $scope.setSelectedFeaturesToFixedShifts();
+            }
+
+            $scope.massEditInputIsDirty = false;
+            $scope.initMassEditValues();
+        };
+
+        $(window).on('resize', redrawFn);
+
+        $scope.$on('$destroy', function() {
+            $(window).off('resize', redrawFn);
+        });
+
+        $scope.isMassEditMode = function(selection) {
+            return TimeseriesFeatureGenerationService.isMassEditMode($scope, selection);
+        };
+
+        $scope.getTableSize = function(numberOfElements, rowHeight) {
+            return TimeseriesFeatureGenerationService.getTableSize(numberOfElements, rowHeight);
+        };
+
+        $scope.initMassEditValues = function () {
+            // If all features have identical forecast or horizon shifts, this function sets the value of the mass
+            // selection shifts to the common value. Otherwise, it defaults to empty shifts.
+            if (!$scope.selection.selectedObjects || $scope.selection.selectedObjects.length === 0) {
+                return;
+            }
+
+            if (!$scope.massEditInputIsDirty) {
+                $scope.initMassEditShifts();
+                $scope.initMassEditTargetShiftMode();
+            }
+
+            $scope.validateMassEditShifts();
+        };
+
+        $scope.initMassEditShifts = function () {
+            let forecastShifts = $scope.mlTaskDesign.preprocessing.feature_generation.shifts[$scope.selection.selectedObjects[0]._name].from_forecast;
+
+            forecastShifts = TimeseriesFeatureGenerationService.parseArrayFromStringIfNeeded(forecastShifts)
+
+            let forecastShiftsAreIdentical = $scope.selection.selectedObjects.every(
+                (feature) => {
+                    let featureShifts = $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_forecast;
+                    featureShifts = TimeseriesFeatureGenerationService.parseArrayFromStringIfNeeded(featureShifts);
+                    return featureShifts.length === forecastShifts.length && forecastShifts.every(v => featureShifts.includes(v));
+                }
+            )
+            if (forecastShiftsAreIdentical) {
+                $scope.uiState.massActionForecastShifts = forecastShifts;
+            }
+            else {
+                $scope.uiState.massActionForecastShifts = [];
+            }
+
+            let horizonShifts = $scope.mlTaskDesign.preprocessing.feature_generation.shifts[$scope.selection.selectedObjects[0]._name].from_horizon;
+
+            horizonShifts = TimeseriesFeatureGenerationService.parseArrayFromStringIfNeeded(horizonShifts);
+
+            let horizonShiftsAreIdentical = $scope.selection.selectedObjects.every(
+                (feature) => {
+                    let featureShifts = $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon;
+                    featureShifts = TimeseriesFeatureGenerationService.parseArrayFromStringIfNeeded(featureShifts);
+                    return featureShifts.length === horizonShifts.length && horizonShifts.every(v => featureShifts.includes(v));
+                }
+            )
+
+            if (horizonShiftsAreIdentical) {
+                $scope.uiState.massActionHorizonShifts = horizonShifts;
+            }
+            else {
+                $scope.uiState.massActionHorizonShifts = [];
+            }
+        }
+
+        $scope.initMassEditTargetShiftMode = function () {
+            let autoShiftMode = $scope.mlTaskDesign.preprocessing.feature_generation.shifts[$scope.selection.selectedObjects[0]._name].from_horizon_mode;
+
+            let autoShiftModesAreIdentical = $scope.selection.selectedObjects.every(
+                (feature) => {
+                    return autoShiftMode === $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon_mode;
+                }
+            )
+            $scope.massEditTargetShiftMode = autoShiftModesAreIdentical ? autoShiftMode : null;
+        }
+
+        $scope.getFeatureHavingShiftModeNumber = function (autoShiftMode) {
+            if (!$scope.massEditTargetShiftMode) {
+                return $scope.someSelectedFeaturesAreInMode(autoShiftMode);
+            }
+            if (autoShiftMode !== $scope.massEditTargetShiftMode) {
+                if (autoShiftMode === "FIXED") {
+                    return $scope.someSelectedFeaturesDontSupportAutoShift();
+                }
+            }
+            return false;
+        };
+
+        $scope.autoShiftModeShouldBeStriped = function (autoShiftMode) {
+            if (!$scope.massEditTargetShiftMode) {
+                return $scope.someSelectedFeaturesAreInMode(autoShiftMode);
+            }
+            if (autoShiftMode !== $scope.massEditTargetShiftMode) {
+                if (autoShiftMode === "FIXED") {
+                    return $scope.someSelectedFeaturesDontSupportAutoShift();
+                }
+            }
+            return false;
+        };
+
+        $scope.selectedFeaturesContainsPastCovariate = function () {
+            return $scope.selection.selectedObjects.some(feature => $scope.isPastOnlyFeature(feature._name));
+        };
+
+        $scope.someSelectedFeaturesSupportAutoShift = function () {
+            return $scope.selection.selectedObjects.some(feature => $scope.featureSupportsAutoShifts(feature));
+        };
+
+        $scope.someSelectedFeaturesDontSupportAutoShift = function () {
+            return $scope.selection.selectedObjects.some(feature => !$scope.featureSupportsAutoShifts(feature));
+        };
+
+        $scope.someSelectedFeaturesArePastOnly = function () {
+            return $scope.selection.selectedObjects.some(feature => $scope.isPastOnlyFeature(feature._name));
+        };
+
+        $scope.setSelectedFeaturesToAutoShift = function (){
+            $scope.selection.selectedObjects.forEach(
+                (feature) => {
+                    if ($scope.featureSupportsAutoShifts(feature)) {
+                        $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon_mode = 'AUTO';
+                    }
+                }
+            )
+            if (document.activeElement) {
+                document.activeElement.blur();
+            }
+        };
+
+        $scope.allSelectedFeaturesAreInMode = function (mode) {
+            return $scope.selection.selectedObjects.every(
+                (feature) => {
+                    return $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon_mode === mode;
+                }
+            );
+        }
+
+        $scope.someSelectedFeaturesAreInMode = function (mode) {
+            return $scope.selection.selectedObjects.some(
+                (feature) => {
+                    return $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon_mode === mode;
+                }
+            );
+        }
+
+        $scope.setSelectedFeaturesToFixedShifts = function (){
+            $scope.selection.selectedObjects.forEach(
+                (feature) => {
+                    $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon_mode = 'FIXED';
+                }
+            )
+        };
+
+        $scope.getFeatureSupportingAutoShiftNumber = function () {
+            return $scope.selection.selectedObjects.filter(
+                (feature) => $scope.featureSupportsAutoShifts(feature)
+            ).length;
+        };
+
+        $scope.setSelectedFeaturesShifts = function() {
+            $scope.selection.selectedObjects.forEach(function(feature) {
+                if (!$scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name]) {
+                    $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name] = {};
+                }
+                $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_forecast = $scope.uiState.massActionForecastShifts || "";
+                if ($scope.massEditTargetShiftMode !== "AUTO") {
+                    $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon = $scope.uiState.massActionHorizonShifts || "";
+                }
+            });
+            $scope.validateShifts();
+            $scope.validateMassEditShifts();
+        };
+
+        $scope.validateMassEditShifts = function () {
+            $scope.uiState.massEditShiftsValidationErrors = {}
+
+            let shiftFromForecastAsString = ($scope.uiState.massActionForecastShifts || "").toString();
+            let shiftFromHorizonAsString = ($scope.uiState.massActionHorizonShifts || "").toString();
+            let hasPastOnlyFeature = $scope.someSelectedFeaturesArePastOnly();
+
+            $scope.uiState.massEditShiftsValidationErrors = $scope.validateShiftsAsString(shiftFromForecastAsString, shiftFromHorizonAsString, hasPastOnlyFeature);
+
+            $scope.updateSelectedAlgosMissingRequiredExternalFeature();
+        }
+
+        // =================
+
+        $scope.setFeatureAutoShiftsParams = function (maxSelectedHorizonShifts, minHorizonShiftPastOnly, maxHorizonShiftPastOnly, minHorizonShiftKnownInAdvance, maxHorizonShiftKnownInAdvance) {
+            if (!$scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params) {
+                $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params = {};
+            }
+
+            $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.max_selected_horizon_shifts = maxSelectedHorizonShifts || 1;
+            $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.min_horizon_shift_past_only = minHorizonShiftPastOnly;
+            $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.max_horizon_shift_past_only = maxHorizonShiftPastOnly;
+            $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.min_horizon_shift_known_in_advance = minHorizonShiftKnownInAdvance;
+            $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.max_horizon_shift_known_in_advance = maxHorizonShiftKnownInAdvance;
+        };
+
+        $scope.validateAutoShiftsParams = function () {
+            let autoShiftsParams = $scope.mlTaskDesign.preprocessing?.feature_generation?.auto_shifts_params;
+            $scope.hasAutoShiftsParamsValidationErrors = autoShiftsParams === undefined
+                || typeof autoShiftsParams.max_selected_horizon_shifts !== 'number' || autoShiftsParams.max_selected_horizon_shifts < 1
+                || $scope.validatePastOnlyAutoHorizonShiftRange(autoShiftsParams.min_horizon_shift_past_only, autoShiftsParams.max_horizon_shift_past_only) !== ""
+                || $scope.validateKnownInAdvanceAutoHorizonShiftRange(autoShiftsParams.min_horizon_shift_known_in_advance, autoShiftsParams.max_horizon_shift_known_in_advance) !== ""
+        };
+
+        $scope.openAutoShiftsModal = function () {
+            CreateModalFromTemplate("/templates/analysis/mlcommon/settings/time-series-edit-auto-shift-modal.html", $scope, null, function (newScope) {
+
+                if ($scope.mlTaskDesign.preprocessing.feature_generation?.auto_shifts_params) {
+                    newScope.maxSelectedHorizonShifts = $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.max_selected_horizon_shifts;
+                    newScope.minHorizonShiftPastOnly = $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.min_horizon_shift_past_only;
+                    newScope.maxHorizonShiftPastOnly = $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.max_horizon_shift_past_only;
+                    newScope.minHorizonShiftKnownInAdvance = $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.min_horizon_shift_known_in_advance;
+                    newScope.maxHorizonShiftKnownInAdvance = $scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params.max_horizon_shift_known_in_advance;
+                } else {
+                    newScope.maxSelectedHorizonShifts = 3 ;
+                    newScope.minHorizonShiftPastOnly = -$scope.mlTaskDesign.predictionLength - 35;
+                    newScope.maxHorizonShiftPastOnly = -$scope.mlTaskDesign.predictionLength;
+                    newScope.minHorizonShiftKnownInAdvance = -35;
+                    newScope.maxHorizonShiftKnownInAdvance = 0;
+                }
+
+                newScope.inputPastOnlyHorizonShiftsRangeValidationError = newScope.validatePastOnlyAutoHorizonShiftRange(newScope.minHorizonShiftPastOnly, newScope.maxHorizonShiftPastOnly);
+                newScope.inputHorizonShiftsRangeValidationError = newScope.validateKnownInAdvanceAutoHorizonShiftRange(newScope.minHorizonShiftKnownInAdvance, newScope.maxHorizonShiftKnownInAdvance);
+
+                newScope.save = () => {
+                    $scope.setFeatureAutoShiftsParams(newScope.maxSelectedHorizonShifts, newScope.minHorizonShiftPastOnly, newScope.maxHorizonShiftPastOnly, newScope.minHorizonShiftKnownInAdvance, newScope.maxHorizonShiftKnownInAdvance);
+                    $scope.validateAutoShiftsParams();
+                    newScope.dismiss();
+                }
+            }, false, false, true)
+        };
+
+        $scope.$on('$destroy', function() {
+            // clean settings when leaving the tab
+            TimeseriesFeatureGenerationService.cleanUpSettings($scope.mlTaskDesign);
+         })
+    });
+
+app.controller('FeatureGenerationWindowController', function($scope, CreateModalFromTemplate, TimeseriesFeatureGenerationService, TimeseriesForecastingUtils) {
+
+        // ================= Mass operations
+        $scope._isMassEditMode = false; // used by TimeseriesFeatureGenerationService
+
+        $scope.initMassEditTargetOperations = function () {
+            $scope.massEditTargetOperations = {'numerical': [], 'categorical': []};
+        }
+
+        $scope.initMassEditTargetOperations();
+
+        let redrawFn = function() {
+            $scope.shouldRerender = true;
+        }
+
+        $(window).on('resize', redrawFn);
+
+        $scope.$on('$destroy', function() {
+            $(window).off('resize', redrawFn);
+        });
+
+        $scope.$watch('selection', function() {
+            if ($scope.selection.selectedObjects && $scope.selection.selectedObjects.length === 0) {
+                // if no more objects are selected, reset the mass edit state
+                $scope.initMassEditTargetOperations();
+            }
+            $scope.getMassEditOperations();
+        }, true);
+
+        $scope.isOperationTargetSet = function (operationName, operationType) {
+            // whether an operation is present in the massEditTargetOperations
+            return $scope.massEditTargetOperations[operationType].some(
+                operation => operation.operation === operationName
+            )
+        };
+
+        $scope.getOperationTarget = function (operationName, operationType) {
+            if ($scope.isOperationTargetSet(operationName, operationType)) {
+                return $scope.massEditTargetOperations[operationType].filter(
+                    operation => operation.operation === operationName
+                )[0];
+            }
+        };
+
+        $scope.isMassEditMode = function(selection) {
+            return TimeseriesFeatureGenerationService.isMassEditMode($scope, selection);
+        };
+
+        $scope.getTableSize = function(numberOfElements, rowHeight) {
+            return TimeseriesFeatureGenerationService.getTableSize(numberOfElements, rowHeight);
+        }
+
+        $scope.addToTargetOperations = function(operationName, enabled, operationType) {
+            if (!$scope.isOperationTargetSet(operationName, operationType)) {
+                $scope.massEditTargetOperations[operationType].push({operation: operationName, enabled: enabled})
+            } else {
+                $scope.getOperationTarget(operationName, operationType).enabled = enabled;
+            }
+
+        };
+        // =================
+
+        $scope.setWindowOperations = function(numOperations, catOperations, windowIndex) {
+            let selectedAndValidFeatures = $scope.uiState.windows[windowIndex].operations_list
+                .filter(el => {
+                    return el.$selected && $scope.uiState.windowableFeatures.map(feature => feature._name).includes(el[0]);
+                })
+            for (let el of selectedAndValidFeatures) {
+                if (el[2].type === "NUMERIC") {
+                    el[1] = angular.copy(numOperations);
+                } else if (el[2].type === "CATEGORY") {
+                    el[1] = angular.copy(catOperations);
+                }
+            }
+            $scope.validateWindows();
+        }
+
+        $scope.getMassEditOperations = function () {
+            let per_feature = $scope.mlTaskDesign.preprocessing.per_feature;
+            let selectedFeaturesNumerical = $scope.window.operations_list.filter(x => per_feature[x[0]].type === "NUMERIC" && x.$selected && TimeseriesForecastingUtils.isWindowCompatible(per_feature[x[0]]));
+            if (selectedFeaturesNumerical.length === 0) {
+                $scope.massEditTargetOperations['numerical'] = [];
+            }
+            let someDisabledOperationsNumerical = new Set(selectedFeaturesNumerical.map(x => x[1]).flatMap(x => x.filter(y => !y.enabled).map(y => y.operation)));
+            let allEnabledOperationsNumerical = (new Set(["MEAN", "MEDIAN", "STD", "MIN", "MAX"])).difference(someDisabledOperationsNumerical);
+            let partiallyEnabledOperationsNumerical = (new Set(selectedFeaturesNumerical.map(x => x[1]).flatMap(x => x.filter(y => y.enabled).map(y => y.operation)))).difference(allEnabledOperationsNumerical);
+            $scope.massEditOperationsNumerical = [
+                {operation: "MEAN", enabled: $scope.isOperationTargetSet("MEAN", 'numerical') ? $scope.getOperationTarget("MEAN", "numerical").enabled : allEnabledOperationsNumerical.has("MEAN"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("MEAN")},
+                {operation: "MEDIAN", enabled: $scope.isOperationTargetSet("MEDIAN", 'numerical') ? $scope.getOperationTarget("MEDIAN", "numerical").enabled : allEnabledOperationsNumerical.has("MEDIAN"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("MEDIAN")},
+                {operation: "STD", enabled: $scope.isOperationTargetSet("STD", 'numerical') ? $scope.getOperationTarget("STD", "numerical").enabled : allEnabledOperationsNumerical.has("STD"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("STD")},
+                {operation: "MIN", enabled: $scope.isOperationTargetSet("MIN", 'numerical') ? $scope.getOperationTarget("MIN", "numerical").enabled : allEnabledOperationsNumerical.has("MIN"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("MIN")},
+                {operation: "MAX", enabled: $scope.isOperationTargetSet("MAX", 'numerical') ? $scope.getOperationTarget("MAX", "numerical").enabled : allEnabledOperationsNumerical.has("MAX"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("MAX")}
+            ];
+
+            let selectedFeaturesCategorical = $scope.window.operations_list.filter(x => per_feature[x[0]].type === "CATEGORY" && x.$selected && TimeseriesForecastingUtils.isWindowCompatible(per_feature[x[0]]));
+            if (selectedFeaturesCategorical.length === 0) {
+                $scope.massEditTargetOperations['categorical'] = [];
+            }
+            let someDisabledOperationsCategorical = new Set(selectedFeaturesCategorical.map(x => x[1]).flatMap(x => x.filter(y => !y.enabled).map(y => y.operation)));
+            let allEnabledOperationsCategorical = (new Set(["FREQUENCY"])).difference(someDisabledOperationsCategorical);
+            let partiallyEnabledOperationsCategorical = (new Set(selectedFeaturesCategorical.map(x => x[1]).flatMap(x => x.filter(y => y.enabled).map(y => y.operation)))).difference(allEnabledOperationsCategorical);
+            $scope.massEditOperationsCategorical = [
+                {operation: "FREQUENCY", enabled: $scope.isOperationTargetSet("FREQUENCY", 'categorical') ? $scope.getOperationTarget("FREQUENCY", "categorical").enabled : allEnabledOperationsCategorical.has("FREQUENCY"), partiallyEnabled: partiallyEnabledOperationsCategorical.has("FREQUENCY")},
+            ];
+
+            $scope.numericalSelectedFeaturesNb = selectedFeaturesNumerical.length;
+
+            $scope.categoricalSelectedFeaturesNb = selectedFeaturesCategorical.length;
+        }
+
+        $scope.setMassEditOperations = function () {
+            for (let targetOperation of $scope.massEditTargetOperations['numerical'].concat($scope.massEditTargetOperations['categorical'])) {
+                let selectedFeature = $scope.selection.selectedObjects.map(feature => feature[0]);
+                $scope.window.operations_list.forEach(function (feature) {
+                    if (selectedFeature.includes(feature[0])) {
+                        for (let operation of feature[1]) {
+                            if (operation.operation === targetOperation.operation) {
+                                operation.enabled = targetOperation.enabled;
+                                break;
+                            }
+                        }
+                    }
+                })
+            }
+
+            $scope.getMassEditOperations();
+        }
+
+        $scope.hasAnyEnabledOperation = function(window) {
+            return window.operations_list.some(
+                operations => operations[1].some(
+                    operation => operation.enabled
+                )
+            );
+        }
+    });
+
 app.component("cyclicalEncodingWarning", {
     bindings: {
         datetimeCyclicalPeriods: '<',
@@ -1153,7 +1731,40 @@ app.component("cyclicalEncodingWarning", {
                     + " periods (" + irrelevantSelectedPeriods.map(period => period.toLowerCase()).join(", ")
                     + ") might not be relevant for the feature.";
             }
-        };
+        }
     }
 });
+
+app.service("MLFeaturesService", function($q, $stateParams, DataikuAPI, DatasetUtils, Fn) {
+    const service = {};
+    let datasetColumnsPromise = null;
+    let analysisId = null;
+    let datasetName = null;
+
+    service.getDatasetColumns = function(analysisCoreParams) {
+        if (!analysisCoreParams || !analysisCoreParams.inputDatasetSmartName) {
+            return $q.reject("analysisCoreParams are not available");
+        }
+        if (datasetColumnsPromise && analysisId && analysisId === analysisCoreParams.id && datasetName && datasetName === analysisCoreParams.inputDatasetSmartName) {
+            return datasetColumnsPromise;
+        }
+        const datasetLoc = DatasetUtils.getLocFromSmart($stateParams.projectKey, analysisCoreParams.inputDatasetSmartName);
+        datasetColumnsPromise = DataikuAPI.datasets.get(datasetLoc.projectKey, datasetLoc.name, $stateParams.projectKey)
+            .then(function(response) {
+                return response.data.schema.columns.map(Fn.prop('name'));
+            });
+        analysisId = analysisCoreParams.id;
+        datasetName = analysisCoreParams.inputDatasetSmartName;
+        return datasetColumnsPromise;
+    };
+
+    service.addDataSetColumnId = function(features, datasetColumns) {
+        if (!features) return;
+        angular.forEach(features, function(feature) {
+            feature.datasetColumnId = datasetColumns.indexOf(feature._name);
+        });
+    };
+
+    return service;
+});
 })();
