@@ -194,12 +194,41 @@ app.controller("exportModelController", function($scope, MLExportService) {
  * Controller for displaying results screen of a prediction model
  * in a PMLTask
  */
-app.controller("PMLModelReportController", function($scope, $controller, TopNav, WebAppsService, GoToStateNameSuffixIfBase, MLModelsUIRouterStates, $state) {
+app.controller("PMLModelReportController", function($scope, $controller, TopNav, WebAppsService, GoToStateNameSuffixIfBase, MLModelsUIRouterStates, $state, TimeseriesForecastingUtils, ExportUtils) {
     TopNav.setLocation(TopNav.TOP_ANALYSES, TopNav.LEFT_ANALYSES, "PREDICTION-ANALYSIS-MODEL", "report");
 
     $controller("_PMLModelBaseController",{$scope:$scope});
     $controller("_PredictionModelReportController",{$scope:$scope});
 
+    $scope.hasTextFeature = function() {
+        return TimeseriesForecastingUtils.hasExternalFeatureOfType($scope.modelData, 'TEXT');
+    };
+    $scope.isPermutationImportanceSupported = function() {
+        return TimeseriesForecastingUtils.isPermutationImportanceSupported($scope.modelData);
+    };
+
+    $scope.onPermutationImportanceExportStateChange = function(permutationImportanceExportStateChangeEvent) {
+        if (!permutationImportanceExportStateChangeEvent?.exporter) {
+            return;
+        }
+        $scope.canExportPermutationImportance = permutationImportanceExportStateChangeEvent.exporter.canExport();
+        $scope.permutationImportanceExporter = permutationImportanceExportStateChangeEvent.exporter;
+    };
+
+    $scope.exportPermutationImportance = function() {
+
+        const exportData = $scope?.permutationImportanceExporter?.getExportData();
+        if (!exportData) {
+            return;
+        }
+
+        ExportUtils.exportUIData($scope, {
+            name: exportData.exportLabel + " of model " + $scope.modelData.userMeta.name,
+            columns: exportData.columns,
+            data: exportData.data
+        }, `Export ${exportData.exportLabel}`);
+    };
+
 
     const baseStateName = "projects.project.analyses.analysis.ml.predmltask.model.report";
     $scope.$on('$stateChangeStart', function (event, toState, toParams, fromState, fromParams) {
@@ -387,7 +416,7 @@ app.controller("AnalysisPredictionDeployController", function($scope, $controlle
             ).success(function(data){
                 WT1.event("analysis-pml-deploy-model-to-the-flow", { from: $scope.deploymentOrigin })
                 $scope.dismiss();
-                $state.go("projects.project.flow");
+                $state.go("projects.project.flow", { id: "savedmodel_" + $stateParams.projectKey + "." + data.second });
             }).error(setErrorInScope.bind($scope));
         };
     };
@@ -411,8 +440,9 @@ app.controller("AnalysisPredictionDeployController", function($scope, $controlle
                 $scope.formData.redeployTrainActivate,
                 options
             ).success(function(data){
-                WT1.event("analysis-pml-redeploy-model-to-the-flow", { from: $scope.deploymentOrigin })
-                var go = $state.go.bind($state, "projects.project.flow");
+                const updatedSavedModelId = $scope.redeployables.find(r => r.recipeName === $scope.formData.redeployTrainRecipeName).savedModel.id
+                WT1.event("analysis-pml-redeploy-model-to-the-flow", { from: $scope.deploymentOrigin });
+                var go = $state.go.bind($state, "projects.project.flow", { id: "savedmodel_" + $stateParams.projectKey + "." + updatedSavedModelId });
                 var parentScope = $scope.$parent;
                 $scope.dismiss();
                 if (data.schemaChanged) {
