@@ -4,13 +4,57 @@
 const app = angular.module('dataiku.collab.discussions', []);
 
 
-app.controller('DiscussionsWidgetController', function($scope, $state, $location, $stateParams, $rootScope, $timeout, $filter, Assert, DataikuAPI, WT1, TopNav, Notification, Dialogs, ActivityIndicator, Debounce) {
+app.controller('DiscussionsWidgetController', function($scope, $state, $location, $stateParams, $rootScope, $timeout, $filter, Assert, DataikuAPI, WT1, TopNav, Notification, Dialogs, ActivityIndicator, Debounce, translate) {
     let currentItem;
 
     $scope.appConfig = $rootScope.appConfig;
     $scope.uiState = {};
     $scope.discussionsContext = $stateParams.workspaceKey ? 'workspace' : 'project';
 
+    $scope.topicInputPlaceholder = translate('WIDGET.DISCUSSION.TOPIC.PLACEHOLDER', 'Topic');
+
+    $scope.getReadinessUserPictureTitle = function(convLogin ) {
+        return translate('WIDGET.DISCUSSION.READINESS.USER_PICTURE.TITLE', '{{ convLogin }} read the discussion up to this message', { convLogin });
+    };
+
+    $scope.getMessageEditButtonTitle = function () {
+        return (
+            !$scope.uiState.replyEditedText.length
+            ? translate('WIDGET.DISCUSSION.MESSAGE.EDIT.DISABLED_REASON.NOT_EMPTY', 'The message cannot be empty!')
+            : (
+                $scope.uiState.replyEditedText.length > 10000 
+                ? translate('WIDGET.DISCUSSION.MESSAGE.EDIT.DISABLED_REASON.10000_CHARACTERS', 'The message cannot be longer than 10000 characters!')
+                : ''
+            ) 
+        );
+    }
+
+    $scope.getDiscussionNewReplyPlaceholder = function () {
+        return (
+            $scope.uiState.selectedConv
+            ? translate('WIDGET.DISCUSSION.REPLY.PLACEHOLDER', 'Reply discussion')
+            : translate('WIDGET.DISCUSSION.MESSAGE.PLACEHOLDER', 'Message')
+        );
+    };
+
+    $scope.getEditTopicSaveButtonTitle = function () {
+        return (
+            !$scope.uiState.newConvTopic.length
+            ? translate('WIDGET.DISCUSSION.TOPIC.EPIC.DISABLED_REASON.NOT_EMPTY', 'The topic cannot be empty!')
+            : (
+                $scope.uiState.newConvTopic.length > 200
+                ? translate('WIDGET.DISCUSSION.TOPIC.EPIC.DISABLED_REASON.200_CHARACTERS', 'The topic cannot be longer than 200 characters!')
+                : ''
+            )
+        )
+    }
+
+    $scope.getDiscussionReplyEditedPlaceholder = function () {
+        return translate('WIDGET.DISCUSSION.REPLY.PLACEHOLDER', 'Reply discussion');
+    };
+
+    $scope.getDiscussionTopic = () => $scope.uiState.selectedConv.topic || translate('WIDGET.DISCUSSION.TOPIC.UNNAMED', 'Unnamed discussion');
+
     /**** UI ****/
     $scope.discussionWidgetPage = function() {
         if (!$scope.discussions) {
@@ -33,7 +77,33 @@ app.controller('DiscussionsWidgetController', function($scope, $state, $location
             return;
         }
         const newRepliesCount = $scope.getNumberOfNewReplies($scope.uiState.selectedConv, $scope.uiState.cachedUserReadTime || 0);
-        return newRepliesCount + ' new repl' + (newRepliesCount > 1 ? 'ies' : 'y');
+        return (
+            translate(
+                (
+                    newRepliesCount <= 1
+                    ? 'WIDGET.DISCUSSION.NEW_REPLY.LABEL'
+                    : 'WIDGET.DISCUSSION.NEW_REPLIES.LABEL'
+                ),
+                newRepliesCount + ' new repl' + (newRepliesCount > 1 ? 'ies' : 'y'),
+            )
+        )
+    };
+
+    $scope.getViewNewRepliesLabel = function() {
+        if (!$scope.uiState.selectedConv) {
+            return;
+        }
+        const newRepliesCount = $scope.getNumberOfNewReplies($scope.uiState.selectedConv, $scope.uiState.cachedUserReadTime || 0);
+        return (
+            translate(
+                (
+                    newRepliesCount <= 1
+                    ? 'WIDGET.DISCUSSION.VIEW_NEW_REPLY.LABEL'
+                    : 'WIDGET.DISCUSSION.VIEW_NEW_REPLIES.LABEL'
+                ),
+                newRepliesCount + 'View new repl' + (newRepliesCount > 1 ? 'ies' : 'y'),
+            )
+        )
     };
 
     $scope.getNumberOfNewReplies = function(discussion, fromTime) {
@@ -45,18 +115,18 @@ app.controller('DiscussionsWidgetController', function($scope, $state, $location
 
     $scope.getDisabledReason = function() {
         if (!$scope.uiState.selectedConv && !($scope.uiState.newConvTopic || '').length) {
-            return 'The topic cannot be empty!';
+            return translate('WIDGET.DISCUSSION.TOPIC.EPIC.DISABLED_REASON.NOT_EMPTY', 'The topic cannot be empty!');
         }
         if (!$scope.uiState.selectedConv && ($scope.uiState.newConvTopic || '').length > 200) {
-            return 'The topic cannot be longer than 200 characters!';
+            return translate('WIDGET.DISCUSSION.TOPIC.EPIC.DISABLED_REASON.200_CHARACTERS', 'The topic cannot be longer than 200 characters!');
         }
         if (!($scope.uiState.newReply || '').length) {
-            return 'The message cannot be empty!';
+            return translate('WIDGET.DISCUSSION.MESSAGE.EDIT.DISABLED_REASON.NOT_EMPTY', 'The message cannot be empty!');
         }
         if (($scope.uiState.newReply || '').length > 10000) {
-            return 'The message cannot be longer than 10000 characters!';
+            return translate('WIDGET.DISCUSSION.MESSAGE.EDIT.DISABLED_REASON.10000_CHARACTERS', 'The message cannot be longer than 10000 characters!');
         }
-        return 'Press Enter to insert a new line. And press Ctrl+Enter to send your message.';
+        return translate('WIDGET.DISCUSSION.MESSAGE.EDIT.DISABLED_REASON.ENTER_NEW_LINE', 'Press Enter to insert a new line. And press Ctrl+Enter to send your message.');
     };
 
     $scope.viewNewReplies = function(scrollToNewReplies) {
@@ -127,7 +197,7 @@ app.controller('DiscussionsWidgetController', function($scope, $state, $location
         }
         participants.sort((a, b) => b.lastReplyTime - a.lastReplyTime);
         for (let i = 0; i < participants.length; i++) {
-            const escapedDisplayName = $filter('escapeHtml')(participants[i].displayName || 'Unknown user');
+            const escapedDisplayName = $filter('escapeHtml')(participants[i].displayName || translate('WIDGET.DISCUSSION.PARTICIPANTS.UNKNOWN_USER', 'Unknown user'));
             if (participantListLength + escapedDisplayName.length + 2 <= MAX_PARTICIPANT_LIST_LENGTH) {
                 displayedParticipants.push(escapedDisplayName);
                 participantListLength += escapedDisplayName.length + 2;
@@ -365,13 +435,18 @@ app.controller('DiscussionsWidgetController', function($scope, $state, $location
 });
 
 
-app.directive('discussionsWidget', function() {
+app.directive('discussionsWidget', function(translate) {
     return {
         restrict: 'AE',
         templateUrl: '/templates/widgets/discussions-widget-content.html',
         scope: {
             selectedItem: '&',
             watchObject: '='
+        },
+        link: function($scope) {
+            $scope.discussionSearchPlaceholder = translate('WIDGET.DISCUSSION.SEARCH.PLACEHOLDER', 'Search discussions…');
+            $scope.showResolvedDiscussionTooltip = translate('WIDGET.DISCUSSION.SHOW_RESOLVED', 'Show resolved discussions');
+            $scope.hideResolvedDiscussionTooltip = translate('WIDGET.DISCUSSION.HIDE_RESOLVED', 'Hide resolved discussions');
         }
     };
 });
@@ -390,7 +465,7 @@ app.directive('discussionsWidgetSingle', function() {
 });
 
 
-app.directive('discussionsButton', function($rootScope, $compile, $q, $templateCache, $http, $state, $stateParams, Assert, DataikuAPI, TopNav, Notification, Debounce) {
+app.directive('discussionsButton', function($rootScope, $compile, $templateRequest, $stateParams, Assert, DataikuAPI, TopNav, Notification, Debounce) {
     return {
         restrict: 'AE',
         templateUrl: '/templates/widgets/discussions-widget-button.html',
@@ -479,7 +554,7 @@ app.directive('discussionsButton', function($rootScope, $compile, $q, $templateC
                 Assert.trueish(!$scope.uiState.loadRequested, 'loadWidget called twice');
                 $scope.uiState.loadRequested = true;
                 const location = '/templates/widgets/discussions-widget-popover.html'
-                $q.when($templateCache.get(location) || $http.get(location, {cache: true}))
+                $templateRequest(location)
                     .then(function(template) {
                         // we need to make sure the item didn't change or the scope didn't get destroyed
                         // between the API call and when we load the widget (or we will display it on the wrong page)
