@@ -6,24 +6,29 @@
     /**
      * Was previously located here: src/main/platypus/static/dataiku/js/dashboards/core.js
      */
-    app.controller('DashboardCoreController', function($scope, $element, $rootScope, $timeout, $stateParams, DataikuAPI, TopNav, $filter, $state, $controller, CreateModalFromTemplate, WT1, DashboardUtils, DashboardFilters, DSSVisualizationThemeUtils) {
+    app.controller('DashboardCoreController', function($scope, $element, $rootScope, $timeout, $stateParams, DataikuAPI, TopNav, $filter, $state, $controller, CreateModalFromTemplate, WT1, DashboardUtils, DashboardFilters, DSSVisualizationThemeUtils, translate, DashboardTileSettingsService, DashboardService, TileUtils, DashboardFixupService) {
         $controller('DashboardsCommonController', { $scope: $scope });
         $scope.uiState = { tooltipReload: 'Reload tiles', displayNotification: false };
 
-        DataikuAPI.dashboards.getSummary($stateParams.projectKey, $stateParams.dashboardId).success(function(data) {
+        DataikuAPI.dashboards.getSummary($stateParams.projectKey, $stateParams.dashboardId, $stateParams.exportId).success(function(data) {
             $scope.ownerDisplayName = data.ownerDisplayName;
 
             $scope.dashboard = data.object;
-            $scope.dashboard.pages.forEach(page => {
-                /*
-                 * client id or cid is a unique identifier automatically assigned
-                 * when they're first created.
-                 * Client ids are handy when the model has not yet been saved to the server,
-                 * and does not yet have its eventual true id, but already needs to be visible in the UI.
-                 */
-                page.cid = _.uniqueId();
+            $scope.dashboard._tileSpacing = $scope.dashboard.tileSpacing;
+            Object.defineProperty($scope.dashboard, 'tileSpacing', {
+                get: function() {
+                    return this._tileSpacing;
+                },
+                set: function(newTileSpacing) {
+                    DashboardTileSettingsService.updateTileSpacing({
+                        dashboard: $scope.dashboard,
+                        newTileSpacing
+                    });
+                }
             });
 
+            DashboardFixupService.fixupDashboard($scope.dashboard);
+
             $scope.interest = data.interest;
             $scope.timeline = data.timeline;
             $scope.canEdit = $scope.canEditDashboard($scope.dashboard);
@@ -57,21 +62,21 @@
             $scope.dashboard = angular.copy($scope.origDashboard);
         };
 
-        $scope.toggleDashboardListed = function() {
-            if (!$scope.dashboard.listed) {
-                DataikuAPI.dashboards.getMissingReaderAuthorizations($scope.dashboard.projectKey, [$scope.dashboard.id]).success(function(data) {
+        $scope.toggleDashboardListed = function(dashboard) {
+            if (dashboard.listed) {
+                DataikuAPI.dashboards.getMissingReaderAuthorizations(dashboard.projectKey, [dashboard.id]).success(function(data) {
                     if (data.length) {
                         CreateModalFromTemplate('/templates/dashboards/insights/insight-access-warning-modal.html', $scope, null, function(newScope) {
-                            newScope.initForDashboardsWithAuths([$scope.dashboard], true, data);
+                            newScope.initForDashboardsWithAuths([dashboard], true, data);
                         }).then(function() {
-                            $scope.makeDashboardListed($scope.dashboard);
+                            $scope.makeDashboardListed(dashboard);
                         });
                     } else {
-                        $scope.makeDashboardListed($scope.dashboard);
+                        $scope.makeDashboardListed(dashboard);
                     }
                 });
             } else {
-                $scope.makeDashboardListed($scope.dashboard);
+                $scope.makeDashboardListed(dashboard);
             }
         };
 
@@ -131,9 +136,11 @@
 
             if (currentFilters && previousFilters) {
                 if (currentFilters.length === previousFilters.length) {
-                    $scope.filterChangedNotificationLabel = 'Filter updated';
+                    $scope.filterChangedNotificationLabel = translate('DASHBOARD.FILTER.PANEL.FILTERS.FILTER_UPDATED', 'Filter updated');
                 } else if (currentFilters.length > previousFilters.length) {
-                    $scope.filterChangedNotificationLabel = 'Filter added';
+                    $scope.filterChangedNotificationLabel = translate('DASHBOARD.FILTER.PANEL.FILTERS.FILTER_ADDED', 'Filter added');
+                } else {
+                    $scope.filterChangedNotificationLabel = translate('DASHBOARD.FILTER.PANEL.FILTERS.FILTER_REMOVED', 'Filter removed');
                 }
 
                 crossFilterNotificationTimeoutId = $timeout(() => {
@@ -154,6 +161,7 @@
         $scope.$on('$destroy', () => {
             unregisterDashboardFiltersChangedListener();
             DSSVisualizationThemeUtils.hideThemeAppliedSnackbar();
+            DashboardFilters.resetAllDashboardPagesFilterPanelStates();
         });
 
         function getCurrentPage() {
@@ -192,6 +200,13 @@
             DashboardFilters.resetAllDashboardPagesFilterPanelStates();
             const pageId = getVisiblePageId(viewMode);
             $scope.uiState.currentPageIdx = getVisiblePageIndex(pageId);
+            DashboardService.updateViewMode({
+                dashboard: {
+                    id: $scope.dashboard.id,
+                    viewMode: $state.current.name.split('.').pop()
+                },
+                newViewMode: viewMode
+            });
             $state.go(`projects.project.dashboards.dashboard.${ viewMode }`, { pageId });
         }
 
