@@ -6,6 +6,8 @@
     app.constant('SavedModelReportInsightHandler', {
         name: 'Saved model report',
         desc: 'Full report of a model',
+        i18nNameKey: 'INSIGHTS.SAVED_MODEL_REPORT.NAME',
+        i18nDescKey: 'INSIGHTS.SAVED_MODEL_REPORT.DESC',
         icon: 'icon-dku-modelize',
         color: 'saved-model',
 
@@ -17,7 +19,10 @@
         defaultTileParams: {
             displayMode: 'summary'
         },
-        defaultTileDimensions: [24, 12]
+        defaultTileDimensions: [24, 12],
+        filterSavedModel: function(taggableSavedModel) {
+            return taggableSavedModel.subtype != 'LLM_GENERIC_RAW';
+        }
 
     });
 
@@ -45,6 +50,7 @@
             const p = DataikuAPI.savedmodels.get($scope.insight.projectKey, $scope.insight.params.savedModelSmartId)
                 .success(function(data) {
                     $scope.insight.$savedModel = data;
+                    if (!$scope.insight.$savedModel.miniTask) return;
                     const version = $scope.insight.params.version || data.activeVersion;
 
                     $scope.insight.$fullModelId = FullModelLikeIdUtils.buildSavedModelFmi({
@@ -505,17 +511,21 @@
     });
 
 
-    app.directive('savedModelReportInsightCreateForm', function(DataikuAPI){
+    app.directive('savedModelReportInsightCreateForm', function(translate) {
         return {
             templateUrl: '/templates/dashboards/insights/saved-model_report/saved-model_report_create_form.html',
             scope: true,
             link: function($scope, element, attrs) {
-                $scope.hook.defaultName = 'Saved model';
+                $scope.hook.defaultName = translate('DASHBOARD.NEW_TILE_MODAL.SAVED_MODEL', 'Saved model');
                 $scope.$watch('hook.sourceObject', function(nv) {
                     if (!nv || !nv.label) {
                         return;
                     }
-                    $scope.hook.defaultName = nv.label + ' table';
+                    $scope.hook.defaultName = translate(
+                        'DASHBOARD.NEW_TILE_MODAL.MODEL_EVALUATION_TABLE',
+                        nv.label + ' table',
+                        { label: nv.label }
+                    );
                 });
 
                 /*
