@@ -41,6 +41,19 @@
                         return dataSpec;
                     }
 
+                    $scope.getLLMChartContext = function() {
+                        return {
+                            context: "shaker",
+                            datasetProjectKey: projectKey,
+                            spec: $scope.getDataSpec(),
+                            datasetName: datasetName,
+                            getSuggestionsCacheKey(chartDefCacheKey) {
+                                return chartDefCacheKey ? projectKey + datasetName + chartDefCacheKey : projectKey + datasetName;
+                            }
+
+                        };
+                    };
+
                     $scope.getExecutePromise = function (request, saveShaker = true, noSpinner = false, requiredSampleId = undefined, dataSpec = $scope.getDataSpec()) {
                         var currentChart = $scope.charts[$scope.currentChart.index];
                         Assert.trueish(currentChart.summary, "Current chart summary is not ready");
