@@ -89,6 +89,10 @@
             this.$newElement.data('this', this);
 
             $('[data-toggle="tooltip"]').tooltip();
+
+            // 1) Make sure to destroy the select picker when the element is removed in order to avoid memory leaks
+            // 2) Use setTimeout() to prevent stack overflow if we are already inside destroy()
+            this.$element.on("remove", () => setTimeout(() => this.destroy()));
         },
 
         createDropdown: function() {
@@ -415,6 +419,9 @@
 
             if (this.options.size == 'auto') {
                 var getSize = function() {
+                    if (that.$lis === null) {
+                        that.$lis = menu.find('li');
+                    }
                     var minHeight,
                         lisVis = that.$lis // all <li>
                             .not('.hide') // remove the ones having class .hide
@@ -783,10 +790,18 @@
                 this.removeNoResultsElements();
                 if (this.$searchbox.val()) {
                     const searchValLower = this.$searchbox.val().toLowerCase();
-                    this.$lis.not('.is-hidden').removeClass('hide').each(function() {
+                    let currentGroupHeader = undefined;
+                    this.$lis.not('.is-hidden').removeClass('hide show-bs-select-group-header').each(function() {
+                        if ($(this).find('dt').length) { // track the header for the current group
+                            currentGroupHeader = this;
+                        }
+
                         const textToSearch = $(this).text() + ' ' + $(this).data('tokens') // We search both the displayed text and the data-tokens attribute
                         if (!textToSearch.toLowerCase().includes(searchValLower)) {
                             $(this).addClass('hide');
+                        } else {
+                            // when an item matches, we need to prevent the group header to be hidden
+                            $(currentGroupHeader)?.addClass('show-bs-select-group-header');
                         }
                     });
                     if (this.$menu.find('li').filter(':visible:not(.no-results)').length === 0) {
