@@ -39,7 +39,7 @@ function friendlyDuration(diffInSeconds, translate) {
     const values = { years: years, months: months, days: days, hours: hrs, minutes: min, seconds: sec };
     if (years > 0) {
         if (years <= 6 && months > 0) {
-            return translate("FILTER.FRIENDLY_DURATION.YEARS_MONTHS", "{{years}} {{years === 1 ? 'year': 'years'}} and {{months}} {{months === 1 ? 'one month': (months + ' months')}}", values);
+            return translate("FILTER.FRIENDLY_DURATION.YEARS_MONTHS", "{{years}} {{years === 1 ? 'year': 'years'}} and {{months === 1 ? 'one month': (months + ' months')}}", values);
         } else {
             return translate("FILTER.FRIENDLY_DURATION.YEARS", "{{years}} {{years === 1 ? 'year': 'years'}}", values);
         }
@@ -156,6 +156,48 @@ function friendlyDurationShort(seconds, ref, noSeconds, translate) {
     }
 }
 
+// This filter is meant to represent durations in a very concise manner
+// for places that lack space with shorthands (s instead of seconds for instance)
+function friendlyDurationVeryShort(seconds, ref, noSeconds, translate) {
+    const sec = Math.floor(seconds >= 60 ? seconds % 60 : seconds);
+    const min = Math.floor((seconds = (seconds / 60)) >= 60 ? seconds % 60 : seconds);
+    const hours = Math.floor((seconds = (seconds / 60)) >= 24 ? seconds % 24 : seconds);
+    const days = Math.floor((seconds = (seconds / 24)) >= 30 ? seconds % 30 : seconds);
+    const months = Math.floor((seconds = (seconds / 30)) >= 12 ? seconds % 12 : seconds);
+    const years = Math.floor((seconds = (seconds / 12)));
+    const values = { years: years, months: months, days: days, hours: hours, minutes: min, seconds: sec };
+    let sb = "";
+
+    if (years > 0) {
+        sb = translate("FILTER.FRIENDLY_DURATION.YEARS", "{{years}} {{years === 1 ? 'year': 'years'}}", values);
+    } else if (months > 0) {
+        sb = translate("FILTER.FRIENDLY_DURATION.MONTHS", "{{months}} {{months === 1 ? 'month': 'months'}}", values);
+    } else if (days > 0) {
+        sb = translate("FILTER.FRIENDLY_DURATION.DAYS", "{{days}} {{days === 1 ? 'day': 'days'}}", values);
+    } else if (hours > 0) {
+        sb = translate("FILTER.FRIENDLY_DURATION.SHORT_HOURS", "{{hours}}h", values);
+    } else if (min > 0) {
+        sb = translate("FILTER.FRIENDLY_DURATION.SHORT_MINUTES", "{{minutes}}min", values);
+    } else if(!noSeconds && sec > 0) {
+        sb = translate("FILTER.FRIENDLY_DURATION.SHORT_SECONDS", "{{seconds}}s", values);
+    }
+
+    switch (ref) {
+        case 'ago':
+            return sb
+                ? translate("FILTER.FRIENDLY_DURATION.DURATION_AGO", "{{duration}} ago", { duration: sb })
+                : translate("FILTER.FRIENDLY_DURATION.JUST_NOW", "just now");
+        case 'in':
+            return sb
+                ? translate("FILTER.FRIENDLY_DURATION.IN_DURATION", "{{duration}}", { duration: sb })
+                : translate("FILTER.FRIENDLY_DURATION.IMMEDIATELY", "immediately");
+        default:
+            return sb ? sb : (noSeconds
+                ? translate("FILTER.FRIENDLY_DURATION.LESS_THAN_ONE_MIN_SHORT", "<1min")
+                : translate("FILTER.FRIENDLY_DURATION.LESS_THAN_ONE_SEC_SHORT", "<1s"));
+    }
+}
+
 function dateDayDiff(date1, date2) {
     var d1 = new Date(date1);
     var d2 = new Date(date2);
@@ -504,6 +546,12 @@ app.filter('friendlyDurationShort', function(translate) {
     };
 });
 
+app.filter('friendlyDurationVeryShort', function(translate) {
+    return function(input, ref, noSeconds) {
+        return friendlyDurationVeryShort(parseInt(input, 10) / 1000, ref, noSeconds, translate);
+    };
+});
+
 
 app.filter('friendlyDurationSec', function(translate) {
     return function(input) {
@@ -651,6 +699,9 @@ app.filter('utcDate', function() {
 
 app.filter('typeToName', function($filter) {
     return function(input) {
+        if (!input) {
+            return '';
+        }
         return $filter('capitalizeWord')($filter('niceConst')(input));
     }
 });
@@ -750,7 +801,9 @@ app.filter('typeToColor', function($filter) {
         "scenario",
         "saved_model",
         "model_evaluation_store",
+        "genai_evaluation_store",
         "model_comparison",
+        "genai_comparison",
         "managed_folder",
         "web_app",
         "code_studio",
@@ -762,7 +815,8 @@ app.filter('typeToColor', function($filter) {
         "labeling_task",
         "prompt_studio",
         "agent_tool",
-        "retrievable_knowledge"
+        "retrievable_knowledge",
+        "semantic_model"
     ];
 
     function getStandardType(type) {
@@ -774,6 +828,12 @@ app.filter('typeToColor', function($filter) {
     }
     return function(type, background) {
         if (!type) return "";
+        
+        // Special case for GEN_AI_MODEL and AI_AGENT
+        if (type === 'GEN_AI_MODEL' || type === 'AI_AGENT') {
+            return 'agent-tool';
+        }
+        
         const recipeColor = $filter('recipeTypeToColor')(type);
         if (recipeColor) {
             return recipeColor;
@@ -902,6 +962,10 @@ app.filter('agentToolTypeToIcon', function(TypeMappingService) {
     return TypeMappingService.mapAgentToolTypeToIcon;
 });
 
+app.filter('llmIdToIcon', function(TypeMappingService) {
+    return TypeMappingService.mapLlmIdToIcon;
+});
+
 // Boldify a pattern in the input.
 //
 // Note:
@@ -1123,6 +1187,20 @@ app.filter('fileSizeInMB', function(){
     };
 });
 
+app.filter('friendlyHugeNumber', function(){
+    return function(number) {
+        if (!Number.isInteger(number)) return "";
+        if (number >= 1e9) {
+            return parseFloat((number / 1e9).toFixed(1)) + 'B';
+        } else if (number >= 1e6) {
+            return parseFloat((number / 1e6).toFixed(1)) + 'M';
+        } else if (number >= 1e3) {
+            return parseFloat((number / 1e3).toFixed(1)) + 'k';
+        } else {
+            return number;
+        }
+    }
+});
 
 app.filter("displayMeasure", function() {
     return function(measure) {
@@ -1438,8 +1516,10 @@ app.filter("bundleProjectContent", function() {
     const bundleContentConfigMap = {
         datasets: 'Datasets',
         recipes: 'Recipes',
+        savedModels: 'Models and Agents',
         mlSavedModels: 'ML Saved models',
-        modelEvaluationStores: 'Evaluation stores',
+        modelEvaluationStores: 'Model Evaluation Stores',
+        genAiEvaluationStores: 'GenAi Evaluation Stores',
         managedFolders: 'Managed folders',
         scenarios: 'Scenarios',
         analysis: 'Analyses',
@@ -1448,6 +1528,7 @@ app.filter("bundleProjectContent", function() {
         sqlNotebooks: 'SQL notebooks',
         insights: 'Insights',
         dashboards: 'Dashboards',
+        agentReviews: 'Agent Reviews',
         knowledgeBanks: 'Knowledge Banks',
         promptStudios: 'Prompt Studios',
         llmFineTunedModels: 'LLM Fine-tune models',
