@@ -303,12 +303,16 @@ app.directive("gitLog", function($controller, DataikuAPI, $stateParams) {
             noCommitDiff: '=',
             noAuthorLink: '=',
             createBranchFromCommit: '=',
+            canChangeBranch: "=",
             scrollToCommitId: '=',
             highlightCommitId: '=',
             noTagEdit: '=',
-            redirectAfterRevertCallback: '=?'
+            redirectAfterRevertCallback: '=?',
+            backgroundColor: '@',
         },
         link: function ($scope, element) {
+            $scope.backgroundColor = $scope.backgroundColor || '#f2f2f2'; // @git-color-grey
+
             const objectRef = $scope.objectRef 
                 ? $scope.objectRef
                 : { //assuming 'project' by default
@@ -323,7 +327,8 @@ app.directive("gitLog", function($controller, DataikuAPI, $stateParams) {
                 revertObjectToRevision: (hash) => DataikuAPI.git.revertObjectToRevision(objectRef.projectKey, objectRef.type, objectRef.id, hash),
                 revertProjectToRevision: (hash) => DataikuAPI.git.revertProjectToRevision($stateParams.projectKey, hash, $scope.projectZone),
                 revertSingleCommit: (hash) => DataikuAPI.git.revertSingleCommit($stateParams.projectKey, $scope.objectRef, hash),
-                createBranchFromCommit: $scope.createBranchFromCommit,
+                createBranchFromCommit: (commitId) => $scope.createBranchFromCommit(commitId), // Keep this as a function because it fixes a rare race condition in which the scope could not be correctly initialized (see sc-275732)
+                canChangeBranch: $scope.canChangeBranch,
                 removeTag: (tagName) => DataikuAPI.git.removeTag(objectRef.projectKey, objectRef.type, objectRef.id, tagName),
                 addTag: (tagRef, tagName, tagMessage) => DataikuAPI.git.addTag(objectRef.projectKey, objectRef.type, objectRef.id, tagRef, tagName, tagMessage),
             };
@@ -344,9 +349,11 @@ app.directive("gitMergeRequestLog", function($controller, DataikuAPI, $statePara
         scope: {
             logEntries: '=',
             noAuthorLink: '=',
-            mergeRequest: '='
+            mergeRequest: '=',
+            backgroundColor: '@',
         },
         link: function ($scope, element) {
+            $scope.backgroundColor = $scope.backgroundColor || 'white';
             $scope.objectRevertable = false;
             $scope.createBranchFromCommit = false;
             $scope.commitRevertable = false;
@@ -407,10 +414,12 @@ app.directive("objectGitHistory", function($stateParams, DataikuAPI) {
             commitRevertable: '=?',
             projectZone: '=?',
             redirectAfterRevertCallback: '=?',
+            backgroundColor: '=?',
         },
         templateUrl : "/templates/git/object-git-history.html",
         link: function($scope) {
             const PAGE_SIZE = 20;
+            $scope.backgroundColor = $scope.backgroundColor || '#f2f2f2'; // @git-color-grey
             $scope.hasMore = true;
 
             $scope.objectRef = {
