@@ -3,6 +3,18 @@ const app = angular.module('dataiku.labeling', ['platypus.utils']);
 app.controller("LabelingTaskController", function($scope, $state, $stateParams, $timeout, DataikuAPI, TopNav, $q, Dialogs, DKUtils, ActivityIndicator) {
     $scope.identifier = $stateParams.identifier;
 
+    $scope.getTabFromState = function() {
+        if ($state.current.name.includes('annotate')) {
+            return 'annotate';
+        } else if ($state.current.name.includes('review')) {
+            return 'review';
+        } else if ($state.current.name.includes('settings')) {
+            return 'settings';
+        } else if ($state.current.name.includes('overview')) {
+            return 'overview';
+        }
+    }
+
     $q.all([
         DataikuAPI.labelingtasks.get($stateParams.projectKey, $stateParams.labelingTaskId),
         DataikuAPI.labelingtasks.getLabelingTaskPrivileges($stateParams.projectKey, $stateParams.labelingTaskId),
@@ -556,11 +568,11 @@ app.directive('labelingTaskRightColumnSummary', function ($rootScope, $controlle
 
             });
 
-            scope.editCustomFields = function() {
+            scope.editCustomFields = function(editingTabIndex = 0) {
                 if (!scope.labelingTask) {
                     return;
                 }
-                let modalScope = angular.extend(scope, {objectType: 'LABELING_TASK', objectName: scope.labelingTask.name, objectCustomFields: scope.labelingTask.customFields});
+                let modalScope = angular.extend(scope, {objectType: 'LABELING_TASK', objectName: scope.labelingTask.name, objectCustomFields: scope.labelingTask.customFields, editingTabIndex});
                 CreateModalFromTemplate("/templates/taggable-objects/custom-fields-edit-modal.html", modalScope).then(function(customFields) {
                     LabelingTasksCustomFieldsService.saveCustomFields(scope.labelingTask, customFields);
                 });
