@@ -3,7 +3,7 @@
 
 const app = angular.module('dataiku.llm', ['dataiku.services', 'dataiku.filters']);
 
-app.directive('guardrailsPipeline', function($state, $stateParams, $rootScope, DataikuAPI) {
+    app.directive('guardrailsPipeline', function ($state, $stateParams, $rootScope, DataikuAPI, PluginsService, PluginConfigUtils) {
     return {
         templateUrl :'/templates/llm/guardrails-pipeline.html',
 
@@ -12,8 +12,8 @@ app.directive('guardrailsPipeline', function($state, $stateParams, $rootScope, D
         },
 
         link : function($scope) {
-            $scope.newGuardrail = {
-            }
+            $scope.newGuardrail = {};
+            $scope.customGuardrailInfo = {};
 
             const customGuardrailsDisabled = !$rootScope.appConfig.licensedFeatures.advancedLLMMeshAllowed;
             const customGuardrailsDisabledMessage = customGuardrailsDisabled ? "Requires the Advanced LLM Mesh add-on" : "";
@@ -41,15 +41,16 @@ app.directive('guardrailsPipeline', function($state, $stateParams, $rootScope, D
                     "label": "Response format checker"
                 }
             ]
-            
-            $rootScope.appConfig.customGuardrails.forEach(cg => {
-                $scope.availableGuardrailTypes.push({
+
+            $scope.availableGuardrailTypes.push(...$rootScope.appConfig.customGuardrails
+                .filter(PluginConfigUtils.shouldComponentBeVisible($rootScope.appConfig.loadedPlugins))
+                .map(cg => ({
                     "type": "Custom_" + cg.desc.id,
-                    "label": cg.desc.meta.label,
+                    "label": cg.desc.meta?.label ?? "Custom guardrail " + cg.desc.id,
                     "disabled": customGuardrailsDisabled,
                     "tooltip": customGuardrailsDisabledMessage
-                })
-            });
+                }))
+            );
 
             $scope.addGuardrail = function() {
                 $scope.newGuardrail.params = {}
@@ -102,7 +103,12 @@ app.directive('guardrailsPipeline', function($state, $stateParams, $rootScope, D
 
                 $scope.pipelineSettings.guardrails.forEach(g => {
                     if (g.type.indexOf("Custom") == 0) {
-                        g.customGuardrailDesc = $rootScope.appConfig.customGuardrails.findLast(c => c.guardrailType === g.type.substring(7));
+                        const customGuardrailDesc = $rootScope.appConfig.customGuardrails.findLast(c => c.guardrailType === g.type.substring(7));
+                        const pluginDesc = PluginsService.getOwnerPluginDesc(customGuardrailDesc);
+                        $scope.customGuardrailInfo[g.type] = {
+                            customGuardrailDesc: customGuardrailDesc,
+                            pluginDesc: pluginDesc
+                        }
                     }
                     else if (g.type === 'ForbiddenTermsDetector') {
                         if (g.params.datasetProject) {
@@ -395,26 +401,53 @@ app.component("vectorStoreFilter", {
     bindings: {
         performFiltering: '=',
         filter: '=',
-        knowledgeBank: '<'
+        knowledgeBank: '<',
+        promptInputDatasetColumns: '<?'
     },
     templateUrl: 'templates/llm/vector_store_filter.html',
     controller: function(Expressions) {
         const FILTERABLE_GENERIC_TYPES = ["num", "string"];
         const $ctrl = this;
         $ctrl.$onInit = function() {
+            $ctrl.knowledgeBank.metadataColumnsSchema = $ctrl.knowledgeBank.metadataColumnsSchema.map(c => {
+                if (c.meaning == "BagOfWordsMeaning") {
+                    c.type = "string";
+                }
+                return c;
+            })
             const [filterableColumns, discardedColumns] = getFilterableAndDiscardedColumns($ctrl.knowledgeBank.metadataColumnsSchema);
 
-            $ctrl.filterableColumnsSchema = {
-                columns: filterableColumns
-            };
+            if ($ctrl.promptInputDatasetColumns) {
+                const leftColumns = filterableColumns.map(c => ({ ...c, column: "left", $$groupKey: 'Metadata dataset columns' }));
+                const rightColumns = $ctrl.promptInputDatasetColumns.map(c => ({ ...c, column: "right", $$groupKey: 'Input dataset columns' }));
+
+                $ctrl.filterableColumnsSchema = {
+                    columns: [...leftColumns, ...rightColumns]
+                };
+            } else {
+                $ctrl.filterableColumnsSchema = {
+                    columns: filterableColumns
+                };
+            }
+            
             $ctrl.discardedColumns = discardedColumns;
 
-            if (filterableColumns.length === 0) {
+            if (filterableColumns.length === 0 && $ctrl.promptInputDatasetColumns?.length === 0) {
                 $ctrl.performFiltering = false;
             } else {
                 $ctrl.supportedBooleanOperators = $ctrl.knowledgeBank.filterCapabilities.supportedBooleanOperators;
                 $ctrl.supportedFiltersOperators = function(op) {
-                    return $ctrl.knowledgeBank.filterCapabilities.supportedFiltersOperators.includes(op.name);
+                    const dynamicFiltersMapping = $ctrl.promptInputDatasetColumns ? {
+                        "== [NaNcolumn]" :"== [string]",
+                        "!= [NaNcolumn]" :"!= [string]",
+                        "== [column]": "== [number]",
+                        "!= [column]": "!= [number]",
+                        ">  [column]": ">  [number]",
+                        ">= [column]": ">= [number]",
+                        "<  [column]": "<  [number]",
+                        "<= [column]": "<= [number]",
+                    } : {};
+                    return $ctrl.knowledgeBank.filterCapabilities.supportedFiltersOperators.includes(op.name) || $ctrl.knowledgeBank.filterCapabilities.supportedFiltersOperators.includes(dynamicFiltersMapping[op.name]);
                 }
             }
         }
@@ -428,4 +461,44 @@ app.component("vectorStoreFilter", {
     }
 });
 
+app.component("contextTemplateSelector", {
+    bindings: {
+        onAddTemplate: '&'
+    },
+    templateUrl: 'templates/llm/context-template-selector.html',
+    controller: function() {
+        const $ctrl = this;
+        $ctrl.displayedTemplates = {
+            DYNAMIC_FILTERS: {
+                name: 'Dynamic Filters',
+                template: {
+                    callerFilters: [
+                        {
+                            filter: {
+                                operator: 'AND',
+                                clauses: [
+                                    { column: 'example_column_1', operator: 'EQUALS', value: 'some_value'},
+                                    { column: 'example_column_2', operator: 'GREATER_THAN', value: 1},
+                                ]
+                            },
+                            toolRef: 'projectKey.myToolId'
+                        }
+                    ]
+                }
+            },
+            SECURITY_TOKENS: {
+                name: 'Security Tokens',
+                template: {
+                    callerSecurityTokens: [
+                        'administrators',
+                        'dss_group:administrators',
+                        'dss_user_login:admin',
+                        'dss_user_emailaddress:admin@localhost'
+                    ]
+                }
+            },
+        };
+    }
+});
+
 })();
