@@ -182,7 +182,8 @@ function getSimpleFullname(DIContext) {
 
 
 const EXPECTED_EXCEPTIONS=[
-    "Possibly unhandled rejection: dismissed modal"
+    "Possibly unhandled rejection: dismissed modal",
+    'Possibly unhandled rejection: {"data":{"aborted":true,"reason":"Superseded by new call"},"status":-1}'
 ];
 
 app.factory('$exceptionHandler', function(Logger, ErrorReporting) {
@@ -218,11 +219,11 @@ app.factory('$exceptionHandler', function(Logger, ErrorReporting) {
                 newException.stack = exception.stack;
                 exception = newException;
             }
-
-            Logger.error("Caught exception: " + exception,
-                "\nStack: ", exception.stack,
-                '\nCaused by : ', cause,
-                '\nMessage :', exception.message);
+            if(cause) {
+                Logger.error("Caught exception: ", exception, '\nCaused by: ', cause);
+            } else {
+                Logger.error("Caught exception: ", exception);
+            }
         }
         /* Send to WT1 */
         ErrorReporting.reportJSException(exception, cause);
