@@ -206,6 +206,45 @@ app.directive('ng1DatasetStatus', function() {
     }
 });
 
+app.directive('ng1DatabaseExplorer', function() {
+    return {
+        scope: {
+            isInProject: '<'
+        },
+        template: `<database-explorer class="h100 w100" is-in-project="isInProject"></database-explorer>`,
+    }
+});
+
+app.directive('ng1DatasetPathInput', function() {
+    return {
+        scope: {
+            title: '<',
+            path: '<',
+            pathChange: '&?',
+            browseFn: '&',
+            canSelectFn: '<?',
+            validateFn: '<?',
+            connection: '<',
+            changeNeedsConfirm: '<',
+        },
+        template: `
+        <dataset-path-input
+            class="flex vertical-flex"
+            data-title="{{title}}"
+            path="pathInternal"
+            browse-fn="browseFn"
+            can-select-fn="canSelectFn"
+            validate-fn="validateFn"
+            connection="connection"
+            change-needs-confirm="changeNeedsConfirm"
+        ></dataset-path-input>`,
+        link: function ($scope) {
+            twoWayBindingPrimitive($scope, 'path');
+        }
+    }
+});
+
+
 app.directive('ng1UiView', function() {
     return {
         scope: {
@@ -386,6 +425,10 @@ app.directive('basicSelect', function ($timeout) {
             searchable: '<',
             clearable: '<',
             groupBy: '@',
+            bindGroupLabel: '@',
+            bindGroupAnnotation: '@',
+            bindGroupAnnotationIcon: '@',
+            bindGroupDescription: '@',
             placeholder: '@',
             bindValue: '@',
             bindAnnotation: '@',
@@ -412,6 +455,10 @@ app.directive('basicSelect', function ($timeout) {
             bindLabelFn: '<',
             bindAnnotationFn: '<',
             groupByFn: '<',
+            bindGroupLabelFn: '<',
+            bindGroupAnnotationFn: '<',
+            bindGroupAnnotationIconFn: '<',
+            bindGroupDescriptionFn: '<',
             bindGhostLabelFn: '<',
             bindDisabledFn: '<',
             bindTooltipFn: '<',
@@ -432,8 +479,12 @@ app.directive('basicSelect', function ($timeout) {
                 [actions-box]="actionsBox"
                 [bind-value]="bindValue || bindValueFn"
                 [bind-annotation]="bindAnnotation || bindAnnotationFn"
-                [annotation-layout]="annotationLayout || 'annotation'" // to make sure a value is passed else it's undefined
+                [annotation-layout]="annotationLayout || 'annotation'"
                 [group-by]="groupBy || groupByFn"
+                [bind-group-label]="bindGroupLabel || bindGroupLabelFn"
+                [bind-group-annotation]="bindGroupAnnotation || bindGroupAnnotationFn"
+                [bind-group-annotation-icon]="bindGroupAnnotationIcon || bindGroupAnnotationIconFn"
+                [bind-group-description]="bindGroupDescription || bindGroupDescriptionFn"
                 [track-by]="trackBy || trackByFn"
                 [search-fn]="searchFn"
                 [bind-ghost-label]="bindGhostLabel || bindGhostLabelFn"
@@ -478,7 +529,29 @@ app.directive('basicSelect', function ($timeout) {
     }
 });
 
-app.directive('ng1DatasetPreview', ['MigrationStoreService', function(MigrationStoreService){
+// AngularJS wrapper on top of the downgraded Angular 2+ <new-project-dropdown>
+app.directive('newProjectDropdown', function ($timeout) {
+    return {
+        scope: { },
+        require: 'ngModel',
+        template: `
+            <ng2-new-project-dropdown
+                ng-model="selectedOption"
+                ng-change="valueChanged()"
+            ></ng2-new-project-dropdown>`,
+        link: function ($scope, _element, attrs, ngModelController) {
+            ngModelController.$render = () => {
+                $scope.selectedOption = ngModelController.$modelValue;
+            }
+
+            $scope.valueChanged = () => {
+                $timeout(() => ngModelController.$setViewValue($scope.selectedOption));
+            }
+        }
+    }
+});
+
+app.directive('ng1DatasetPreview', ['MigrationStoreService', '$timeout', function(MigrationStoreService, $timeout){
     return {
         scope: {
             projectKey: '<',
@@ -496,8 +569,8 @@ app.directive('ng1DatasetPreview', ['MigrationStoreService', function(MigrationS
             shakerColoringScheme: '<?'
         },
         template:
-        `<dataset-preview-table 
-            project-key="projectKey" 
+        `<dataset-preview-table
+            project-key="projectKey"
             dataset-name="datasetName"
             embedded-loading-display="embeddedLoadingDisplay"
             show-name="showName"
@@ -514,7 +587,39 @@ app.directive('ng1DatasetPreview', ['MigrationStoreService', function(MigrationS
         link: function ($scope) {
             const subId = MigrationStoreService.subscribe('refreshDatasetPreview', () => $scope.$broadcast('refresh-preview-table-without-cache'));
 
-            $scope.$on('$destroy', () => MigrationStoreService.unsubscribe(subId));
+
+
+            // preview scroll to column logic. We must track the data loaded events in order to wait for the preview to be rendered before we try to scroll
+            let lastLoadedProjectKey, lastLoadedDataset, pendingScroll;
+            const flashColumnSubId = MigrationStoreService.subscribeIfExists('DatasetPreview.scrollToColumn', ({projectKey, dataset, column}) => {
+                if(projectKey === lastLoadedProjectKey && dataset === lastLoadedDataset) {
+                    // the correct dataset data is already loaded => do it now
+                    $scope.$broadcast('scrollToColumn', column);
+                    pendingScroll = undefined;
+                } else {
+                    // the correct dataset data is not loaded, we just store the request for scroll
+                    pendingScroll = {projectKey, dataset, column};
+                }
+            });
+
+            // this event is triggered just after a preview data is loaded
+            $scope.$on("refresh-preview-table-done", (ev, projectKey, dataset) => {
+                lastLoadedDataset = dataset;
+                lastLoadedProjectKey = projectKey;
+
+                // if the last request for scroll matched the data loaded, execute it
+                if(pendingScroll && pendingScroll.projectKey === projectKey && pendingScroll.dataset === dataset) {
+                    $timeout(() => { // magic rendering delay
+                        $scope.$broadcast('scrollToColumn', pendingScroll.column);
+                        pendingScroll = undefined;
+                    }, 0)
+                }
+            });
+
+            $scope.$on('$destroy', () => {
+                MigrationStoreService.unsubscribe(subId);
+                MigrationStoreService.unsubscribe(flashColumnSubId);
+            });
         }
     }
 }]);
@@ -888,8 +993,8 @@ app.directive('ng1DateRangePicker', function() {
             format="{{ format }}"
         ></div>`,
         link: function ($scope) {
-            twoWayBingingPrimitive($scope, 'start', {transform: (val) => val || ''});
-            twoWayBingingPrimitive($scope, 'end', {transform: (val) => val || ''});
+            twoWayBindingPrimitive($scope, 'start', {transform: (val) => val || ''});
+            twoWayBindingPrimitive($scope, 'end', {transform: (val) => val || ''});
         }
     }
 });
@@ -965,12 +1070,14 @@ app.directive('ng1DashboardTileContent', function() {
             activeFilters: '<',
             filters: '<',
             page: '<',
+            pageIdx: '<',
             dashboardTheme: '<',
             showGrid: '<',
             selected: '<',
             currentGroupTileIdInEditMode: '<',
             tileSpacing: '<',
             preselectedTile: '<',
+            autoStackUp: '<',
             insightsMap: '<',
             accessMap: '<',
             canExportDatasets: '<',
@@ -989,12 +1096,14 @@ app.directive('ng1DashboardTileContent', function() {
                 active-filters="activeFilters"
                 filters="filters"
                 page="page"
+                page-idx="pageIdx"
                 dashboard-theme="dashboardTheme"
                 show-grid="showGrid"
                 selected="selected"
                 current-group-tile-id-in-edit-mode="currentGroupTileIdInEditMode"
                 tile-spacing="tileSpacing"
                 preselected-tile="preselectedTile"
+                auto-stack-up="autoStackUp"
                 insights-map="insightsMap"
                 access-map="accessMap"
                 can-export-datasets="canExportDatasets"
@@ -1078,7 +1187,7 @@ app.directive('ng1StdAggrMeasureDropzone', ['ChartTypeChangeHandler', function(C
             // re inject scope from parents
             ['activeDragDrop', 'validity', 'addClassHereAndThere',
                 'removeClassHereAndThere', 'onDragEnd', 'chart',
-                'onPercentileChoice', 'customMeasures', 'editEntity', 
+                'onPercentileChoice', 'customMeasures', 'editEntity', 'onChartDefPropertyChange',
                 'openSection', 'toggleContextualMenu', 'globallyOpenContextualMenu'].map(reassignScopeProperty);
             $scope.acceptMeasure = function(data) {
                 return ChartTypeChangeHandler.stdAggregatedAcceptMeasureWithAlphanumResults(data);
@@ -1136,7 +1245,14 @@ app.directive('ng1ProjectScenariosRuns', function() {
     }
 });
 
-
+app.directive('ng1JsonFormater', function () {
+    return {
+        scope: {
+            jsonData: '<'
+        },
+        template: `<pre json-formater="jsonData"></pre>`
+    };
+});
 
 app.directive('ng1NestedFilter', function() {
     return {
@@ -1231,6 +1347,160 @@ app.directive('ng1SchemaEditor', function($rootScope) {
     }
 });
 
+app.directive('ng1FolderPathInput', () => {
+    return {
+        restrict: 'E',
+        scope: {
+            title: '@',
+            path: '=',
+            browseFn: '=',
+            validateFn: '=',
+            connection: '@',
+            changeNeedsConfirm: '=',
+            displayItemFn: '=?',
+            canSelectFn: '=?',
+            folder: '=?',
+            showRootFolderPath: '=?',
+            cantWriteContentVerb: '@',
+            searchable: '@',
+            inputId: '@',
+            inputEnabled: '&?',
+        },
+
+        template: `
+        <folder-path-input
+            title="{{title}}"
+            connection="{{connection}}"
+            cant-write-content-verb="{{cantWriteContentVerb}}"
+            searchable="{{searchable}}"
+            input-id="{{inputId}}"
+            
+            path="path"
+            browse-fn="browseFn"
+            validate-fn="validateFn"
+            change-needs-confirm="changeNeedsConfirm"
+            display-item-fn="displayItemFn"
+            can-select-fn="canSelectFn"
+            folder="folder"
+            show-root-folder-path="showRootFolderPath"
+            
+            input-enabled="inputEnabled(locals)">
+        </folder-path-input>
+    `
+    };
+});
+
+app.directive('ng1SimpleLogs', function($compile) {
+    return {
+        scope: {
+            objId  : '<',
+            logs : '<',
+            getLog : '<',
+            downloadUrl : '<',
+            downloadAllUrl : '<?',
+            deleteLog : '<?',
+            hideFilter : '<?',
+            preventPolling : '<?',
+        },
+        template: '',
+        link: {
+            pre: function($scope, $el) {
+                function monkeyPatchPromise(promise) {
+                    promise.success = function(callback) {
+                        promise.then(function(data, status, headers, config, statusText, xhrStatus) {
+                            callback(data === 'null' ? null : data, status, headers, config, statusText, xhrStatus);
+                        });
+                        return promise;
+                    };
+                    promise.error = function(callback) {
+                        promise.then(null, function(data, status, headers) {
+                            callback(data, status, headers);
+                        });
+                        return promise;
+                    };
+                    return promise;
+                }
+
+                let div = '<div simple-logs logs="logs" obj-id="objId"';
+
+                $scope.getLogWrapper = (...args) => {
+                    const logName = args.pop();
+                    return monkeyPatchPromise($scope.getLog(logName, ...args));
+                };
+                div += ' get-log="getLogWrapper"';
+
+                $scope.downloadUrlWrapper = (...args) => {
+                    const logName = args.pop();
+                    return $scope.downloadUrl(logName, ...args);
+                };
+                div += ' download-url="downloadUrlWrapper"';
+
+                if ($scope.downloadAllUrl) {
+                    $scope.downloadAllUrlWrapper = (...args) => $scope.downloadAllUrl(...args);
+                    div += ' download-all-url="downloadAllUrlWrapper"';
+                }
+
+                if ($scope.deleteLog) {
+                    $scope.deleteLogWrapper = (...args) => {
+                        const logName = args.pop();
+                        return monkeyPatchPromise($scope.deleteLog(logName, ...args));
+                    }
+                    div += ' delete-log="deleteLogWrapper"';
+                }
+
+                div += ' hide-filter="hideFilter"';
+                div += ' prevent-polling="preventPolling"';
+
+                div += '></div>';
+
+                const $div = $compile(div)($scope);
+                $el.append($div);
+            }
+        }
+    }
+});
+
+app.directive('ng1LlmSelector', function() {
+    return {
+        scope: {
+            availableLlms: '<',
+            selectedLlmId: '<',
+            selectedLlmIdChange: '&?',
+            placeholder: '<',
+            nullValueLabel: '<',
+            nullValueId: '<',
+            hideCopyButton: '<',
+            hideGoToLlmButton: '<',
+            disableAlertWarning: '<',
+            dropdownPosition: '@',
+            defaultOpened: '<',
+            disabled: '<',
+            clearable: '<'
+        },
+
+        template: `
+        <llm-selector
+            available-llms="availableLlms"
+            selected-llm-id="selectedLlmIdInternal"
+            placeholder="placeholder"
+            null-value-label="nullValueLabel"
+            null-value-id="nullValueId"
+            hide-copy-button="hideCopyButton"
+            hide-go-to-llm-button="hideGoToLlmButton"
+            default-opened="defaultOpened"
+            disable-alert-warning="disableAlertWarning"
+            dropdown-position="{{dropdownPosition}}"
+            disabled="disabled"
+            clearable="clearable"
+            >
+        </llm-selector>`,
+
+        link: function ($scope) {
+            twoWayBindingPrimitive($scope, 'selectedLlmId');
+        }
+    }
+});
+
 /* A helper around watches to make the upgraded component more Angular-behaved with emission when value changes from the angularjs side only
  * Won't work with objects in most case as deep compare & deep copies would be required to properly handle in-place mutations & immutability
  * example:
@@ -1242,14 +1512,14 @@ app.directive('ng1Wrapped', function() {
         },
         template: `<wrapped param="paramInternal"></wrapped>`,
         link: function ($scope) {
-            twoWayBingingPrimitive($scope, 'param');
+            twoWayBindingPrimitive($scope, 'param');
         }
     }
 });
 * optionally you can apply transformations from / to Angular
 */
 
-function twoWayBingingPrimitive($scope, name, opts) {
+function twoWayBindingPrimitive($scope, name, opts = {}) {
     const internalName = opts.internalName || name + 'Internal';
     const eventEmitterName = opts.eventEmitterName || name + 'Change';
     const ngx2ng = opts.ngx2ng || ((x) => x);
