@@ -8,7 +8,7 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
             Assert, DataikuAPI, WT1, TopNav, Breadcrumb, CreateModalFromTemplate, TaggingService, FlowGraphSelection,
             GlobalProjectActions, Notification, HistoryService, FutureWatcher, ProgressStackMessageBuilder, ProjectActionsService,
             AlationCatalogChooserService, WebAppsService, DatasetUtils, FlowGraph, TaggableObjectsService, ComputableSchemaRecipeSave,
-            TypeMappingService, TYPE_MAPPING, Logger, Ng1SamplesService) {
+            TypeMappingService, TYPE_MAPPING, Logger, Ng1SamplesService, LastOpenedHistoryService) {
     $controller('_CreateRecipesBehavior', {$scope: $scope});
 
     $scope.standardizedSidePanel = {};
@@ -20,6 +20,9 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
         $scope.standardizedSidePanel.tabToToggle = '';
         $timeout(() => { $scope.standardizedSidePanel.tabToToggle = tabName; }, 50);
     }
+    $scope.$on("opalsCurrentRightPanelTab", function (_, tab) {
+        $scope.standardizedSidePanel.activeTab = tab;
+    });
 
     $scope.$on('$stateChangeStart', function (event, toState, toParams, fromState) {
         if (fromState.name === 'projects.project.flow') {
@@ -27,6 +30,8 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
         }
     });
 
+    LastOpenedHistoryService.projects.opened($stateParams.projectKey);
+
     const getProjectAccessInfo = () => {
         DataikuAPI.projects.getProjectAccessInfo($stateParams.projectKey).success(function(accessInfo) {
             delete $rootScope.projectSummary;
@@ -56,18 +61,24 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
             $scope.projectSummaryStatus = data.objectsCounts;
 
             TopNav.setProjectData($scope.projectSummary, $scope.projectCurrentBranch);
+            HistoryService.recordProjectOpen($scope.projectSummary);
+
             $scope.topNav.isProjectAnalystRO = $scope.isProjectAnalystRO();
             $scope.topNav.isProjectAnalystRW = $scope.isProjectAnalystRW();
             $scope.topNav.isCurrentProjectAdmin = $scope.isProjectAdmin();
             $scope.topNav.canReadDashboards = $scope.canReadDashboards();
-            $scope.topNav.canAccessProjectSettings = $scope.isProjectAdmin();
+            $scope.topNav.canAccessProjectSettings = $scope.canAccessProjectSettings();
             $scope.topNav.canAccessProjectSecurity = $scope.canAccessProjectSecurity();
             $scope.topNav.isAppInstance = $scope.isAppInstance();
+            $scope.topNav.isBusinessAppInstance = $scope.isBusinessAppInstance();
             $scope.topNav.showFlowNavLink = $scope.showFlowNavLink();
             $scope.topNav.showGenAiNavLink = $scope.showGenAiNavLink();
             $scope.topNav.showCodeNavLink = $scope.showCodeNavLink();
             $scope.topNav.showLabNavLink = $scope.showLabNavLink();
+            $scope.topNav.showAutomationNavLink = $scope.showAutomationNavLink();
             $scope.topNav.showVersionControlFeatures = $scope.showVersionControlFeatures();
+            $scope.topNav.showDashboardsNavLink = $scope.showDashboardsNavLink();
+            $scope.topNav.showWikiNavLink = $scope.showWikiNavLink();
 
             if ($scope.projectSummary.tutorialProject) {
                 WT1.setVisitorParam("tutorial-project", "true");
@@ -89,6 +100,8 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
 
             $scope.isObjectSharingRequestEnabled = $rootScope.appConfig.objectSharingRequestsMode === 'ALL_ENABLED' ||
                                             ($rootScope.appConfig.objectSharingRequestsMode !== 'ALL_DISABLED' && $scope.projectSummary.sharingRequestsEnabled);
+            
+            $scope.updateDatasetStorageConflictsWarning();
         }).error(setErrorInScope.bind($scope));
     };
 
@@ -124,11 +137,15 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
     };
     // To be in sync with PagesSettingsCatalogPermissionService:canAccessProjectSecurity
     $scope.canAccessProjectSecurity = function() {
-        return $scope.projectSummary != null &&
-            ($scope.projectSummary.canManageDashboardAuthorizations ||
-             $scope.projectSummary.canManageExposedElements ||
-             $scope.projectSummary.canManageAdditionalDashboardUsers ||
-             $scope.projectSummary.canEditPermissions);
+        return $scope.projectSummary != null
+            && ($scope.projectSummary.canManageDashboardAuthorizations
+                || $scope.projectSummary.canManageExposedElements
+                || $scope.projectSummary.canManageAdditionalDashboardUsers
+                || $scope.projectSummary.canEditPermissions)
+            && !$scope.isBusinessAppInstance();
+    };
+    $scope.canAccessProjectSettings = function() {
+        return $scope.isProjectAdmin() && !$scope.isBusinessAppInstance();
     };
     $scope.isProjectAnalystRO = function() {
         return profileMayReadProjectContent() && $scope.projectSummary != null && $scope.projectSummary.canReadProjectContent;
@@ -161,22 +178,34 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
         return $scope.projectSummary != null && $scope.projectSummary.canExecuteApp;
     };
     $scope.isAppInstance = function() {
-        return $scope.projectSummary != null && $scope.projectSummary.projectAppType == 'APP_INSTANCE';
+        return $scope.projectSummary != null && $scope.projectSummary.projectAppType === 'APP_INSTANCE';
+    };
+    $scope.isBusinessAppInstance = function() {
+        return $scope.projectSummary != null && $scope.projectSummary.projectType === 'BUSINESS_APP';
     };
     $scope.showFlowNavLink = function() {
-        return !$scope.isAppInstance() || $scope.projectSummary.appManifest.instanceFeatures.showFlowNavLink;
+        return !$scope.isBusinessAppInstance() && (!$scope.isAppInstance() || $scope.projectSummary.appManifest.instanceFeatures.showFlowNavLink);
     };
     $scope.showGenAiNavLink = function() {
-        return !$scope.isAppInstance() || $scope.projectSummary.appManifest.instanceFeatures.showGenAiNavLink;
+        return !$scope.isBusinessAppInstance() && (!$scope.isAppInstance() || $scope.projectSummary.appManifest.instanceFeatures.showGenAiNavLink);
     };
     $scope.showCodeNavLink = function() {
-        return !$scope.isAppInstance() || $scope.projectSummary.appManifest.instanceFeatures.showCodeNavLink;
+        return !$scope.isBusinessAppInstance() && (!$scope.isAppInstance() || $scope.projectSummary.appManifest.instanceFeatures.showCodeNavLink);
     };
     $scope.showLabNavLink = function() {
-        return !$scope.isAppInstance() || $scope.projectSummary.appManifest.instanceFeatures.showLabNavLink;
+        return !$scope.isBusinessAppInstance() && (!$scope.isAppInstance() || $scope.projectSummary.appManifest.instanceFeatures.showLabNavLink);
+    };
+    $scope.showAutomationNavLink = function() {
+        return !$scope.isBusinessAppInstance();
+    };
+    $scope.showWikiNavLink = function() {
+        return !$scope.isBusinessAppInstance();
+    };
+    $scope.showDashboardsNavLink = function() {
+        return !$scope.isBusinessAppInstance();
     };
     $scope.showVersionControlFeatures = function() {
-        return !$scope.isAppInstance() || $scope.projectSummary.appManifest.instanceFeatures.showVersionControlFeatures;
+        return !$scope.isBusinessAppInstance() && (!$scope.isAppInstance() || $scope.projectSummary.appManifest.instanceFeatures.showVersionControlFeatures);
     };
 
 
@@ -222,9 +251,9 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
         function isEligibleRecipe(recipeType) {
             // generate statistics recipes -> completely different schema, could not possibly want to reconnect
             // eval recipes -> makes no sense to reconnect
-            const excluded_categories = [TYPE_MAPPING.RECIPE_CATEGORIES.EVALUATION, TYPE_MAPPING.RECIPE_CATEGORIES.STANDALONE, TYPE_MAPPING.RECIPE_CATEGORIES.EDA];
-            const excluded_types = ['nlp_llm_evaluation'];
-            return !excluded_categories.includes(TypeMappingService.mapRecipeTypeToCategory(recipeType)) && !excluded_types.includes(recipeType);
+            const excludedCategories = [TYPE_MAPPING.RECIPE_CATEGORIES.EVALUATION, TYPE_MAPPING.RECIPE_CATEGORIES.STANDALONE, TYPE_MAPPING.RECIPE_CATEGORIES.EDA];
+            const excludedTypes = ['nlp_llm_evaluation', 'nlp_agent_evaluation'];
+            return !excludedCategories.includes(TypeMappingService.mapRecipeTypeToCategory(recipeType)) && !excludedTypes.includes(recipeType);
         }
 
         function hasSuccessors(dsNode) {
@@ -417,11 +446,21 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
     $scope.newManagedFolder = function() {
         CreateModalFromTemplate("/templates/managedfolder/new-box-modal.html", $scope);
     };
-    $scope.newModelEvaluationStore = function() {
-        CreateModalFromTemplate("/templates/modelevaluationstores/new-model-evaluation-store-modal.html", $scope);
+    $scope.newModelEvaluationStore = function(allowTabular = true, allowGenAi = false) {
+        $scope.allowTabularCreation = allowTabular;
+        $scope.allowGenAiCreation = allowGenAi;
+        if (!$rootScope.appConfig.licensedFeatures.advancedLLMMeshAllowed && allowGenAi && !allowTabular) {
+            CreateModalFromTemplate("/templates/llm/advanced-llm-mesh-required.html", $scope, null, function(modalScope) {
+                modalScope.uiState = {
+                    "forbiddenFeature" : "GenAI Evaluation Store"
+                }
+            });
+        } else {
+            CreateModalFromTemplate("/templates/modelevaluationstores/new-model-evaluation-store-modal.html", $scope);
+        }
     };
     $scope.newSample = function() {
-        Ng1SamplesService.openSampleModal($scope.projectSummary.projectKey);
+        Ng1SamplesService.openSampleModal($scope.projectSummary.projectKey, $scope.getRelevantZoneId());
     };
 
     $scope.importFromAlation = function(){
@@ -688,11 +727,11 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
             });
     };
 
-    $scope.editCustomFields = function() {
+    $scope.editCustomFields = function(editingTabIndex = 0) {
         if (!$scope.projectSummary) {
             return;
         }
-        let modalScope = angular.extend($scope, {objectType: 'PROJECT', objectName: $scope.projectSummary.name, objectCustomFields: $scope.projectSummary.customFields});
+        let modalScope = angular.extend($scope, {objectType: 'PROJECT', objectName: $scope.projectSummary.name, objectCustomFields: $scope.projectSummary.customFields, editingTabIndex});
         CreateModalFromTemplate("/templates/taggable-objects/custom-fields-edit-modal.html", modalScope).then(function(customFields) {
             $scope.saveCustomFields(customFields);
         });
@@ -826,6 +865,29 @@ app.controller('ProjectBaseController', function($scope, $controller, $state, $s
             $scope.projectGovernanceStatus = { loading: false, error: fatalAPIError };
         });
     };
+
+    /* Dataset table names conflicts warning */
+    $scope.updateDatasetStorageConflictsWarning = () => {
+        if($scope.isProjectAnalystRW()) {
+            return DataikuAPI.flow.testDatasetStorageConflicts($stateParams.projectKey).then(({data}) => {
+                $scope.testDatasetStorageConflicts = data;
+            }).catch(setErrorInScope.bind($scope));
+        } else {
+            $scope.testDatasetStorageConflicts = {
+                totalManagedWithIssue: 0,
+                duplicates: [],
+            };
+            return $q.resolve();
+        }
+    };
+
+    $scope.showTableNameDuplicateDetails = () => {
+        // we update before showing the modal in case user changed something since
+        $scope.updateDatasetStorageConflictsWarning().then(() => {
+            CreateModalFromTemplate('/templates/datasets/modals/duplicate-table-name-flow-modal.html', $scope, null)
+        })
+    };
+
 });
 
 app.controller("MoveProjectModalController", function($scope, $rootScope, $controller, DataikuAPI, $stateParams, translate, ActivityIndicator) {
@@ -867,7 +929,7 @@ app.controller("MoveProjectModalController", function($scope, $rootScope, $contr
     };
 });
 
-app.service('ProjectActionsService', function($rootScope, $stateParams, CreateModalFromTemplate, DataikuAPI, FutureWatcher, $timeout, $state, WT1, Assert, ProgressStackMessageBuilder, AI_EXPLANATION_MODAL_MODES, ProjectFolderContext)  {
+app.service('ProjectActionsService', function($rootScope, $stateParams, CreateModalFromTemplate, DataikuAPI, FutureWatcher, $timeout, $state, WT1, Assert, ProgressStackMessageBuilder, AI_EXPLANATION_MODAL_MODES, ProjectFolderContext, FutureProgressModal)  {
     this.deleteThisProject = function(scope, projectSummary, projectKey = $stateParams.projectKey, transition = 'home', transitionParams = {}) {
         DataikuAPI.projects.checkDeletability(projectKey).success((data) => {
             if (data.anyMessage) {
@@ -878,29 +940,32 @@ app.service('ProjectActionsService', function($rootScope, $stateParams, CreateMo
                 });
             } else {
                 CreateModalFromTemplate("/templates/projects/delete-project-confirm-dialog.html", scope, null, function(newScope) {
-                    newScope.clearManagedDatasets = false;
+                    newScope.clearManagedDatasets = $rootScope.appConfig.dropDataWithDatasetDeleteEnabled;
                     newScope.clearOutputManagedFolders = false;
                     newScope.clearJobAndScenarioLogs = true;
+                    newScope.clearManagedKnowledgeBanks = true;
                     newScope.projectSummary = projectSummary;
 
-                    newScope.confirmProjectDeletion = function(clearManagedDatasets, clearOutputManagedFolders, clearJobAndScenarioLogs) {
-                        DataikuAPI.projects.delete(projectKey, clearManagedDatasets, clearOutputManagedFolders, clearJobAndScenarioLogs).success(function(deletionResult) {
-                            if(deletionResult.anyMessage) {
-                                CreateModalFromTemplate("/templates/projects/delete-project-results.html", scope, null, function(newScope) {
-                                    newScope.beforeDeletion = false;
-                                    newScope.results = deletionResult.messages;
-                                    newScope.$on('$destroy',function() {
-                                        $timeout(function() {
-                                            $state.transitionTo(transition, transitionParams);
+                    newScope.confirmProjectDeletion = function(clearManagedDatasets, clearOutputManagedFolders, clearJobAndScenarioLogs, clearManagedKnowledgeBanks) {
+                        DataikuAPI.projects.delete(projectKey, clearManagedDatasets, clearOutputManagedFolders, clearJobAndScenarioLogs, clearManagedKnowledgeBanks).success(function(future) {
+                            FutureProgressModal.show(scope, future, "Deleting project").then(function(deletionResult) {
+                                if(deletionResult.anyMessage) {
+                                    CreateModalFromTemplate("/templates/projects/delete-project-results.html", scope, null, function(newScope) {
+                                        newScope.beforeDeletion = false;
+                                        newScope.results = deletionResult.messages;
+                                        newScope.$on('$destroy',function() {
+                                            $timeout(function() {
+                                                $state.transitionTo(transition, transitionParams);
+                                            });
                                         });
                                     });
-                                });
-                            } else {
-                                $state.transitionTo(transition, transitionParams);
-                            }
+                                } else {
+                                    $state.transitionTo(transition, transitionParams);
+                                }
+                            });
 
                         }).error(setErrorInScope.bind($rootScope));
-                        WT1.event("project-delete", {clearManagedDatasets, clearOutputManagedFolders, clearJobAndScenarioLogs});
+                        WT1.event("project-delete", {clearManagedDatasets, clearOutputManagedFolders, clearJobAndScenarioLogs, clearManagedKnowledgeBanks});
                     }
                 });
             }
@@ -932,6 +997,7 @@ app.service('ProjectActionsService', function($rootScope, $stateParams, CreateMo
                 exportInsights: true,
                 exportNotebooksWithOutputs: true,
                 exportEditableDatasets: true,
+                exportDatasetResources: true,
                 exportGitRepository: canExportGitRepository
             };
 
@@ -1005,7 +1071,7 @@ app.service('ProjectActionsService', function($rootScope, $stateParams, CreateMo
     }
 });
 
-app.controller('_CreateRecipesBehavior', function($scope, CreateModalFromTemplate, RecipeDescService, SmartId) {
+app.controller('_CreateRecipesBehavior', function($scope, $state, $stateParams, CreateModalFromTemplate, RecipeDescService, SmartId, DatasetUtils, FeatureFlagsService, SavedModelsService, WT1) {
     function preselect(inputDatasetSmartName,zone,recipeAdditionalParams) {
         return function(newScope) {
             newScope.zone = zone;
@@ -1015,6 +1081,7 @@ app.controller('_CreateRecipesBehavior', function($scope, CreateModalFromTemplat
             }
         };
     }
+
     $scope.showCreateShakerModal = function(inputDatasetSmartName, zone, recipeAdditionalParams) {
         CreateModalFromTemplate('/templates/recipes/single-output-recipe-creation.html', $scope, 'ShakerRecipeCreationController', preselect(inputDatasetSmartName, zone, recipeAdditionalParams));
     };
@@ -1075,6 +1142,27 @@ app.controller('_CreateRecipesBehavior', function($scope, CreateModalFromTemplat
     $scope.showCreateLabelingTaskModal = function(inputDatasetSmartName, zone) {
         CreateModalFromTemplate('/templates/labelingtasks/labeling-task-creation.html', $scope, "LabelingTaskCreationController", preselect(inputDatasetSmartName, zone));
     };
+    $scope.showCreateAgentReviewModal = function(agentId, versionId) {
+        if ($scope.appConfig.licensedFeatures.advancedLLMMeshAllowed) {
+            WT1.event('agent-review-action', { action: 'review:new-review' });
+            $scope.agentId = agentId;
+            $scope.versionId = versionId;
+            CreateModalFromTemplate('/templates/agentreviews/agent-review-creation.html', $scope, "AgentReviewCreationController").catch(() => {
+                $state.go(
+                    'projects.project.agentreviews.list',
+                    { projectKey: $stateParams.projectKey, agentId: null, versionId: null },
+                    { location: 'replace' }
+                );
+            });
+        } else {
+            CreateModalFromTemplate("/templates/llm/advanced-llm-mesh-required.html", $scope, null, function(modalScope) {
+                modalScope.uiState = {
+                    "forbiddenFeature" : "Agent Review"
+                }
+            });
+        }
+
+    };
     $scope.showCreatePivotModal = function(inputDatasetSmartName,zone) {
         CreateModalFromTemplate("/templates/recipes/pivot-recipe-creation.html", $scope, null, preselect(inputDatasetSmartName, zone));
     };
@@ -1173,12 +1261,18 @@ app.controller('_CreateRecipesBehavior', function($scope, CreateModalFromTemplat
         });
     };
 
-    $scope.showCreateListAccessModal = function(inputFolderSmartName, inputFolderName, zone) {
+    $scope.showCreateListAccessModal = function(singleSelectedFolder, singleSelectedDataset, zone) {
         CreateModalFromTemplate('/templates/recipes/list_access-recipe-creation.html', $scope, 'ListAccessRecipeCreationController', function(newScope) {
             newScope.zone = zone;
-            if (inputFolderSmartName && inputFolderName) {
-                newScope.recipe.inputs.main.items.push({ ref: inputFolderSmartName });
-                newScope.$broadcast('preselectInputFolder', { smartName: inputFolderSmartName, name: inputFolderName });
+            if (singleSelectedFolder) {
+                const inputFolderSmartName = DatasetUtils.makeSmart(DatasetUtils.makeLoc(singleSelectedFolder.projectKey, singleSelectedFolder.name), $scope.$stateParams.projectKey);
+                newScope.recipe.inputs.main.items.push({ ref: inputFolderSmartName, description: singleSelectedFolder.description });
+                newScope.$broadcast('preselectInputFolder', inputFolderSmartName);
+                if (singleSelectedDataset) {
+                    const inputDatasetSmartName = DatasetUtils.makeSmart(DatasetUtils.makeLoc(singleSelectedDataset.projectKey, singleSelectedDataset.name), $scope.$stateParams.projectKey);
+                    newScope.recipe.inputs.manualMapping.items.push({ ref: inputDatasetSmartName });
+                    newScope.$broadcast('preselectInputDataset', inputDatasetSmartName);
+                }
             }
         });
     };
@@ -1197,6 +1291,17 @@ app.controller('_CreateRecipesBehavior', function($scope, CreateModalFromTemplat
         });
     };
 
+    $scope.showCreateExtractContentModal = function(folderId, folderProjectKey, folderName, zone) {
+        CreateModalFromTemplate('/templates/recipes/Nto1-recipe-creation.html', $scope, 'ExtractContentRecipeCreationController', function(newScope) {
+            newScope.zone = zone;
+            if (folderId && folderProjectKey && folderName) {
+                let inputFolderSmartName = SmartId.create(folderId, folderProjectKey);
+                newScope.recipe.inputs.main.items.push({ref: inputFolderSmartName});
+                newScope.$broadcast('preselectInputFolder', {smartName: inputFolderSmartName, name: folderName});
+            }
+        });
+    };
+
     $scope.showSQLRecipeModal = function(inputDatasetSmartName, zone, recipeAdditionalParams) {
         CreateModalFromTemplate('/templates/flow-editor/new-sql-recipe-box.html', $scope, null, function(newScope) {
             newScope.zone = zone;
@@ -1224,6 +1329,19 @@ app.controller('_CreateRecipesBehavior', function($scope, CreateModalFromTemplat
         });
     };
 
+    $scope.getLLMIdFromSelection = function(selection) {
+        if (!SavedModelsService.isLLM(selection)) {
+            return;
+        }
+        const llmId = SmartId.create(selection.interest.objectId, selection.interest.projectKey);
+        if (SavedModelsService.isAgent(selection)) {
+            return "agent:" + llmId;
+        }
+        if (SavedModelsService.isRetrievalAugmentedLLM(selection)) {
+            return "retrieval-augmented-llm:" + llmId;
+        }
+    }
+
     $scope.showCreatePromptModal = function(preselectedInput, preselectedImageFolder, initialPayload, zone) {
         CreateModalFromTemplate('/templates/recipes/prompt-recipe-creation.html', $scope, 'PromptRecipeCreationController', function(newScope) {
             newScope.zone = zone;
@@ -1291,11 +1409,21 @@ app.controller('_CreateRecipesBehavior', function($scope, CreateModalFromTemplat
         });
     };
 
-    $scope.showCreateNLPLLMEvaluationRecipeModal = function(preselectedInput, initialPayload, zone) {
+    $scope.showTestInPythonNotebookModal = function (preselectedInput) {
+        const kb = preselectedInput[0];
+        $scope.retrievableKnowledge = {
+            id: SmartId.create(kb.id, kb.projectKey),
+            name: kb.displayName,
+        };
+        CreateModalFromTemplate('/templates/retrievable-knowledge/test-in-python-notebook.html', $scope, 'TestInPythonNotebookModalController');
+    };
+
+    $scope.showCreateLLMEvaluationRecipeModal = function(preselectedInput, initialPayload, zone) {
         if ($scope.appConfig.licensedFeatures.advancedLLMMeshAllowed) {
-            CreateModalFromTemplate('/templates/recipes/new-llm-evaluate-recipe.html', $scope, 'NLPLLMEvaluationRecipeCreationController', function(newScope) {
+            CreateModalFromTemplate('/templates/recipes/new-llm-evaluate-recipe.html', $scope, 'LLMEvaluationRecipeCreationController', function(newScope) {
                 newScope.zone = zone;
                 newScope.initialPayload = initialPayload;
+                newScope.mesFlavor = "LLM";
                 if (preselectedInput) {
                     newScope.uiState.inputDs = preselectedInput;
                 }
@@ -1309,6 +1437,25 @@ app.controller('_CreateRecipesBehavior', function($scope, CreateModalFromTemplat
         }
     };
 
+    $scope.showCreateAgentEvaluationRecipeModal = function(preselectedInput, initialPayload, zone) {
+        if ($scope.appConfig.licensedFeatures.advancedLLMMeshAllowed) {
+            CreateModalFromTemplate('/templates/recipes/new-agent-evaluate-recipe.html', $scope, 'AgentEvaluationRecipeCreationController', function(newScope) {
+                newScope.zone = zone;
+                newScope.initialPayload = initialPayload;
+                newScope.mesFlavor = "AGENT";
+                if (preselectedInput) {
+                    newScope.uiState.inputDs = preselectedInput;
+                }
+            });
+        } else {
+            CreateModalFromTemplate("/templates/llm/advanced-llm-mesh-required.html", $scope, null, function(modalScope) {
+                modalScope.uiState = {
+                    "forbiddenFeature" : "Agent evaluation"
+                }
+            });
+        }
+    };
+
     $scope.showCreateCustomCodeRecipeModal = function(recipeType, inputRefs, inputRole, zone){
         CreateModalFromTemplate('/templates/recipes/custom-code-recipe-creation.html', $scope, null, function(newScope) {
             newScope.zone = zone;
@@ -1397,6 +1544,10 @@ app.controller('_CreateRecipesBehavior', function($scope, CreateModalFromTemplat
         newScope.zone = labelingTask.zone;
         CreateModalFromTemplate('/templates/labelingtasks/labeling-task-creation.html', newScope);
     };
+
+    $scope.allowKsqlRecipeCreation = function() {
+        return FeatureFlagsService.featureFlagEnabled('ignoreKsqlDeprecation');
+    };
 });
 
 app.controller('ContextualProjectHomeController', function($scope, $state, $stateParams, TopNav, HistoryService, $timeout, ProjectHomeContextService) {
@@ -1406,7 +1557,6 @@ app.controller('ContextualProjectHomeController', function($scope, $state, $stat
         if (!nv)  return;
         if (nv.projectKey === $stateParams.projectKey) {
             let stateToRedirectTo = 'projects.project.home.summary';
-            HistoryService.recordProjectOpen($scope.projectSummary);
             $timeout(() => ProjectHomeContextService.recordProjectViewed($stateParams.projectKey));
             if (!$stateParams.discussionId) {
                 stateToRedirectTo = ProjectHomeContextService.getProjectRedirectState(nv, $scope.projectSummaryStatus);
@@ -1475,8 +1625,8 @@ app.controller('ProjectHomeController', function($scope, $state, $stateParams, $
     $scope.$watch("projectSummary", function(nv) {
         if (!nv)  return;
         if (nv.projectKey == $stateParams.projectKey) {
+            WT1.event("open-project-summary", { projectAppType: $scope.projectSummary.projectAppType });
             if ($scope.projectSummary.name) {
-                HistoryService.recordProjectOpen($scope.projectSummary);
                 TopNav.setPageTitle($scope.projectSummary.name);
             }
             if ($scope.projectSummary.projectAppType == 'APP_INSTANCE' && nv.projectKey) {
