@@ -92,7 +92,7 @@ app.controller("_SQLRecipeController", function($scope, DataikuAPI, $stateParams
             dialect = $scope.computablesMap[firstInputNameFromRecipe].datasetType;
             return {
                 ...AISqlGenerationService.buildWT1SqlGenerationParams($scope.wt1RecipeType, recipeSerialized.inputs.main.items.length, dialect),
-                aiServer: $rootScope.appConfig.isUsingLocalAiAssitant && $rootScope.appConfig.isUsingLocalAiAssitant.aiGenerateSQL ? "webapp" : "default"
+                aiServer: $rootScope.appConfig.isUsingLocalAIAssistant ? "webapp" : "default"
             };
         })
         $scope.ratingFeedbackParams.showRatingFeedback = false;
@@ -109,7 +109,7 @@ app.controller("_SQLRecipeController", function($scope, DataikuAPI, $stateParams
             $scope.aiQuery = AISqlGenerationService.processSQLGenerationResponse(data.result.queryName, data.result.sqlQuery, data.result.reasoning, data.result.messages);
             $scope.installingFuture = null;
             requestId = data.result.requestId;
-            if (!$rootScope.appConfig.isUsingLocalAiAssitant?.aiGenerateSQL) {
+            if (!$rootScope.appConfig.isUsingLocalAIAssistant) {
                 $scope.ratingFeedbackParams.showRatingFeedback = true;
                 $scope.ratingFeedbackParams.requestIdForFeedback = requestId;
                 $scope.ratingFeedbackParams.featureRated = "text2sql";
@@ -117,7 +117,7 @@ app.controller("_SQLRecipeController", function($scope, DataikuAPI, $stateParams
             WT1.tryEvent('ai-sql-generation-response', () => {
                 return {
                     ...AISqlGenerationService.buildWT1SqlGenerationResponseParams($scope.wt1RecipeType, dialect, requestId, $scope.aiQuery.warnings, $scope.aiQuery.errors),
-                    aiServer: $rootScope.appConfig.isUsingLocalAiAssitant && $rootScope.appConfig.isUsingLocalAiAssitant.aiGenerateSQL ? "webapp" : "default"
+                    aiServer: $rootScope.appConfig.isUsingLocalAIAssistant ? "webapp" : "default"
                 };
             });
         }).error(function (data, status, headers) {
@@ -125,7 +125,7 @@ app.controller("_SQLRecipeController", function($scope, DataikuAPI, $stateParams
             WT1.tryEvent('ai-sql-generation-error', () => {
                 return {
                     ...AISqlGenerationService.buildWT1SqlErrorResponseParams($scope.wt1RecipeType, dialect),
-                    aiServer: $rootScope.appConfig.isUsingLocalAiAssitant && $rootScope.appConfig.isUsingLocalAiAssitant.aiGenerateSQL ? "webapp" : "default"
+                    aiServer: $rootScope.appConfig.isUsingLocalAIAssistant ? "webapp" : "default"
                 };
             });
         }).finally(function() {
@@ -144,7 +144,7 @@ app.controller("_SQLRecipeController", function($scope, DataikuAPI, $stateParams
         WT1.tryEvent('ai-sql-insert-query', () => {
             return {
                 ...AISqlGenerationService.buildWT1SqlInsertQueryParams($scope.wt1RecipeType, dialect, requestId),
-                aiServer: $rootScope.appConfig.isUsingLocalAiAssitant && $rootScope.appConfig.isUsingLocalAiAssitant.aiGenerateSQL ? "webapp" : "default"
+                aiServer: $rootScope.appConfig.isUsingLocalAIAssistant? "webapp" : "default"
             };
         });
     };
