@@ -394,221 +394,427 @@ app.controller("OutcomesBaseController", function($scope, Fn) {
     $scope.selected = {column : null, row : null};
 });
 
-app.controller("OutcomesViewController", function($scope, $controller, $stateParams, DataikuAPI, $state, TopNav, ScenarioUtils, CreateModalFromTemplate, WT1) {
-    $controller('OutcomesBaseController', {$scope: $scope});
+app.controller(
+    "OutcomesViewController",
+    function ($scope, $controller, $state, $stateParams, DataikuAPI, ScenarioUtils, StateUtils, WT1) {
+        $controller("OutcomesBaseController", { $scope: $scope });
 
-    var selectedElement = null;
-    var pickerFormat = 'YYYY-MM-DD';
-    var backendFormat = "YYYY-MM-DD";
-    var nextHour = moment().add(1, 'hour').startOf('hour');
-    $scope.outcomesRange = {from : moment(nextHour).subtract(30, 'days').format(pickerFormat), to : moment(nextHour).format(pickerFormat)};
+        let selectedElement = null;
+        const pickerFormat = "YYYY-MM-DD";
+        const backendFormat = "YYYY-MM-DD";
+        const nextHour = moment().add(1, "hour").startOf("hour");
 
-    $scope.projectKey = $stateParams.projectKey;
-    $scope.scenarioId = $stateParams.scenarioId;
+        $scope.executionOptions = [
+            ["all", "All"],
+            ["aborted", "Aborted"],
+            ["warning", "Warning"],
+            ["failed", "Failed"],
+            ["success", "Success"],
+        ];
 
-    $scope.reportRange = {from : 0, to : 0}
-    $scope.displayedColumns = [];
-    $scope.displayedRows = [];
-    $scope.scenarioOwners = [];
-    $scope.automationMonitoringScenarioOutcomeFilter = {
-        owner: null,
-        status: 'ALL',
-        tags: []
-    };
-    $scope.allTags = [];
-
-    function deduplicateArrayOfArrays(arrayWithDuplicates) {
-      let seen = new Set();
-      let deduplicatedArray = [];
-      for (let index = 0; index < arrayWithDuplicates.length; index++) {
-        if (!seen.has(arrayWithDuplicates[index][0])) {
-          seen.add(arrayWithDuplicates[index][0]);
-          deduplicatedArray.push(arrayWithDuplicates[index]);
-        }
-      }
-      return deduplicatedArray;
-    }
-
-    $scope.refreshOutcomes = function() {
-        WT1.event("refresh-scenarios-outcomes");
-        // send time as is, so that the backend get the day range inputted by the user
-        var rangeFromTs = moment($scope.outcomesRange.from, pickerFormat);
-        var rangeToTs = moment($scope.outcomesRange.to, pickerFormat);
-        $scope.loading = true;
-        DataikuAPI.scenarios.getOutcomes(rangeFromTs.format(backendFormat), rangeToTs.add(1, 'days').format(backendFormat), $scope.projectKey, $scope.scenarioId).success(function(data){
-            $scope.outcomes = data;
-            if ($scope.outcomes && $scope.outcomes.rows) {
-                $scope.scenarioOwners = [];
-                $scope.outcomes.rows.forEach(
-                    function(item, index) {
-                        $scope.scenarioOwners.push([item.info.login, item.info.runAsDisplayName]);
-                    }
-                );
-                $scope.scenarioOwners = deduplicateArrayOfArrays($scope.scenarioOwners);
-                $scope.scenarioOwners = $scope.scenarioOwners.sort(function(a,b){
-                    return a[1].toLowerCase() > b[1].toLowerCase() ? 1 : -1;
-               });
-               if ($scope.scenarioOwners.length > 0 && $scope.scenarioOwners[0][0] != null) {
-                    $scope.scenarioOwners.unshift([null, "All"]);
-               }
+        /** Get a formatted date string from an input date string.
+         *
+         * @param str The date string to validate and format.
+         * @returns a string that can be:
+         * - a formatted date string
+         * - undefined if the input is null or undefined
+         * - null if the input is not a valid date string
+         */
+        function getFormattedDateString(str) {
+            if (isDefined(str)) {
+                const formattedDate = moment(str, pickerFormat, true);
+                if (formattedDate.isValid()) {
+                    return formattedDate.format(pickerFormat);
+                }
+                return null;
             }
-            $scope.fixupOutcomes(data.columns);
-            // refresh displayed range
-//            if ( $scope.outcomes.key.from > $scope.reportRange.to || $scope.outcomes.key.to < $scope.reportRange.from ) {
-                // replace range
-                $scope.reportRange.from = $scope.outcomes.key.from;
-                $scope.reportRange.to = $scope.outcomes.key.to;
-//            } else {
-//                // clip range
-//                $scope.reportRange.from = $scope.outcomes.key.from > $scope.reportRange.from ? $scope.outcomes.key.from : $scope.reportRange.from;
-//                $scope.reportRange.to = $scope.outcomes.key.to < $scope.reportRange.to ? $scope.outcomes.key.to : $scope.reportRange.to;
-//            }
+            return undefined;
+        }
+        const inputStartDate = getFormattedDateString($stateParams.start);
+        const inputEndDate = getFormattedDateString($stateParams.end);
+
+        $scope.outcomesRange = {
+            from: isDefined(inputStartDate)
+                ? inputStartDate
+                : moment(nextHour).subtract(30, "days").format(pickerFormat),
+            to: isDefined(inputEndDate) ? inputEndDate : moment(nextHour).format(pickerFormat),
+        };
+
+        $scope.projectKey = $stateParams.projectKey;
+        $scope.scenarioId = $stateParams.scenarioId;
+
+        $scope.reportRange = { from: 0, to: 0 };
+        $scope.displayedColumns = [];
+        $scope.displayedRows = [];
+        $scope.scenarioOwners = [[null, "All"]];
+        $scope.automationMonitoringScenarioOutcomeFilter = {
+            owner: null,
+            status: getExecutionFromParams($stateParams.execution),
+            tags: getTagsFromParams($stateParams.tags),
+        };
+        $scope.allTags = [];
+        $scope.filterTags = [];
+        $scope.lastOutcomesRangeLoaded = null;
+        $scope.isPeriodDefined = isDefined(inputStartDate) || isDefined(inputEndDate);
+        $scope.hasPeriodChanged = !$scope.isPeriodDefined;
+
+        function updateUrlIfInvalidParams() {
+            let urlToUpdate = false;
+            // Period check
+            if (inputStartDate === null || inputEndDate === null) {
+                urlToUpdate = true;
+            }
+            // Execution check
+            else if (isDefined($stateParams.execution) && $scope.automationMonitoringScenarioOutcomeFilter.status !== $stateParams.execution) {
+                urlToUpdate = true;
+            }
+            // User check
+            else if (isDefined($stateParams.user) && $scope.automationMonitoringScenarioOutcomeFilter.owner !== $stateParams.user) {
+                urlToUpdate = true;
+            }
+
+            if (urlToUpdate) {
+                updateUrl();
+            }
+        }
+
+        function getExecutionFromParams(data) {
+            if (isDefined(data) && $scope.executionOptions.some(option => option[0] === data)) {
+                return data;
+            }
+            return "all";
+        }
+
+        function getTagsFromParams(data) {
+            let tags = [];
+            if (isDefined(data)) {
+                if (Array.isArray(data)) {
+                    return [...new Set(data)];
+                } else {
+                    tags.push(data);
+                }
+            }
+            return tags;
+        }
+
+        // First gets the user list at the loading page...
+        // After getting the user list:
+        // - sets the owner
+        // - updates the URL if some URL params are invalid
+        // - displays the outcomes automatically if a period is defined in the URL params
+        DataikuAPI.security
+            .listUsers()
+            .success(function (data) {
+                $scope.scenarioOwners = _.sortBy(data, user => user.displayName.toLowerCase()).map(user => [
+                    user.login,
+                    user.displayName,
+                ]);
+                if ($scope.scenarioOwners.length > 0 && $scope.scenarioOwners[0][0] != null) {
+                    $scope.scenarioOwners.unshift([null, "All"]);
+                }
+            })
+            .error(setErrorInScope.bind($scope))
+            .finally(() => {
+                // Sets or resets the user
+                if (
+                    isDefined($stateParams.user) &&
+                    $scope.scenarioOwners.some(owner => owner[0] === $stateParams.user)
+                ) {
+                    $scope.automationMonitoringScenarioOutcomeFilter.owner = $stateParams.user;
+                } else {
+                    // Sets to all if not exists
+                    $scope.automationMonitoringScenarioOutcomeFilter.owner = null;
+                }
+
+                // Updates the URL if necessary
+                updateUrlIfInvalidParams();
+
+                // Displays the outcomes automatically if a period is defined in the URL params
+                if ($scope.isPeriodDefined) {
+                    setTimeout(function () {
+                        $scope.refreshOutcomes();
+                    }, 50);
+                }
+            });
+
+        $scope.refreshOutcomes = function () {
+            if ($scope.loading) {
+                return;
+            }
+
+            WT1.event("refresh-scenarios-outcomes");
+            // send time as is, so that the backend get the day range inputted by the user
+            var rangeFromTs = moment($scope.outcomesRange.from, pickerFormat);
+            var rangeToTs = moment($scope.outcomesRange.to, pickerFormat);
+            $scope.loading = true;
+            storeLastFiltersLoaded();
+            DataikuAPI.scenarios
+                .getOutcomes(
+                    rangeFromTs.format(backendFormat),
+                    rangeToTs.add(1, "days").format(backendFormat),
+                    $scope.projectKey,
+                    $scope.scenarioId
+                )
+                .success(function (data) {
+                    $scope.outcomes = data;
+                    $scope.fixupOutcomes(data.columns);
+                    $scope.reportRange.from = $scope.outcomes.key.from;
+                    $scope.reportRange.to = $scope.outcomes.key.to;
+
+                    // Tags need to be refreshed first because are used to filter/display rows
+                    $scope.refreshTags();
+                    $scope.refreshDisplayedColumns();
+                    $scope.refreshDisplayedRows();
+                })
+                .error(setErrorInScope.bind($scope))
+                .finally(() => {
+                    // Waits a little bit that allTags is up to date in order to restore the previous tags list set by user
+                    // Permits to change the period keeping the same tags list
+                    setTimeout(function () {
+                        $scope.loading = false;
+                        $scope.automationMonitoringScenarioOutcomeFilter = angular.copy(
+                            $scope.lastAutomationMonitoringScenarioOutcomeFilter
+                        );
+                        $scope.$apply(); // To refresh loading
+                    }, 100);
+                });
+        };
+        $scope.$watch("outcomesRange.from", function (nv, ov) {
+            if (nv == null || ov == nv) return;
+            $scope.onFilterUpdated();
+        });
+        $scope.$watch("outcomesRange.to", function (nv, ov) {
+            if (nv == null || ov == nv) return;
+            $scope.onFilterUpdated();
+        });
+
+        $scope.onFilterUpdated = function () {
+            if ($scope.loading) {
+                return;
+            }
+            if ($scope.lastOutcomesRangeLoaded) {
+                // true if the period has changed
+                $scope.hasPeriodChanged =
+                    $scope.outcomesRange.from !== $scope.lastOutcomesRangeLoaded.from ||
+                    $scope.outcomesRange.to !== $scope.lastOutcomesRangeLoaded.to;
+            } else {
+                $scope.hasPeriodChanged = true;
+            }
+            updateUrl();
+
             $scope.refreshDisplayedColumns();
             $scope.refreshDisplayedRows();
-            $scope.fetchTags();
-        }).error(setErrorInScope.bind($scope))
-        .finally(() => {$scope.loading = false;});       
-    };
-    $scope.$watch("outcomesRange.from", function(nv, ov) {
-        if ( nv == null || ov == nv ) return;
-        $scope.refreshOutcomes();
-    });
-    $scope.$watch("outcomesRange.to", function(nv, ov) {
-        if ( nv == null || ov == nv ) return;
-        $scope.refreshOutcomes();
-    });
+        };
 
-    $scope.onFilterUpdated = function onFilterUpdated() {
-        $scope.refreshDisplayedColumns();
-        $scope.refreshDisplayedRows();
-    };
+        $scope.onTagsChanged = function () {
+            computeFilterTags();
+            $scope.onFilterUpdated();
+        };
 
-    $scope.fetchTags = function fetchTags() {
-        for (const row of $scope.displayedRows) {
-            const taggableObject = {
-                type: row.target.type,
-                projectKey: row.target.projectKey,
-                id: row.target.scenarioId
+        function computeFilterTags() {
+            const tags = $scope.automationMonitoringScenarioOutcomeFilter.tags;
+            if (tags && tags.length && $scope.allTags.length) {
+                // Ignores tags which are not in the dropdown tags
+                $scope.filterTags = tags.filter(tag => $scope.allTags.includes(tag));
+            } else {
+                $scope.filterTags = [];
+            }
+        }
+
+        $scope.refreshTags = function () {
+            if ($scope.outcomes && $scope.outcomes.rows) {
+                const flattenedOutcomeTags = $scope.outcomes.rows.map(row => (row.info && row.info.tags) || []).flat();
+                $scope.allTags = [...new Set(flattenedOutcomeTags)];
+            }
+            computeFilterTags();
+        };
+
+        $scope.refreshDisplayedColumns = function () {
+            if ($scope.outcomes == null || $scope.outcomes.columns == null || $scope.reportRange == null) {
+                $scope.displayedColumns = null;
+            } else {
+                var startDate = moment($scope.reportRange.from).format("YYYY-MM-DD");
+                var endDate = moment($scope.reportRange.to).format("YYYY-MM-DD");
+                $scope.displayedColumns = $scope.outcomes.columns.filter(function (column) {
+                    return column.date >= startDate && column.date <= endDate;
+                });
+            }
+        };
+
+        $scope.refreshDisplayedRows = function refreshDisplayedRows() {
+            if ($scope.outcomes == null || $scope.outcomes.rows == null || $scope.reportRange == null) {
+                $scope.displayedRows = null;
+            } else {
+                $scope.displayedRows = $scope.scenarioSearch($scope.outcomes.rows, $scope.displayedColumns);
+            }
+        };
+
+        function storeLastFiltersLoaded() {
+            $scope.lastOutcomesRangeLoaded = angular.copy($scope.outcomesRange);
+            $scope.lastAutomationMonitoringScenarioOutcomeFilter = angular.copy(
+                $scope.automationMonitoringScenarioOutcomeFilter
+            );
+
+            // To not display the load button
+            $scope.hasPeriodChanged = false;
+        }
+
+        function isDefined(variable) {
+            return variable !== null && variable !== undefined;
+        }
+
+        function updateUrl() {
+            const params = {
+                start: $scope.outcomesRange.from,
+                end: $scope.outcomesRange.to,
             };
-            DataikuAPI.taggableObjects.getMetadata(taggableObject).success((result) => {
-                row.tags = result.tags;
-                $scope.allTags = Array.from(new Set([...$scope.allTags, ...result.tags]));
+            // User
+            if (isDefined($scope.automationMonitoringScenarioOutcomeFilter.owner)) {
+                params.user = $scope.automationMonitoringScenarioOutcomeFilter.owner;
+            } else {
+                params.user = null;
+            }
+            // Execution
+            if ($scope.automationMonitoringScenarioOutcomeFilter.status !== "all") {
+                params.execution = $scope.automationMonitoringScenarioOutcomeFilter.status;
+            } else {
+                params.execution = null;
+            }
+            // Tags
+            const tags = $scope.automationMonitoringScenarioOutcomeFilter.tags;
+            if (tags && tags.length) {
+                params.tags = angular.copy(tags);
+            } else {
+                params.tags = null;
+            }
+
+            // Updates the URL without reloading the page
+            StateUtils.go.updateUrlParams(params);
+        }
+
+        $scope.$watch(
+            "reportRange",
+            function (nv) {
+                if (nv == null) return;
+                $scope.refreshDisplayedColumns();
+            },
+            true
+        );
+
+        $scope.scenarioQuery = null;
+        $scope.scenarioSearch = function scenarioSearch(rows, columns) {
+            if (!rows || !rows.length || (!$scope.scenarioQuery && !$scope.automationMonitoringScenarioOutcomeFilter))
+                return rows;
+            return rows.filter(row => {
+                if ($scope.automationMonitoringScenarioOutcomeFilter) {
+                    return (
+                        searchInTargetAndInfo(row, $scope.scenarioQuery) && filterScenarioSearchResults(row, columns)
+                    );
+                } else {
+                    return searchInTargetAndInfo(row, $scope.scenarioQuery);
+                }
             });
-        }
-    }
+        };
 
-    $scope.refreshDisplayedColumns = function() {
-        if ( $scope.outcomes == null || $scope.outcomes.columns == null || $scope.reportRange == null ) {
-            $scope.displayedColumns = null;
-        } else {
-            var startDate = moment($scope.reportRange.from).format('YYYY-MM-DD');
-            var endDate = moment($scope.reportRange.to).format('YYYY-MM-DD');
-            $scope.displayedColumns = $scope.outcomes.columns.filter(function(column) {return column.date >= startDate && column.date <= endDate});
-        }
-    };
-
-    $scope.refreshDisplayedRows = function refreshDisplayedRows() {
-        if ( $scope.outcomes == null || $scope.outcomes.rows == null || $scope.reportRange == null ) {
-            $scope.displayedRows = null;
-        } else {
-            $scope.displayedRows = $scope.scenarioSearch($scope.outcomes.rows, $scope.displayedColumns);
-        }
-    }
-
-    $scope.$watch('reportRange', function(nv) {
-        if ( nv == null ) return;
-        $scope.refreshDisplayedColumns();
-    }, true);
-
-    $scope.scenarioQuery = null;
-    $scope.scenarioSearch = function scenarioSearch(rows, columns) {
-        if (!rows || !rows.length || (!$scope.scenarioQuery && !$scope.automationMonitoringScenarioOutcomeFilter)) return rows;
-        return rows.filter(row => {
-            if ($scope.automationMonitoringScenarioOutcomeFilter) {
-                return searchInTargetAndInfo(row, $scope.scenarioQuery) && filterScenarioSearchResults(row, columns);
-            } else {
-                return searchInTargetAndInfo(row, $scope.scenarioQuery);
+        /**
+         *
+         * @param {object[]} row
+         * @param {object[]} columns
+         * @returns
+         */
+        function filterScenarioSearchResults(row, columns) {
+            const { owner, status, tags } = $scope.automationMonitoringScenarioOutcomeFilter;
+            if (owner && row.info.login != owner) {
+                return false;
             }
-        });
-    };
 
-    /**
-     * 
-     * @param {object[]} row 
-     * @param {object[]} columns 
-     * @returns 
-     */
-    function filterScenarioSearchResults(row, columns) {
-        const { owner, status, tags } = $scope.automationMonitoringScenarioOutcomeFilter;
-        if (owner && row.info.login != owner) {
-            return false;
-        }
-        const scenarioContainsTag = tags && tags.length > 0 && (!row.tags || !row.tags.some(tag => tags.includes(tag)));
-        if (scenarioContainsTag) {
-            return false;
-        }
+            if ($scope.filterTags.length) {
+                // There is some filter applied on tags
+                const rowTags = (row.info && row.info.tags) || [];
+                if (!rowTags.some(tag => $scope.filterTags.includes(tag))) {
+                    return false;
+                }
+            }
 
-        const automationId = row.uniqueId;
-        switch (status) {
-            case 'failed':
-            case 'warning':
-            case 'aborted':
-                return columns.some(col => $scope.getCellGlobalOutcome(col.actions[automationId]) === status);
-            case 'success':
-                // None of the actions is failed
-                return columns.every(col => $scope.getCellGlobalOutcome(col.actions[automationId]) !== 'failed');
-            case 'all':
-            default:
-                return true;
-
-        }
-    }
-
-    $scope.brushChanged = function() { $scope.$digest(); };
-
-    $scope.hover = function(evt, column, row, localScope) {
-        $scope.hovered.date = column.date;
-        $scope.hovered.row = row;
-        $scope.hovered.actions = row ? column.actions[row.uniqueId] : null;
-    };
-    $scope.unhover = function(evt, column, row, localScope) {
-        $scope.hovered.date = null;
-        $scope.hovered.row = null;
-        $scope.hovered.actions = null;
-    };
-    $scope.select = function(evt, column, row, localScope) {
-        $scope.selected = {column : column, row : row, formattedDate : moment(column.date).format('dddd, MMMM Do')};
-        if(selectedElement) selectedElement.classList.remove('active');
-        selectedElement = evt.currentTarget;
-        selectedElement.classList.add('active');
-    };
-
-    $scope.scenarioIsActive = function(scenarioRow) {
-        return scenarioRow.info.active && !ScenarioUtils.getAutoTriggerDisablingReason($scope.appConfig, [$scope.projectSummaryMap[scenarioRow.target.projectKey]]);
-    }
-
-    var selectedTarget = null;
-    var selectedTargetDate = null;
-    $scope.$watch('selected', function() {
-        if ( $scope.selected == null || $scope.selected.row == null || $scope.selected.row.target == null ) {
-            $scope.selected.scenario = null;
-            selectedTarget = null;
-            selectedTargetDate = null;
-        } else {
-            var newSelectedTarget = $scope.selected.row.target;
-            var newSelectedTargetDate = moment($scope.selected.column.date).format('YYYY-MM-DD');
-            if (newSelectedTarget != selectedTarget || newSelectedTargetDate != selectedTargetDate) {
-                selectedTarget = newSelectedTarget;
-                selectedTargetDate = newSelectedTargetDate;
-                DataikuAPI.scenarios.getOutcomesDetails(selectedTarget.projectKey, selectedTarget.scenarioId, selectedTargetDate).success(function(data){
-                    $scope.selected.scenario = data;
-                }).error(setErrorInScope.bind($scope));
-            } else {
-                // no change, don't bother re-fetching
+            const automationId = row.uniqueId;
+            switch (status) {
+                case "failed":
+                case "warning":
+                case "aborted":
+                    return columns.some(col => $scope.getCellGlobalOutcome(col.actions[automationId]) === status);
+                case "success":
+                    // None of the actions is failed
+                    return columns.every(col => $scope.getCellGlobalOutcome(col.actions[automationId]) !== "failed");
+                case "all":
+                default:
+                    return true;
             }
         }
-    }, true);
-});
+
+        $scope.brushChanged = function () {
+            $scope.$digest();
+        };
+
+        $scope.hover = function (evt, column, row, localScope) {
+            $scope.hovered.date = column.date;
+            $scope.hovered.row = row;
+            $scope.hovered.actions = row ? column.actions[row.uniqueId] : null;
+        };
+        $scope.unhover = function (evt, column, row, localScope) {
+            $scope.hovered.date = null;
+            $scope.hovered.row = null;
+            $scope.hovered.actions = null;
+        };
+        $scope.select = function (evt, column, row, localScope) {
+            $scope.selected = { column: column, row: row, formattedDate: moment(column.date).format("dddd, MMMM Do") };
+            if (selectedElement) selectedElement.classList.remove("active");
+            selectedElement = evt.currentTarget;
+            selectedElement.classList.add("active");
+        };
+
+        $scope.scenarioIsActive = function (scenarioRow) {
+            return (
+                scenarioRow.info.active &&
+                !ScenarioUtils.getAutoTriggerDisablingReason($scope.appConfig, [
+                    $scope.projectSummaryMap[scenarioRow.target.projectKey],
+                ])
+            );
+        };
+
+        var selectedTarget = null;
+        var selectedTargetDate = null;
+        $scope.$watch(
+            "selected",
+            function () {
+                if ($scope.selected == null || $scope.selected.row == null || $scope.selected.row.target == null) {
+                    $scope.selected.scenario = null;
+                    selectedTarget = null;
+                    selectedTargetDate = null;
+                } else {
+                    var newSelectedTarget = $scope.selected.row.target;
+                    var newSelectedTargetDate = moment($scope.selected.column.date).format("YYYY-MM-DD");
+                    if (newSelectedTarget != selectedTarget || newSelectedTargetDate != selectedTargetDate) {
+                        selectedTarget = newSelectedTarget;
+                        selectedTargetDate = newSelectedTargetDate;
+                        DataikuAPI.scenarios
+                            .getOutcomesDetails(
+                                selectedTarget.projectKey,
+                                selectedTarget.scenarioId,
+                                selectedTargetDate
+                            )
+                            .success(function (data) {
+                                $scope.selected.scenario = data;
+                            })
+                            .error(setErrorInScope.bind($scope));
+                    } else {
+                        // no change, don't bother re-fetching
+                    }
+                }
+            },
+            true
+        );
+    }
+);
 
 app.directive('outcomeCells', function($filter, $timeout) {
     return {
