@@ -19,7 +19,7 @@
         ChartDimension, ChartLabels, ChartRequestComputer, $state, $timeout, PluginsService, Logger, ActivityIndicator,
         ChartTypeChangeHandler, ChartDefinitionChangeHandler, ChartUADimension, _MapCharts, ChartsStaticData, ChartConfigurationCopyPaste, DetectUtils, ChartIconUtils, ChartSetErrorInScope, ChartFeatures, PolygonSources, ChartAxesUtils,
         ChartZoomControlAdapter, ChartDataUtils, WT1, CHART_TYPES, CHART_VARIANTS, ChartStoreFactory, ColorUtils, $rootScope, $stateParams, ChartsAvailableTypes, RegressionTypes, ChartFilterUtils, ChartFormattingPane, ChartFormattingPaneSections,
-        DataGroupingMode, ChartColorSelection, ConditionalFormattingOptions, ChartDataWrapperFactory, DSSVisualizationThemeUtils, ChartDrilldown, ChartHierarchyDimension, ChartHierarchies, DRILL_UP_SOURCE) {
+        DataGroupingMode, ChartColorSelection, ConditionalFormattingOptions, ChartDataWrapperFactory, DSSVisualizationThemeUtils, ChartDrilldown, ChartHierarchyDimension, ChartHierarchies, DRILL_UP_SOURCE, localStorageService, ChartActivityIndicator, DashboardUtils) {
         return {
             restrict: 'AE',
             templateUrl: '/static/dataiku/js/simple_report/directives/chart-configuration/chart-configuration.directive.html',
@@ -28,6 +28,7 @@
                 scope.$watch(attrs['dropdownMaxHeightPx'], function(newVal) {
                     scope.dropdownMaxHeightPx = newVal;
                 });
+                scope.chartActivityIndicator = ChartActivityIndicator.buildDefaultActivityIndicator();
                 //needs to be put in the scope so these functions are accessible in the dashboards
                 scope.canHaveZoomControls = ChartFeatures.canHaveZoomControls;
                 scope.canDisplayLegend = ChartFeatures.canDisplayLegend;
@@ -357,7 +358,7 @@
 
                     scope.computeChartOptionsPreview = function() {
                         const macos = DetectUtils.getOS() === 'macos';
-                        return macos ? '/static/dataiku/images/charts/previews/cmd.svg' : '/static/dataiku/images/charts/previews/ctrl.svg';
+                        return macos ? '/static/dataiku/images/charts/previews/cp_cmd.svg' : '/static/dataiku/images/charts/previews/cp_ctrl.svg';
                     };
 
                     scope.computeChartPreview = function(type, variant) {
@@ -715,17 +716,30 @@
                         const newValue = formatDefinition(nv);
                         const oldValue = formatDefinition(ov);
 
+                        const broadCastUpdateChartPromptSuggestions = () => {
+                            if ($rootScope.appConfig.aiText2ChartEnabled && newValue.type != 'webapp' && (scope.createAndPinInsight || scope.isInAnalysis || scope.isInNoteBooks) && $rootScope.featureFlagEnabled('chartAssistant')) {
+                                $rootScope.$broadcast('updateChartPromptSuggestions');
+                            }
+                        };
+
                         // Check for a change which triggers save + recompute + redraw
                         const importantChangedDefinition = ChartDefinitionChangeHandler.getRecomputeChange(newValue, oldValue);
                         if (importantChangedDefinition) {
+                            broadCastUpdateChartPromptSuggestions();
                             $timeout.cancel(themeChangeTimeoutId);
                             $timeout.cancel(frontImportantChangeTimeoutId); // apply immediate change, prevail on delayed changes
                             onChartImportantDefinitionChanged(importantChangedDefinition, newValue);
+
+                            if (importantChangedDefinition.name === 'filters') {
+                                $rootScope.$broadcast('chartFiltersUpdate', nv?.id);
+                            }
+
                             return;
                         }
                         // Check for a change which triggers save + redraw
                         const frontChangedDefinition = ChartDefinitionChangeHandler.getFrontImportantChange(newValue, oldValue);
                         if (frontChangedDefinition) {
+                            broadCastUpdateChartPromptSuggestions();
                             $timeout.cancel(themeChangeTimeoutId);
                             $timeout.cancel(frontImportantChangeTimeoutId); // apply immediate change, prevail on delayed changes
                             onChartFrontImportantDefinitionChanged(frontChangedDefinition, newValue);
@@ -734,6 +748,7 @@
                         // Check for a change which triggers save + redraw after a timeout
                         const delayedChangedDefinition = ChartDefinitionChangeHandler.getDelayedFrontImportantChange(newValue, oldValue);
                         if (delayedChangedDefinition) {
+                            broadCastUpdateChartPromptSuggestions();
                             $timeout.cancel(themeChangeTimeoutId);
                             // Timeouting to prevent excessive refresh while user is still typing in a custom input.
                             $timeout.cancel(frontImportantChangeTimeoutId);
@@ -871,8 +886,8 @@
                             CHART_TYPES.BINNED_XY
                         ].includes(scope.chart.def.type);
 
-                        scope.canAnimate = ChartFeatures.canAnimate(scope.chart.def.type, scope.chart.def.variant);
-                        scope.canFacet = ChartFeatures.canFacet(scope.chart.def.type, scope.chart.def.webAppType, scope.chart.def.variant);
+                        scope.canAnimate = ChartFeatures.canAnimate(scope.chart.def.type);
+                        scope.canFacet = ChartFeatures.canFacet(scope.chart.def.type, scope.chart.def.webAppType);
                         scope.canFilter = ChartFeatures.canFilter(scope.chart.def.type, scope.chart.def.webAppType);
                         scope.updateHierarchyInfo();
 
@@ -1235,6 +1250,56 @@
                     scope.display0Warning = false;
                     scope.hasUnusableHierarchy = false;
                 };
+
+                scope.aiText2ChartEnabled = $rootScope.appConfig.aiText2ChartEnabled;
+                scope.chartsAIAssistantVisible = (
+                    scope.aiText2ChartEnabled &&
+                    !scope.readOnly &&
+                    localStorageService.get('chartsAIAssistantVisible') !== 'false'
+                ) ? true : false;
+                scope.toggleAIAssistant = function(value) {
+                    if (typeof value === 'boolean' && value !== scope.chartsAIAssistantVisible) {
+                        scope.chartsAIAssistantVisible = value;
+                        localStorageService.set('chartsAIAssistantVisible', scope.chartsAIAssistantVisible);
+                        scope.redraw(); // redraw bc of chart resize
+                    }
+                };
+
+                scope.setHasOngoingLLMRequest = function(hasOngoingLLMRequest) {
+                    scope.hasOngoingLLMRequest = hasOngoingLLMRequest;
+
+                    if (!scope.$$phase) {
+                        scope.$apply();
+                    }
+                };
+
+                scope.LLMUpdateChartDef = function(newChartDef) {
+                    scope.charts[scope.currentChart.index].def = newChartDef;
+
+                    scope.chartTypePickerOptions = {
+                        ...scope.chartTypePickerOptions,
+                        chartVariant: newChartDef.variant,
+                        chartType: newChartDef.type
+                    };
+
+                    if (!scope.$$phase) {
+                        scope.$apply();
+                    }
+                };
+
+                scope.LLMRevertChartDef = function(prevChartDef) {
+                    // It's the same memory reference, but we reassign in case the internal implementation will change
+                    scope.charts[scope.currentChart.index].def = prevChartDef;
+                    scope.chartTypePickerOptions = {
+                        ...scope.chartTypePickerOptions,
+                        chartVariant: prevChartDef.variant,
+                        chartType: prevChartDef.type
+                    };
+
+                    if (!scope.$$phase) {
+                        scope.$apply();
+                    }
+                };
             }
         };
 
