@@ -12,7 +12,7 @@
      * @typedef {import("../../../../../../../../../server/src/frontend/src/app/features/simple-report/services/chart-axis/chart-axis.type").ChartAxesUtilsReturnType} ChartAxesUtilsReturnType
      * @returns {ChartAxesUtilsReturnType}
      */
-    function chartAxesUtils(ChartDataUtils, ChartUADimension, ChartDimension, ChartsStaticData, ChartFeatures, ChartYAxisPosition, AxisTicksConfigMode, AxisTicksFormatting, AxisTitleFormatting, ChartLabels, translate) {
+    function chartAxesUtils(ChartDataUtils, ChartUADimension, ChartDimension, ChartsStaticData, ChartFeatures, ChartYAxisPosition, AxisTicksConfigMode, AxisTicksFormatting, AxisTitleFormatting, ColumnAvailability, translate, CHART_VARIANTS) {
 
         const AUTO_EXTENT_MODE = 'AUTO';
         const MANUAL_EXTENT_MODE = 'MANUAL';
@@ -106,7 +106,7 @@
                     if (axisSpec.type === 'UNAGGREGATED') {
                         extent = axisSpec.extent || ChartDataUtils.getUnaggregatedAxisExtent(axisSpec.dimension, axisSpec.data);
                     } else {
-                        extent = ChartDataUtils.getAxisExtent(chartData, axisSpec.name, axisSpec.dimension, axisSpec.customExtent && axisSpec.customExtent.$autoExtent, ignoreLabels);
+                        extent = ChartDataUtils.getAxisExtent(chartData, axisSpec.name, axisSpec.dimension, { ignoreLabels, initialExtent: axisSpec.customExtent && axisSpec.customExtent.$autoExtent });
                     }
                 }
 
@@ -238,7 +238,7 @@
             },
 
             contatenateMeasuresNames(measures) {
-                return measures.map(this.getTitleText).join(', ');
+                return measures.map(m => ColumnAvailability.getAggregatedLabel(m)).join(', ');
             },
 
             /**
@@ -259,8 +259,8 @@
                     titleText = xAxisSpec.dimension.column;
                 } else if (xAxisSpec.measure !== undefined) {
                     return this.contatenateMeasuresNames(xAxisSpec.measure);
-                } else if (chartDef.genericDimension0.length == 1) {
-                    titleText = chartDef.genericDimension0[0].column;
+                } else if (chartDef.genericDimension0.length === 1 || chartDef.genericHierarchyDimension.length === 1) {
+                    titleText = (ChartDimension.getGenericDimension(chartDef) || {}).column;
                 }
 
                 return titleText;
@@ -283,7 +283,7 @@
                 } else if (yAxisSpec.dimension !== undefined) {
                     titleText = yAxisSpec.dimension.column;
                 } else if (yAxisSpec.measure !== undefined) {
-                    return this.getTitleText(yAxisSpec.measure[0]);
+                    return ColumnAvailability.getAggregatedLabel(yAxisSpec.measure[0]);
                 } else if (chartDef != null) {
                     const measuresOnAxis = svc.getMeasuresDisplayedOnAxis(yAxisSpec.position, chartDef.genericMeasures);
                     if (measuresOnAxis.length >= 1) {
@@ -294,15 +294,6 @@
                 return titleText;
             },
 
-            getTitleText(measure) {
-                if (measure.column == null && measure['function'] === 'COUNT') {
-                    return ChartLabels.COUNT_OF_RECORDS_LABEL;
-                } else {
-                    const agg = measure['function'] === 'PERCENTILE' ? `PCT(${measure['percentile']})` : measure['function'];
-                    return measure.column + ' (' + agg + ')';
-                }
-            },
-
             getMeasuresDisplayedOnAxis: function(axisPosition, measures) {
                 const displayAxis = axisPosition === ChartYAxisPosition.LEFT ? 'axis1' : 'axis2';
                 return measures.filter(measure => measure.displayAxis === displayAxis);
@@ -347,6 +338,7 @@
                     if (axisLabels && axisSpecs[specId] && axisSpecs[specId].dimension) {
                         axisSpecs[specId].dimension.$numberOfBins = axisLabels.length;
                         axisSpecs[specId].dimension.$isInteractiveChart = ChartDimension.isInteractiveChart(chartDef);
+                        axisSpecs[specId].dimension.$forceOneTickPerBin = chartDef.variant === CHART_VARIANTS.waterfall;
                     }
                 }
             },
