@@ -7,7 +7,7 @@
     /**
      * (!) This service previously was in static/dataiku/js/simple_report/common/data.js
      */
-    function ChartDataUtils(ChartDimension, ChartUADimension, Fn, $filter, CHART_DATES_LABELS, ChartLabels, CHART_TYPES, ChartsStaticData, translate) {
+    function ChartDataUtils(ChartDimension, ChartMeasure, ChartUADimension, Fn, $filter, CHART_DATES_LABELS, ChartLabels, CHART_TYPES, ChartsStaticData, translate) {
         /**
          * Returns true if:
          * <ul>
@@ -156,12 +156,13 @@
             /**
              * Returns the min & max values across all dimensions & all measures for the two display axes
              * @param {ChartDef.java} chartDef
-             * @param {PivotTableTensorResponse.java} data
+             * @param {ChartTensorDataWrapper} chartData
+             * @param {string} mainAxisName - id of the main axis
              * @param {Array} timestampRange min and max used to filter the data based on their timestamp
              * @param {Boolean} includeEmptyBins - whether or not to include empty bins
              * @return {Object} { y1: { extent: [Number, Number], onlyPercent: Boolean }, y2: { extent: [Number, Number], onlyPercent: Boolean }, recordsCount: Number, pointsCount: Number }
              */
-            getMeasureExtents: function(chartDef, data, timestampRange, includeEmptyBins) {
+            getMeasureExtents: function(chartDef, chartData, mainAxisName, timestampRange, includeEmptyBins) {
                 const result = {
                     [ChartsStaticData.LEFT_AXIS_ID]: buildDefaultExtent(),
                     [ChartsStaticData.RIGHT_AXIS_ID]: buildDefaultExtent(),
@@ -169,20 +170,25 @@
                     pointsCount: 0
                 };
 
-                const mainAxisLabel = data.axisLabels[0];
-                const countsTensor = data.counts.tensor;
+                const countsTensor = chartData.data.counts.tensor;
+                const mainAxisLabels = chartData.getAxisLabels(mainAxisName);
                 chartDef.genericMeasures.forEach(function(measure, measureIndex) {
-                    const aggregationTensorForMeasure = data.aggregations[measureIndex].tensor;
-                    const measureExtent = d3.extent(
-                        filterTensorOnTimestampRange(aggregationTensorForMeasure, mainAxisLabel, countsTensor, timestampRange, includeEmptyBins)
-                    );
+                    let measureExtent = [];
+                    const measureData = chartData.data.aggregations[measureIndex];
+                    if (ChartMeasure.isRealUnaggregatedMeasure(measure)) {
+                        const filterTensor = (t) => filterTensorOnTimestampRange(t, mainAxisLabels, countsTensor, timestampRange, includeEmptyBins);
+                        measureExtent = ChartMeasure.getUaMeasureExtent(measureData, filterTensor);
+                    } else {
+                        const filteredTensor = filterTensorOnTimestampRange(measureData.tensor, mainAxisLabels, countsTensor, timestampRange, includeEmptyBins);
+                        measureExtent = d3.extent(filteredTensor);
+                    }
                     const axis = measure.displayAxis === 'axis1' ? ChartsStaticData.LEFT_AXIS_ID : ChartsStaticData.RIGHT_AXIS_ID;
                     result[axis].onlyPercent = result[axis].onlyPercent && ChartDimension.isPercentScale([measure]);
                     result[axis].extent[0] = _.isFinite(measureExtent[0]) ? Math.min(measureExtent[0], result[axis].extent[0]) : result[axis].extent[0];
                     result[axis].extent[1] = _.isFinite(measureExtent[1]) ? Math.max(measureExtent[1], result[axis].extent[1]) : result[axis].extent[1];
                 });
 
-                const countsTensorInRange = filterTensorOnTimestampRange(countsTensor, mainAxisLabel, countsTensor, timestampRange, includeEmptyBins);
+                const countsTensorInRange = filterTensorOnTimestampRange(countsTensor, mainAxisLabels, countsTensor, timestampRange, includeEmptyBins);
                 result.recordsCount = countsTensorInRange.reduce((currentCount, countInBin) => currentCount + countInBin, 0);
                 result.pointsCount = countsTensorInRange.length;
                 return result;
@@ -239,7 +245,14 @@
                 // We now have all the values we want we create the extent, mix can't have timestamp yet so we don't care of them
                 chartDef.genericMeasures.forEach((measure, measureIndex) => {
                     const tensorValues = ignoreForMeasure && ignoreForMeasure(measure) ? this.retrieveAggrData(chartData, valuesToCheckForColumns, measureIndex) : this.retrieveAggrData(chartData, valuesToCheckForLines, measureIndex);
-                    const measureExtent = d3.extent(tensorValues);
+                    let measureExtent = [];
+                    if (ChartMeasure.isRealUnaggregatedMeasure(measure)) {
+                        measureExtent = ChartMeasure.getUaMeasureExtent(this.retrieveUnaggrExtents(chartData, ignoreForMeasure && ignoreForMeasure(measure) ? valuesToCheckForColumns : valuesToCheckForLines, measureIndex));
+                    } else {
+                        measureExtent = d3.extent(tensorValues);
+                    }
+
+
                     const axis = measure.displayAxis === 'axis1' ? ChartsStaticData.LEFT_AXIS_ID : ChartsStaticData.RIGHT_AXIS_ID;
                     result[axis].onlyPercent = result[axis].onlyPercent && ChartDimension.isPercentScale([measure]);
                     result[axis].extent[0] = Math.min(measureExtent[0], result[axis].extent[0]);
@@ -259,25 +272,41 @@
                 }, []);
             },
 
+            retrieveUnaggrExtents: function(chartData, coordsArray, measureIndex) {
+                return coordsArray.reduce((acc, coords) => {
+                    const extents = chartData.getPointExtents(chartData.data.aggregations[measureIndex], Array.isArray(coords) ? coords : [coords]);
+                    acc.uaDataNegativeExtent.push(extents[0]);
+                    acc.uaDataPositiveExtent.push(extents[1]);
+                    return acc;
+                }, { uaDataNegativeExtent: [], uaDataPositiveExtent: [] });
+            },
+
             /**
              * Returns the min & max values across all dimensions for the given measure
-             * @param {PivotTableTensorResponse.java} data
+             * @param {ChartTensorDataWrapper} chartData
              * @param {Number} mIdx - measure index
              * @param {Boolean} ignoreEmptyBins - whether or not to ignore empty bins
              * @param {Set<int> | null} binsToInclude - subtotal tensor indexes to include, the other will be excluded
+             * @param {number[]} axesIdx - indexes of other axes
              * @return {Array} extent as [min, max]
              */
-            getMeasureExtent: function(chartData, mIdx, ignoreEmptyBins, binsToInclude = null, aggrFn = null) {
+            getMeasureExtent: function(chartData, mIdx, ignoreEmptyBins, binsToInclude = null, aggrFn = null, axesIdx = []) {
+                const allMeasureIndexes = axesIdx.concat(mIdx);
                 const data = chartData.data;
 
-                if (!data.aggregations[mIdx]) {
+                if (allMeasureIndexes.every(mIdx => !data.aggregations[mIdx])) {
                     return null;
                 }
 
                 let accessor = Fn.SELF;
                 if (ignoreEmptyBins) {
                     accessor = function(d, i) {
-                        if (chartData.isBinMeaningful(aggrFn, i, mIdx)) {
+                        // if the current axis has null for a value, but one of the other axes don't, we should still consider that value in the extent
+                        const hasNonNullElement = allMeasureIndexes.some(mIdx => chartData.getNonNullCount(i, mIdx) > 0);
+
+                        // maybe check isBinMeaningful for every axis ?
+
+                        if (hasNonNullElement || chartData.isBinMeaningful(aggrFn, i, mIdx)) {
                             return !binsToInclude || binsToInclude.has(i) ? d : null;
                         }
                         return null;
@@ -313,14 +342,21 @@
              * @param {ChartTensorDataWrapper} chartData
              * @param {String} axisName: the name of the axis in chartData
              * @param {DimensionDef.java} dimension
+             * @param {{ ignoreLabels?: boolean, initialExtent?: [number, number] }} extraOptions
              * @return {Object} extent as {values: [], min: min, max: max}
              */
-            getAxisExtent: function(chartData, axisName, dimension, intialExtent = [Infinity, -Infinity], ignoreLabels = new Set()) {
-                const labels = chartData.getAxisLabels(axisName);
-                let min = intialExtent[0];
-                let max = intialExtent[1];
+            getAxisExtent: function(chartData, axisName, dimension, extraOptions = {}) {
+                const ignoreLabels = extraOptions.ignoreLabels || new Set();
+                const initialExtent = extraOptions.initialExtent || [Infinity, -Infinity];
+                const labels = chartData.getCustomAxisLabels(axisName) ?? chartData.getAxisLabels(axisName);
+                const isDualAxis = !!chartData.hasDualDimensionAxis?.(axisName);
+                let min = initialExtent[0];
+                let max = initialExtent[1];
 
                 const values = labels.filter(label => !ignoreLabels.has(label.label)).map(function(label) {
+                    if (isDualAxis) {
+                        return label.label; // for now dual axis is used only in waterfall, and values are already merged at this point, so treat them as alphanum values
+                    }
                     if (ChartDimension.isTimelineable(dimension)) {
                         min = Math.min(min, label.min);
                         max = Math.max(max, label.max);
@@ -348,7 +384,7 @@
              * @return {Object} extent as {values: [], min: min, max: max}
              */
             getUnaggregatedAxisExtentByType: function(uaDimensionType, axisData, afterFilterRecords) {
-                switch(uaDimensionType) {
+                switch (uaDimensionType) {
                     case 'str':
                         return {
                             values: angular.copy(axisData.str.sortedMapping)
@@ -466,7 +502,7 @@
                 if (chartType === CHART_TYPES.SCATTER_MULTIPLE_PAIRS) {
                     return svc.getScatterMPRecordsSummary(axisPairs, visibleCountMap);
                 } else {
-                    return `<strong>${ afterFilterRecords }</strong> ${translate('CHARTS.HEADER.RECORDS_REMAIN', 'records remain after applying filtering')}`;
+                    return `<strong>${afterFilterRecords}</strong> ${translate('CHARTS.HEADER.RECORDS_REMAIN', 'records remain after applying filtering')}`;
                 }
             },
 
@@ -501,7 +537,16 @@
                 }
                 return dateDisplayUnit;
             },
-            isPivotRequestAborted:  (errorData) => errorData && errorData.hasResult && errorData.aborted
+            isSameDay(minTimestamp, maxTimestamp) {
+                if (minTimestamp === undefined || maxTimestamp === undefined) {
+                    return false;
+                }
+
+                const minDateWrapper = moment.utc(minTimestamp);
+                const maxDateWrapper = moment.utc(maxTimestamp);
+                return minDateWrapper.year() === maxDateWrapper.year() && minDateWrapper.dayOfYear() === maxDateWrapper.dayOfYear();
+            },
+            isPivotRequestAborted: (errorData) => errorData && errorData.hasResult && errorData.aborted
         };
 
         return svc;
