@@ -6,7 +6,7 @@
     /**
      * Generic helpers for svg manipulation
      */
-    app.factory('SVGUtils', function(D3ChartAxes, ColorUtils, ValuesInChartOverlappingStrategy, MATCH_CHART_COLOR, ChartFeatures, CHART_COLOR_AUTO) {
+    app.factory('SVGUtils', function(D3ChartAxes, ColorUtils, ValuesInChartOverlappingStrategy, MATCH_CHART_COLOR, ChartFeatures, CHART_COLOR_AUTO, ChartColorUtils) {
 
         /**
          * Quadtree-based label collision detection handler for a chart.
@@ -166,18 +166,18 @@
 
             },
 
-            drawLabels(drawContext, chartPrefix) {
+            drawLabels(drawContext, chartPrefix, hasColorDim = false) {
                 // first draw final labels
-                const labelGroups = this.drawLabelsRaw(drawContext, chartPrefix, false);
+                const labelGroups = this.drawLabelsRaw(drawContext, chartPrefix, false, hasColorDim);
 
                 // then draw alternate labels, without overlapping detection with other measures
-                this.drawLabelsRaw(drawContext, chartPrefix, true);
+                this.drawLabelsRaw(drawContext, chartPrefix, true, hasColorDim);
 
                 // both "drawLabelsRaw" calls return the same selection, as alternate labels are drawn in the same label group as the main ones
                 return labelGroups;
             },
 
-            drawLabelsRaw(drawContext, chartPrefix, onHoverMode = false) {
+            drawLabelsRaw(drawContext, chartPrefix, onHoverMode = false, hasColorDim = false) {
 
                 if (!drawContext.overlappingStrategy) {
                     return;
@@ -237,7 +237,7 @@
                     .attr('fill', function(d) {
                         const formattingOptions = d.valuesInChartDisplayOptions.textFormatting;
 
-                        const chartColor = drawContext.colorScale(d.color + d.measure);
+                        const chartColor = drawContext.colorScale(ChartColorUtils.getColorIndex(d, hasColorDim));
                         const backgroundColor = drawContext.hasBackground ? chartColor : '#fff';
 
                         let fontColor = computeMatchChartColor(formattingOptions.fontColor, chartColor, drawContext.opacity);
@@ -299,7 +299,8 @@
                             d3.select(this).attr('visibility', null);
                         }
                     })
-                    .transition()
+                    .interrupt('updateLabels')
+                    .transition('updateLabels')
                     .duration(200)
                     .ease('easeOutQuad')
                     .attr('x', function(d, i) {
@@ -312,6 +313,9 @@
                         let y = drawContext.getLabelYPosition(d, i);
                         // but as we use the dominant-baseline "middle", we have to adjust the y position passed to the svg element
                         const firstSubText = d.textsElements[0];
+                        if (!firstSubText) {
+                            return y;
+                        }
                         const firstSubTextHeight = firstSubText.height || 0;
                         const firstSubTextFontSize = firstSubText.valuesInChartDisplayOptions.textFormatting.fontSize;
                         const offset = (firstSubTextHeight / 2 + (firstSubTextHeight - firstSubTextFontSize) / 2);
@@ -332,12 +336,12 @@
                 }
 
                 this.drawLabelsBackground(labelGroups, subTexts, drawContext.getBackgroundXPosition, drawContext.getBackgroundYPosition,
-                    drawContext.backgroundXMargin, drawContext.colorScale, labelClass, drawContext.opacity, onHoverMode);
+                    drawContext.backgroundXMargin, drawContext.colorScale, labelClass, drawContext.opacity, onHoverMode, hasColorDim);
 
                 return labelGroups;
             },
 
-            drawLabelsBackground(labelGroups, subTexts, getBackgroundXPosition, getBackgroundYPosition, xMargin, colorScale, labelClass, opacity, onHoverMode) {
+            drawLabelsBackground(labelGroups, subTexts, getBackgroundXPosition, getBackgroundYPosition, xMargin, colorScale, labelClass, opacity, onHoverMode, hasColorDim) {
 
                 const rectBackgrounds = labelGroups.selectAll(`rect.background-${labelClass}`).data(function(d) {
                     return Array.isArray(d) ? _.flatten(d.map(d => (d.textsElements))) : d.textsElements;
@@ -350,7 +354,7 @@
                     })
                     .attr('fill', function(d) {
                         const formattingOptions = d.valuesInChartDisplayOptions.textFormatting;
-                        const chartColor = colorScale(d.color + d.measure);
+                        const chartColor = colorScale(ChartColorUtils.getColorIndex(d, hasColorDim));
                         return computeMatchChartColor(formattingOptions.backgroundColor, chartColor, opacity);
                     })
                     .style('pointer-events', 'none');
@@ -371,7 +375,8 @@
                             d.hideBackground = false;
                         }
                     })
-                    .transition()
+                    .interrupt('updateLabelsBackground')
+                    .transition('updateLabelsBackground')
                     .duration(200)
                     .ease('easeOutQuad')
                     .attr('x', function(d, i) {
@@ -488,6 +493,22 @@
                     y += textElem.height;
                 }
                 return y;
+            },
+
+            addPlotAreaContextMenuAnchor(chart, plotWidth, plotHeight) {
+                let bg = chart.select('.chart-contextual-menu-anchor');
+
+                if (bg.empty()) {
+                    bg = chart.insert('rect', ':first-child')
+                        .attr('class', 'chart-contextual-menu-anchor')
+                        .attr('x', 0)
+                        .attr('y', 0)
+                        .attr('width', plotWidth)
+                        .attr('height', plotHeight)
+                        .style('fill', 'transparent')
+                        .style('pointer-events', 'all');
+                }
+                return bg;
             }
         };
 
