@@ -4,7 +4,7 @@
 const app = angular.module('dataiku.webapps', []);
 
 app.factory('WebAppSharedState', function () {
-    return {};
+    return {created:new Date()};
 });
 
 app.controller("WebAppsCommonController", function($scope, $state, $rootScope, $stateParams, $q, $controller, TopNav,
@@ -18,6 +18,9 @@ app.controller("WebAppsCommonController", function($scope, $state, $rootScope, $
      * are below this one */
     $scope.sharedState = WebAppSharedState;
 
+    $scope.projectKey = $stateParams.projectKey;
+    $scope.webAppId = $stateParams.webAppId;
+
     $scope.setupTypeSpecificWebAppBehaviour = function() {
         const baseType = $scope.getBaseType($scope.app.type);
         if (baseType == 'STANDARD') {
@@ -28,6 +31,8 @@ app.controller("WebAppsCommonController", function($scope, $state, $rootScope, $
             $controller("DashWebAppController", {$scope});
         } else if (baseType == 'SHINY') {
             $controller("ShinyWebAppController", {$scope});
+        } else if (baseType == 'STREAMLIT') {
+            $controller("StreamlitWebAppController", {$scope});
         } else if (baseType == 'CODE_STUDIO_AS_WEBAPP') {
             $controller("CodeStudioWebAppController", {$scope});
         } else {
@@ -85,7 +90,11 @@ app.controller("WebAppsCommonController", function($scope, $state, $rootScope, $
     };
 
     $scope.backendEnabled = function () {
-        return $scope.app && ($scope.app.params.backendEnabled || ['BOKEH', 'DASH', 'SHINY', 'CODE_STUDIO_AS_WEBAPP'].indexOf($scope.app.type) >= 0);
+        return $scope.app && ($scope.app.params.backendEnabled || ['BOKEH', 'DASH', 'SHINY', 'STREAMLIT', 'CODE_STUDIO_AS_WEBAPP'].includes($scope.app.type));
+    };
+
+    $scope.refreshLogs = function(updateCurrent) {
+        $rootScope.$broadcast(!!updateCurrent ? 'refreshRunLogsAndUpdateCurrent' : 'refreshRunLogs');
     };
 
     $scope.setBackendLogs = function(data) {
@@ -107,13 +116,20 @@ app.controller("WebAppsCommonController", function($scope, $state, $rootScope, $
             $scope.errorsInLogs = false;
             $scope.warningsInLogs = false;
         }
+        $scope.refreshLogs(true);
     };
 
     $scope.setBackendState = function(data) {
         $scope.sharedState.backendState = data;
-        $scope.sharedState.backendRunning = !!(data && data.futureInfo && data.futureInfo.alive);
+        if (data == null) {
+            $scope.sharedState.backendRunning = null;
+        } else if (data.futureInfo == null || !data.futureInfo.alive) {
+            $scope.sharedState.backendRunning = 'stopped';
+        } else {
+            $scope.sharedState.backendRunning = 'started';
+        }
         if ($scope.sharedState.backendState) {
-            if ($scope.backendEnabled() && !$scope.sharedState.backendRunning) {
+            if ($scope.backendEnabled() && $scope.sharedState.backendRunning !== 'started') {
                 ActivityIndicator.error("Backend not running");
             } else {
                 $scope.$broadcast("previewDataUpdated");
@@ -122,8 +138,8 @@ app.controller("WebAppsCommonController", function($scope, $state, $rootScope, $
             $scope.$broadcast("previewDataUpdated");
         }
     };
-    
-    const editableTypes = ['BOKEH', 'DASH', 'SHINY', 'STANDARD'];
+
+    const editableTypes = ['BOKEH', 'DASH', 'SHINY', 'STREAMLIT', 'STANDARD'];
     $scope.isEditableWebAppType = function(webappType) {
         return editableTypes.includes(webappType);
     };
@@ -143,19 +159,24 @@ app.controller("WebAppsCommonController", function($scope, $state, $rootScope, $
         $scope.setBackendLogs(data);
         ActivityIndicator.error("Backend start failed (check logs)");
         $scope.sharedState.backendFuture = null;
-        $scope.sharedState.backendFatalError = getErrorDetails(data, status, headers);
-        $scope.sharedState.backendRunning = false;
+        let fatalError = getErrorDetails(data, status, headers);
+        $scope.sharedState.backendFatalError = fatalError;
+        if (fatalError && fatalError.errorType && fatalError.errorType.indexOf("BackendStartFailedException") >= 0) {
+            $scope.sharedState.backendRunning = 'stopped'; // we know the backend gave up starting it
+        } else {
+            $scope.sharedState.backendRunning = 'unknown'; // not 'stopped' because it could be still starting
+        }
         deferred.reject();
     };
     $scope.restartSuccess = function(data, deferred) {
         $scope.setBackendLogs(data.result);
         $scope.setBackendState(data.result);
         $scope.sharedState.backendFuture = null;
-        $scope.sharedState.backendRunning = true;
+        $scope.sharedState.backendRunning = 'started';
         $rootScope.$broadcast("previewDataUpdated");
         deferred.resolve();
     };
-    
+
     $scope.handleStartFuture = function(result, deferred) {
         $scope.sharedState.backendFuture = result;
 
@@ -173,13 +194,13 @@ app.controller("WebAppsCommonController", function($scope, $state, $rootScope, $
             });
         }
     };
-    
+
     $scope.start = function(app){
-        if ($scope.sharedState.backendRunning == null) {
+        if ($scope.sharedState.backendRunning == 'starting') {
             throw new Error("webapp already starting");
         }
-        $scope.sharedState.backendRunning = null;
-        
+        $scope.sharedState.backendRunning = 'starting';
+
         WT1.event("webapp-start", {webAppId: $stateParams.id, type: app.type});
         const deferred = $q.defer();
 
@@ -200,7 +221,6 @@ app.controller("WebAppsCommonController", function($scope, $state, $rootScope, $
                     $scope.restartError(a,b,c, deferred);
                 });
             }
-            $rootScope.$broadcast('backendRestarted');
         }).error(function(a,b,c) {
             $scope.restartError(a,b,c, deferred);
         });
@@ -210,24 +230,26 @@ app.controller("WebAppsCommonController", function($scope, $state, $rootScope, $
 
 
     $scope.stop = function(webapp) {
-        if ($scope.sharedState.backendRunning == null) {
+        if ($scope.sharedState.backendRunning == 'starting') {
             throw new Error("webapp already starting");
         }
-        $scope.sharedState.backendRunning = null;
-        
+        $scope.sharedState.backendRunning = 'starting';
+
         WT1.event("webapp-stop", {webAppId: $stateParams.id, type: webapp.type});
         return DataikuAPI.webapps.stopBackend(webapp)
             .success(function(){
-                $scope.sharedState.backendRunning = false;
-                $rootScope.$broadcast('backendStopped');
+                $scope.sharedState.backendRunning = 'stopped';
+                $scope.refreshLogs(true);
             })
             .error(setErrorInScope.bind($scope));
     };
 
     $scope.stopBackendWithDialog = function(webapp) {
         Dialogs.confirm($scope, 'Stop webapp backend', 'Are you sure you want to stop the backend?').then(function () {
-            $scope.stop(webapp).then(function(){
-                $scope.list();
+            $scope.stop(webapp).then(function() {
+                if ($scope.list != null) { // don't list on the webapp page. There is no listing there
+                    $scope.list();
+                }
             });
         });
     };
@@ -253,11 +275,11 @@ app.controller("WebAppsCommonController", function($scope, $state, $rootScope, $
         });
     };
 
-    $scope.editCustomFields = function() {
+    $scope.editCustomFields = function(editingTabIndex = 0) {
         if (!$scope.app) {
             return;
         }
-        let modalScope = angular.extend($scope, {objectType: 'WEB_APP', objectName: $scope.app.name, objectCustomFields: $scope.app.customFields});
+        let modalScope = angular.extend($scope, {objectType: 'WEB_APP', objectName: $scope.app.name, objectCustomFields: $scope.app.customFields, editingTabIndex});
         CreateModalFromTemplate("/templates/taggable-objects/custom-fields-edit-modal.html", modalScope).then(function(customFields) {
             $scope.saveCustomFields(customFields);
         });
@@ -265,8 +287,8 @@ app.controller("WebAppsCommonController", function($scope, $state, $rootScope, $
 });
 
 
-app.controller("WebAppsListController", function($scope, $controller, $stateParams, DataikuAPI, CreateModalFromTemplate,$state,$q, TopNav, Fn, $filter, Dialogs,
-                                                 WT1, CachedAPICalls) {
+app.controller("WebAppsListController", function ($scope, $rootScope, $controller, $stateParams, DataikuAPI, CreateModalFromTemplate,$state,$q, TopNav, Fn, $filter, Dialogs,
+                                                    WT1, CachedAPICalls, WebAppsService, PluginConfigUtils) {
 
     $controller('_TaggableObjectsListPageCommon', {$scope: $scope});
     $controller("WebAppsCommonController", {$scope: $scope});
@@ -306,9 +328,13 @@ app.controller("WebAppsListController", function($scope, $controller, $statePara
     CachedAPICalls.webAppTypes.then(webAppTypes => {
         const descriptors = Object.values(webAppTypes);
         // in the flow, only show webapps with roles
-        $scope.hasVisualWebapps = descriptors.some(descriptor => descriptor.baseType && (descriptor.roles == null || descriptor.roles.length == 0));
+        const visibilityFilter = PluginConfigUtils.shouldComponentBeVisible($rootScope.appConfig.loadedPlugins);
+        $scope.hasVisualWebapps = descriptors
+            .filter(descriptor => descriptor.baseType && (descriptor.roles == null || descriptor.roles.length == 0))
+            .filter(desc => visibilityFilter(WebAppsService.getWebAppLoadedDesc(desc.id)))
+            .length > 0;
     });
-    
+
     // override because there is no meaningfull sharedState
     $scope.stopBackendInList = function(webapp) {
         Dialogs.confirm($scope, 'Stop webapp backend', 'Are you sure you want to stop the backend?').then(function () {
@@ -326,140 +352,8 @@ app.controller("WebAppsListController", function($scope, $controller, $statePara
     }
 });
 
-
-
-/**
- * Temporary: it's in this file by laziness / quick fix for 13.4.0 -- Should be moved to a proper file
- */
-app.controller("DataikuAnswersListController", function($scope, $controller, $stateParams, DataikuAPI, CreateModalFromTemplate,$state,$q, TopNav, Fn, $filter, Dialogs, WT1, PluginsService, AppConfigService) {
-    
-    $controller('_TaggableObjectsListPageCommon', {$scope: $scope});
-    $controller("WebAppsCommonController", {$scope: $scope});
-
-    const answersPluginId = "document-question-answering";
-    const answersComponentId = "webapp_document-question-answering_document-intelligence-explorer"
-    const portalPluginId = "agent-connect";
-    const portalComponentId = "webapp_agent-connect_portal";
-
-    $scope.answersComponentId = answersComponentId;
-    $scope.portalComponentId = portalComponentId;
-
-
-    $scope.listHeads = function(projectKey, tagsFilter) {
-        const deferred = $q.defer();
-        deferred.promise.success = function(callback) {
-            deferred.promise.then( (resp) => {return callback(resp.data, resp.status, resp.headers, resp.config, resp.statusText, resp.xhrStatus)});
-            return deferred.promise;
-        };
-        deferred.promise.error = function(callback) {
-            deferred.promise.then(null, resp => callback(resp.data, resp.status, resp.headers, resp.config, resp.statusText, resp.xhrStatus));
-            return deferred.promise;
-        };
-        DataikuAPI.webapps.listHeads(projectKey, tagsFilter).success(function(data) {
-            data.items = data.items.filter(w => (w.type == answersComponentId || w.type == portalComponentId));
-            deferred.resolve({"data": data})
-        }).error(setErrorInScope.bind($scope));
-        return deferred.promise;
-    }
-
-    PluginsService.checkInstalledStatus(answersPluginId).then(isPluginInstalled => {
-        $scope.pluginUrl = PluginsService.getPluginDetailsLink(answersPluginId, isPluginInstalled, AppConfigService.isDeploymentCloud());
-        $scope.answersPluginInstalled = isPluginInstalled;
-    }).catch(setErrorInScope.bind($scope));
-     PluginsService.checkInstalledStatus(portalPluginId).then(isPluginInstalled => {
-        $scope.pluginUrl = PluginsService.getPluginDetailsLink(portalPluginId, isPluginInstalled, AppConfigService.isDeploymentCloud());
-        $scope.portalPluginInstalled = isPluginInstalled;
-    }).catch(setErrorInScope.bind($scope));
-
-    $scope.showCreateAnswersModal = function(webappCategory, loadedWebApp, targetKey, targetValue, defaultName) {
-        $scope.webappCategory = "visual";
-        const templateName = '/templates/webapps/new-answers-webapp-modal.html';
-
-        CreateModalFromTemplate(templateName, $scope, null, function(modalScope) {
-            modalScope.preselectedType = answersComponentId;
-            modalScope.app.name = defaultName;
-            modalScope.app.configFromRole = modalScope.app.configFromRole || {};
-            modalScope.app.configFromRole[targetKey] = targetValue;
-            modalScope.app.config = modalScope.app.config || {};
-            modalScope.app.config[targetKey] = targetValue;
-
-        }).then(function(webapp) {
-            if (webapp.backendReadyOrNoBackend) {
-                // backend up and running, go directly to view
-                $state.go("projects.project.webapps.webapp.view", {projectKey : $stateParams.projectKey, webAppId: webapp.id, webAppName: $filter('slugify')(webapp.name)});
-            } else {
-                $state.go("projects.project.webapps.webapp.edit", {projectKey : $stateParams.projectKey, webAppId: webapp.id, webAppName: $filter('slugify')(webapp.name)});
-            }
-        });
-    }
-     $scope.showCreatePortalModal = function(webappCategory, loadedWebApp, targetKey, targetValue, defaultName) {
-        $scope.webappCategory = "visual";
-        const templateName = '/templates/webapps/new-portal-webapp-modal.html';
-
-        CreateModalFromTemplate(templateName, $scope, null, function(modalScope) {
-            modalScope.preselectedType = portalComponentId;
-            modalScope.app.name = defaultName;
-            modalScope.app.configFromRole = modalScope.app.configFromRole || {};
-            modalScope.app.configFromRole[targetKey] = targetValue;
-            modalScope.app.config = modalScope.app.config || {};
-            modalScope.app.config[targetKey] = targetValue;
-
-        }).then(function(webapp) {
-            if (webapp.backendReadyOrNoBackend) {
-                // backend up and running, go directly to view
-                $state.go("projects.project.webapps.webapp.view", {projectKey : $stateParams.projectKey, webAppId: webapp.id, webAppName: $filter('slugify')(webapp.name)});
-            } else {
-                $state.go("projects.project.webapps.webapp.edit", {projectKey : $stateParams.projectKey, webAppId: webapp.id, webAppName: $filter('slugify')(webapp.name)});
-            }
-        });
-    }
-
-
-    $scope.sortBy = [
-        { value: 'name', label: 'Name' },
-        { value: '-lastModifiedOn', label: 'Last modified' }
-    ];
-    $scope.selection = $.extend({
-        filterQuery: {
-            userQuery: '',
-            tags: [],
-            interest: {
-                starred: '',
-            },
-        },
-        filterParams: {
-            userQueryTargets: ["name","tags"],
-            propertyRules: {tag: "tags"},
-        },
-        orderQuery: "-lastModifiedOn",
-        orderReversed: false,
-    }, $scope.selection || {});
-
-    $scope.sortCookieKey = 'webapps';
-    $scope.maxItems = 20;
-
-
-    TopNav.setLocation("DATAIKU_ANSWERS", 'dataiku answers', TopNav.TABS_NONE, null);
-    TopNav.setNoItem();
-    $scope.list();
-
-    // override because there is no meaningfull sharedState
-    $scope.stopBackendInList = function(webapp) {
-        Dialogs.confirm($scope, 'Stop webapp backend', 'Are you sure you want to stop the backend?').then(function () {
-            WT1.event("webapp-stop", {webAppId: $stateParams.id, type: webapp.type});
-            return DataikuAPI.webapps.stopBackend(webapp)
-                .success(function(){
-                    $scope.list();
-                })
-                .error(setErrorInScope.bind($scope));
-        });
-    };
-});
-
-
-
-
-app.controller("WebAppCoreController", function($scope, $stateParams, $filter, $state, $controller, $timeout, $q, FutureWatcher, WT1, DKUtils, DataikuAPI, TopNav, WebAppsService, Assert, $interval, Notification, WebappContextService) {
+app.controller("WebAppCoreController", function($scope, $stateParams, $filter, $state, $controller, $timeout, $q, FutureWatcher, WT1, DKUtils, DataikuAPI,
+                                                TopNav, WebAppsService, Assert, $interval, Notification, WebappContextService, CodeStudiosService) {
 
     $controller("WebAppsCommonController", {$scope: $scope});
 
@@ -478,7 +372,7 @@ app.controller("WebAppCoreController", function($scope, $stateParams, $filter, $
 
             $scope.$watch("app.name", function(nv) {
                 if (!nv) return;
-                $state.go($state.current, {webAppName: $filter('slugify')(nv)}, {location: true, inherit:true, notify:false, reload:false});
+                $state.go($state.current, {webAppName: $filter('slugify')(nv)}, {location: 'replace', inherit:true, notify:false, reload:false});
             });
 
             $scope.pluginDesc = WebAppsService.getOwnerPluginDesc($scope.app.type);
@@ -498,7 +392,7 @@ app.controller("WebAppCoreController", function($scope, $stateParams, $filter, $
 
         $scope.$watch("app.name", function(nv) {
             if (!nv) return;
-            $state.go($state.current, {webAppName: $filter('slugify')(nv)}, {location: true, inherit:true, notify:false, reload:false});
+            $state.go($state.current, {webAppName: $filter('slugify')(nv)}, {location: 'replace', inherit:true, notify:false, reload:false});
         });
     }).then(function() {
         if ($scope.app && $scope.app.isVirtual) {
@@ -529,6 +423,17 @@ app.controller("WebAppCoreController", function($scope, $stateParams, $filter, $
         })).then(getSummary).then($scope.startUpdatePreview);
     }
 
+    $scope.canEditWebAppInCodeStudio = false;
+    DataikuAPI.codeStudioTemplates.canEditInCodeStudio($stateParams.projectKey).success(function(data) {
+        $scope.canEditWebAppInCodeStudio = data.canEdit;
+    })
+
+    $scope.editWebAppInCodeStudio = function() {
+        $scope.saveWebAppWithCode(null, false).then(function() {
+            CodeStudiosService.editFileInCodeStudio($scope, "webapps",  $stateParams.webAppId);
+        }).catch(setErrorInScope.bind($scope));
+    }
+
     $scope.saveWebAppWithCode = function(commitMessage, forceRestartBackend) {
         WT1.event("webapp-save", {webAppId: $stateParams.id, type: $scope.app.type});
         var deferred = $q.defer();
@@ -543,6 +448,7 @@ app.controller("WebAppCoreController", function($scope, $stateParams, $filter, $
             if (result.backendState == null) {
                 // no backend, just update preview
                 $scope.$broadcast("previewDataUpdated");
+                deferred.resolve();
             } else {
                 $scope.handleStartFuture(result.backendState, deferred);
             }
@@ -574,9 +480,6 @@ app.controller("WebAppCoreController", function($scope, $stateParams, $filter, $
     $scope.getBackendLogURL = function(app) {
         return DataikuAPI.webapps.getBackendLogURL(app.projectKey, app.id);
     };
-    $scope.getBackendDiagnosis = function(app) {
-        downloadURL(DataikuAPI.webapps.getBackendDiagnosisURL(app.projectKey, app.id));
-    };
 });
 
 app.controller("WebAppPageRightColumnActions", async function($controller, $scope, $q, $rootScope, $stateParams, ActiveProjectKey, DataikuAPI, WT1) {
@@ -611,14 +514,9 @@ app.controller("WebAppPageRightColumnActions", async function($controller, $scop
 
     const interestsListener = $rootScope.$on('userInterestsUpdated', $scope.updateUserInterests);
 
-    const backendRestartListener = $rootScope.$on('backendRestarted',function () {$scope.sharedState.backendRunning = true;});
-    const backendStopListener = $rootScope.$on('backendStopped',function () {$scope.sharedState.backendRunning = false;});
-
     $scope.$on("$destroy", function() {
-            interestsListener();
-            backendRestartListener();
-            backendStopListener();
-        });
+        interestsListener();
+    });
 
 });
 
@@ -688,8 +586,29 @@ app.controller("NewWebAppModalController", function($scope, $rootScope, $window,
             }
             return 0;
         }
-        $scope.codeWebApps = descriptors.filter(d => !d.baseType).sort(f).reverse();
-        $scope.visualWebApps = descriptors.filter(d => d.baseType && (d.roles == null || d.roles.length == 0)).sort(f);
+
+        // tweak the order to have
+        // 1. Standard
+        // 2. Shiny (R)
+        // 3... other web app types (Python) in alphabetical order
+        let codeWebAppDescriptions = [];
+        if (webAppTypes["STANDARD"]) {
+            codeWebAppDescriptions.push(webAppTypes["STANDARD"]);
+        }
+        if (webAppTypes["SHINY"]) {
+            codeWebAppDescriptions.push(webAppTypes["SHINY"]);
+        }
+        codeWebAppDescriptions = codeWebAppDescriptions.concat(descriptors
+            .filter(d => !d.baseType)
+            .filter(d => !["SHINY", "STANDARD"].includes(d.id))
+            .sort(f));
+        $scope.codeWebApps = codeWebAppDescriptions;
+
+        const visibilityFilter = PluginConfigUtils.shouldComponentBeVisible($rootScope.appConfig.loadedPlugins);
+        $scope.visualWebApps = descriptors
+            .filter(d => d.baseType && (d.roles == null || d.roles.length == 0))
+            .filter(desc => visibilityFilter(WebAppsService.getWebAppLoadedDesc(desc.id)))
+            .sort(f);
 
         if ($scope.webappCategory === 'visual' && (!$scope.app || !$scope.app.type) && $scope.visualWebApps.length > 0) {
             $scope.chooseType($scope.visualWebApps[0].id);
@@ -745,6 +664,8 @@ app.controller("NewWebAppModalController", function($scope, $rootScope, $window,
             $controller("DashWebAppController", {$scope: $scope});
         } else if (type == 'SHINY') {
             $controller("ShinyWebAppController", {$scope: $scope});
+        } else if (type == 'STREAMLIT') {
+            $controller("StreamlitWebAppController", {$scope: $scope});
         } else if (type == 'CODE_STUDIO_AS_WEBAPP') {
             $controller("CodeStudioWebAppController", {$scope: $scope});
         }
@@ -759,13 +680,9 @@ app.controller("NewWebAppModalController", function($scope, $rootScope, $window,
             PluginConfigUtils.setDefaultValues($scope.desc.params, $scope.app.config);
         }
 
-        DataikuAPI.webapps.listTemplates(type).success(function(data) {
-            $scope.availableTemplates = data.templates.filter(function (template) {
-                var plugin = Array.dkuFindFn($rootScope.appConfig.loadedPlugins, function (n) {
-                    return n.id == template.pluginId;
-                });
-                return plugin == null || !plugin.hideComponents;
-            });
+        DataikuAPI.webapps.listTemplates(type).success(function (data) {
+            const visibilityFilter = PluginConfigUtils.shouldComponentBeVisible($rootScope.appConfig.loadedPlugins);
+            $scope.availableTemplates = data.templates.filter(visibilityFilter);
             $scope.app.template = $scope.availableTemplates[0];
         }).error(setErrorInScope.bind($scope));
 
@@ -834,7 +751,7 @@ app.controller("WebAppLogsController", function($scope, $stateParams, $timeout,
 
 app.controller("_PreviewWebAppController", function($scope, $q, $sce, ActivityIndicator, DataikuAPI, Logger, $rootScope) {
     $scope.startUpdatePreview = function() {
-        if ($scope.backendEnabled() && $scope.sharedState.backendRunning !== true) {
+        if ($scope.backendEnabled() && $scope.sharedState.backendRunning !== 'started') {
             Logger.warn("Not updating preview, backend is not running ...", printStackTrace());
             return;
         }
@@ -863,7 +780,13 @@ app.controller("_PreviewWebAppController", function($scope, $q, $sce, ActivityIn
     function handleBackendStateResult(backendState) {
         if (backendState) {
             if ($scope.backendEnabled()) {
-                $scope.sharedState.backendRunning = !!(backendState.futureInfo && backendState.futureInfo.alive);
+                if (backendState == null) {
+                    $scope.sharedState.backendRunning = null;
+                } else if (backendState.futureInfo == null || !backendState.futureInfo.alive) {
+                    $scope.sharedState.backendRunning = 'stopped';
+                } else {
+                    $scope.sharedState.backendRunning = 'started';
+                }
                 $scope.setBackendLogs(backendState);
                 if (!backendState.futureInfo || !backendState.futureInfo.alive) {
                     ActivityIndicator.error("Backend not running");
@@ -882,7 +805,7 @@ app.controller("_PreviewWebAppController", function($scope, $q, $sce, ActivityIn
 
 app.controller("_BokehDashOrShinyLikeWebAppController", function($scope, $controller, $q, $sce, ActivityIndicator, DataikuAPI, Logger, $rootScope) {
     $controller("_PreviewWebAppController", {$scope:$scope});
-    
+
     $scope.availableTemplates = [
         {
             id : "default",
@@ -1012,6 +935,11 @@ app.directive('webAppInfra', function(DataikuAPI, $stateParams, $controller, Plu
             $scope.$watch("infra.containerSelection", function(nv, ov) {
                 if (!angular.equals(ov, nv)) fetchInfo()
             }, true);
+            $scope.canDisplayAdvancedSettings = function() {
+                let isK8s = $scope.containerInfo?.type === 'KUBERNETES';
+                let isDocker = $scope.containerInfo?.type === 'DOCKER';
+                return (isK8s || isDocker) && !$scope.isVirtualWebappSettings;
+            }
         }
     }
 });
@@ -1093,7 +1021,7 @@ app.directive('serviceExposition', function(DataikuAPI, $stateParams, $controlle
                 }
             };
             $scope.$watch("$containerInfo", fixupExposition, true);
- 
+
             // watch for changes in the $exposition to set the type
             $scope.$watch("exposition.type", function(nv, ov) {
                 if ($scope.exposition && $scope.exposition.type) {
@@ -1111,7 +1039,7 @@ app.directive('serviceExposition', function(DataikuAPI, $stateParams, $controlle
         }
     }
 });
-   
+
 app.directive('deploymentHpa', function(DataikuAPI, $stateParams, $controller, PluginConfigUtils, $timeout){
     return {
         templateUrl :'/templates/webapps/deployment-hpa.html',
@@ -1167,9 +1095,9 @@ app.directive('yamlModifier', function(DataikuAPI, $stateParams, $controller, Pl
         },
         link : function($scope, $element, attrs) {
             $controller('FetchContainerInfoController', {$scope});
-            
+
             $scope.uiState = {runnableType:null, runnableTypes:[{runnableType:null, desc:{meta:{label:'None'}, params:[]}}]};
-            
+
             let fixupRunnableType = function() {
                 // clear the runnable type if it's not valid
                 if ($scope.$containerInfo && $scope.yamlModifier && $scope.yamlModifier.runnableType) {
@@ -1182,7 +1110,7 @@ app.directive('yamlModifier', function(DataikuAPI, $stateParams, $controller, Pl
                     }
                 }
             };
-            
+
             $scope.$watch('$containerInfo', function() {
                 if ($scope.$containerInfo) {
                     $scope.uiState.runnableTypes.splice(1, $scope.uiState.runnableTypes.length - 1);
@@ -1192,11 +1120,11 @@ app.directive('yamlModifier', function(DataikuAPI, $stateParams, $controller, Pl
                 }
                 fixupRunnableType();
             });
-            
+
             let setRunnableTypeFromRunnable = function() {
                 $scope.uiState.runnableType = $scope.yamlModifier.runnableType;
             };
-            
+
             let setRunnableTypeFromUI = function() {
                 let choice = $scope.uiState.runnableTypes.filter(function(e) {return e.runnableType == $scope.uiState.runnableType})[0];
                 if (!choice) return;
@@ -1204,7 +1132,7 @@ app.directive('yamlModifier', function(DataikuAPI, $stateParams, $controller, Pl
                 $scope.yamlModifier.runnableType = choice.runnableType;
             };
             $scope.$watch('uiState.runnableType', setRunnableTypeFromUI);
-            
+
             let init = function() {
                 if (!$scope.yamlModifier.config) {
                     $scope.yamlModifier.config = {};
@@ -1240,6 +1168,6 @@ app.directive('yamlModifier', function(DataikuAPI, $stateParams, $controller, Pl
         }
     }
 });
-   
+
 
 })();
