@@ -61,6 +61,12 @@
         this.generateGrid();
 
         this.positionPositionlessElements();
+
+        this._sortItemsByPosition();
+
+        for (const item of this.items) {
+            this._resolveCollisions(item);
+        }
     };
 
     GridList.cloneItems = function(sourceItems, destinationItems) {
@@ -147,6 +153,16 @@
             }
         },
 
+        toggleGapMode(allowDirectionalGaps) {
+            if (allowDirectionalGaps === this._options.allowDirectionalGaps) {
+                return;
+            }
+            this._options.allowDirectionalGaps = allowDirectionalGaps;
+            if (!this._options.allowDirectionalGaps) {
+                this.fillDirectionalGaps();
+            }
+        },
+
         resizeGrid: function(lanes) {
             if (lanes === this._options.lanes) {
                 return;
@@ -163,6 +179,9 @@
             this._resetGrid();
             this._sortItemsByPosition();
             this._resolveItemCollisionsAfterLaneChange();
+            if (!this._options.allowDirectionalGaps) {
+                this.fillDirectionalGaps();
+            }
         },
 
         captureItemSizeAndPositionSnapshot: function() {
@@ -199,7 +218,7 @@
         },
 
         findPositionForItem: function(item, start, fixedRow) {
-            /**
+            /*
              * This method has two options for the position we want for the item:
              * - Starting from a certain row/column number and only looking for
              *   positions to its right
@@ -258,15 +277,18 @@
         },
 
         moveItemToPosition: function(item, newPosition) {
+            const previousItemPosition = this._getItemPosition(item);
             const position = this._getItemPosition({
                 x: newPosition[0],
                 y: newPosition[1],
                 w: item.w,
                 h: item.h
             });
-
             this._updateItemPosition(item, [position.x, position.y]);
-            this._resolveCollisions(item);
+            this._resolveCollisions(item, previousItemPosition);
+            if (!this._options.allowDirectionalGaps) {
+                this.fillDirectionalGaps();
+            }
         },
 
         moveAndResizeItem: function(item, newItem) {
@@ -278,9 +300,14 @@
             });
 
             this._updateItemPosition(item, [position.x, position.y]);
+
             this._updateItemSize(item, position.w, position.h);
 
             this._resolveCollisions(item);
+
+            if (!this._options.allowDirectionalGaps) {
+                this.fillDirectionalGaps();
+            }
         },
 
         resizeItem: function(item, size) {
@@ -299,18 +326,42 @@
             this._updateItemSize(item, width, height);
 
             this._resolveCollisions(item);
+            if (!this._options.allowDirectionalGaps) {
+                this.fillDirectionalGaps();
+            }
         },
 
         deleteItem: function(item) {
             const index = this._getItemIndex(item);
 
-            if (index > -1) {
-                if (this._isANumber(item.x) && this._isANumber(item.y)) {
-                    this._deleteItemPositionFromGrid(item);
-                }
-
-                this.items.splice(index, 1);
+            if (index <= -1) {
+                return;
             }
+
+            if (this._isANumber(item.x) && this._isANumber(item.y)) {
+                this._deleteItemPositionFromGrid(item);
+            }
+
+            this.items.splice(index, 1);
+
+            if (!this._options.allowDirectionalGaps) {
+                this.fillDirectionalGaps();
+                return;
+            }
+
+            if (item.isDisplacing) {
+                const itemsUnder = this.getItemsBelowOrOverlapping(item);
+                this._shiftItems(itemsUnder, -item.h);
+            }
+
+        },
+
+        getItemsBelowOrOverlapping: function(item) {
+            this._sortItemsByPosition();
+            return this.items.filter(item_ => {
+                const isBelow = this._isBelowItem(item, item_);
+                return item_.id !== item.id && isBelow;
+            });
         },
 
         addItem: function(item) {
@@ -319,9 +370,84 @@
             }
             this.items.push(item);
             this.positionPositionlessElements();
+            if (!this._options.allowDirectionalGaps) {
+                this.fillDirectionalGaps();
+            }
+        },
+
+        fillDirectionalGaps: function() {
+            this._sortItemsByPosition();
+            for (const item of this.items) {
+                const position = this._getItemPosition(item);
+                const newPosition = this._findPositionForItemFillingHorizontalGap(item, position);
+                if (position.x !== newPosition[0]) {
+                    this._updateItemPosition(item, newPosition);
+                }
+            }
+            this._removeOverflowingColumns();
+        },
+
+        hasDirectionalGaps: function() {
+            this._sortItemsByPosition();
+            for (const item of this.items) {
+                const position = this._getItemPosition(item);
+                const newPosition = this._findPositionForItemFillingHorizontalGap(item, position);
+                if (position.x !== newPosition[0]) {
+                    return true;
+                }
+            }
+            return false;
+        },
+
+        compactItems: function() {
+            this.fillDirectionalGaps();
+            this._sortItemsByPosition();
+            this._resetGrid();
+
+            // Locked items must be positioned first if allowing gaps
+            if (this._options.allowDirectionalGaps) {
+                for (const item of this.items) {
+                    if (item.locked) {
+                        const lockedPosition = this._getItemPosition(item);
+                        this._updateItemPosition(item, [lockedPosition.x, lockedPosition.y]);
+                    }
+                }
+            }
+
+            for (const item of this.items) {
+                if (item.locked && this._options.allowDirectionalGaps) {
+                    continue;
+                }
+                // Resolve position for unlocked items
+                const newPosition = this.findPositionForItem(item, { x: 0 , y: 0 });
+                this._updateItemPosition(item, newPosition);
+            }
+        },
+
+        _solveCollisionDisplacingItem(newItem) {
+            const itemsUnder = this.getItemsBelowOrOverlapping(newItem);
+            const maxOffset = itemsUnder.reduce((offset, itemUnder) => {
+                return Math.max(
+                    Math.max(
+                        newItem.y - itemUnder.y,
+                        0
+                    ) + newItem.h,
+                    offset
+                );
+            }, 0);
+
+            this._shiftItems(itemsUnder, maxOffset);
+            const newPosition = this._getItemPosition(newItem);
+            // Set the new item at its desired position
+            this._updateItemPosition(newItem, [newPosition.x, newPosition.y]);
         },
 
         solveCollisionWithNewItemPositionning(newItem){
+            if (newItem.isDisplacing) {
+                this._solveCollisionDisplacingItem(newItem);
+                return;
+            }
+
             // If new item is colliding with a locked item, we move the new item down
             for (const item of this.items) {
                 if (item.locked && this._itemsAreColliding(newItem, item)) {
@@ -330,13 +456,18 @@
                     return;
                 };
             }
-            // If new item is colliding with an unlocked item, we move the unlock items down
+
+            /*
+             * If new item is colliding with an unlocked item or the new item is colliding with a lock item and has overrideLockItems,
+             * we move the unlock items down
+             */
             for (const item of this.items) {
                 if (!item.locked && this._itemsAreColliding(newItem, item)) {
                     this._deleteItemPositionFromGrid(item);
                     this._setItemPosition(item, [undefined, undefined]);
                 }
             }
+
             // Set the new item at its desired position
             const position = this._getItemPosition(newItem);
             this._updateItemPosition(newItem, [position.x, position.y]);
@@ -390,6 +521,14 @@
                         return position1.y - position2.y;
                     }
 
+                    if (item1.isDisplacing && !item2.isDisplacing) {
+                        return -1;
+                    }
+
+                    if (!item1.isDisplacing && item2.isDisplacing) {
+                        return 1;
+                    }
+
                     // The items are placed on the same position.
                     return 0;
                 }.bind(this)
@@ -474,13 +613,17 @@
             });
         },
 
-        _shiftAllItems: function(dx = 0, dy = 0) {
-            this.items.forEach(item => {
+        _shiftItems: function(items, dx = 0, dy = 0) {
+            items.forEach(item => {
                 const position = this._getItemPosition(item);
-                this._setItemPosition(item, [position.x + dx, position.y + dy ]);
+                this._updateItemPosition(item, [position.x + dx, position.y + dy]);
             });
         },
 
+        _shiftAllItems: function(dx = 0, dy = 0) {
+            this._shiftItems(this.items, dx, dy);
+        },
+
         _hasOverflowingItems: function() {
             return this.items.some(item => this._isItemOverflowing(item));
         },
@@ -612,9 +755,16 @@
                 this._deleteItemPositionFromGrid(item);
             }
 
+            const initialHeight = item.h;
+
             item.w = width;
             item.h = height;
 
+            if (item.isDisplacing) {
+                const itemsUnder = this.getItemsBelowOrOverlapping(item);
+                this._shiftItems(itemsUnder, (height - initialHeight ));
+            }
+
             this._markItemPositionToGrid(item);
         },
 
@@ -677,6 +827,16 @@
             }
         },
 
+        _removeOverflowingColumns: function() {
+            for (let i = this.grid.length - 1; i >= 0; i--) {
+                if (this.grid[i].length === 0 || this.grid[i].every(row => row == null)) {
+                    this.grid.splice(i, 1);
+                } else {
+                    break;
+                }
+            }
+        },
+
         _getItemsCollidingWithItem: function(item) {
             const collidingItems = [];
             for (let i = 0; i < this.items.length; i++) {
@@ -694,6 +854,30 @@
             return this._arePositionsColliding(position1, position2);
         },
 
+        _isBelowItem: function(referenceItem, candidateItem) {
+            const referencePosition = this._getItemPosition(referenceItem);
+            const candidatePosition = this._getItemPosition(candidateItem);
+
+            const candidateIsAtTheLeft = candidatePosition.x + candidatePosition.w > referencePosition.x;
+
+            const candidateTopHasVerticalColliding = (
+                candidatePosition.y + candidatePosition.h - referencePosition.y > 0
+                && referencePosition.y + referencePosition.h - candidatePosition.y - candidatePosition.h > 0
+            );
+
+            const candidateBottomHasVerticalColliding = (
+                referencePosition.y + referencePosition.h - candidatePosition.y > 0
+                && candidatePosition.y + candidatePosition.h - referencePosition.y - referencePosition.h >= 0
+            );
+
+            const candidateHasVerticalColliding = (
+                candidateTopHasVerticalColliding
+                || candidateBottomHasVerticalColliding
+            );
+
+            return candidateIsAtTheLeft && candidateHasVerticalColliding;
+        },
+
         _arePositionsColliding: function(position1, position2) {
             return !(
                 position2.x >= position1.x + position1.w ||
@@ -703,13 +887,41 @@
             );
         },
 
-        _resolveCollisions: function(item) {
-            if (!this._tryToResolveCollisionsLocally(item)) {
+        _resolveCollisions: function(item, previousItemPosition = null) {
+            if (!this._tryToResolveCollisionsLocally(item, previousItemPosition)) {
                 this._pullItemsToLeft(item);
             }
         },
 
-        _tryToResolveCollisionsLocally: function(item) {
+        _tryToSwapWithRightCollidingItemWhenMovingItemRight: function(item, previousItemPosition, collidingItemIndex) {
+            const collidingItemPosition = this._getItemPosition(this.items[collidingItemIndex]);
+            const wasItemOnTheLeftOfCollidingItem = previousItemPosition.x < collidingItemPosition.x;
+            if (!wasItemOnTheLeftOfCollidingItem) {
+                return false;
+            }
+
+            const gridClone = new GridList([], this._options);
+            GridList.cloneItems(this.items, gridClone.items);
+            gridClone.generateGrid();
+
+            const itemIndex = this._getItemIndex(item);
+
+            const itemClone = gridClone.items[itemIndex];
+            const collidingItem = gridClone.items[collidingItemIndex];
+
+            gridClone._updateItemPosition(itemClone, [previousItemPosition.x, previousItemPosition.y]);
+            const itemPosition = gridClone._getItemPosition(itemClone);
+            this._updateItemPosition(collidingItem, [itemPosition.x, collidingItemPosition.y]);
+            this._updateItemPosition(itemClone, [collidingItemPosition.x + collidingItemPosition.w, itemPosition.y]);
+            if (gridClone._getItemsCollidingWithItem(itemClone).length === 0 && gridClone._getItemsCollidingWithItem(collidingItem).length === 0) {
+                GridList.cloneItems(gridClone.items, this.items);
+                this.generateGrid();
+                return true;
+            }
+            return false;
+        },
+
+        _tryToResolveCollisionsLocally: function(item, previousItemPosition = null) {
             /**
              * Attempt to resolve the collisions after moving a an item over one or more
              * other items within the grid, by shifting the position of the colliding
@@ -722,21 +934,36 @@
             if (!collidingItems.length) {
                 return true;
             }
-            const _gridList = new GridList([], this._options);
+
+            /*
+             * In non-allowDirectionalGaps mode, items are compacted toward the left (or toward the top in vertical mode).
+             * This makes it difficult to move a tile to the right (or downward in vertical mode) if it immediately
+             * collides with another one.
+             *
+             * When such a collision occurs, and the dragged item comes from the left (or from above in vertical mode),
+             * this method attempts to resolve the collision by swapping the dragged item with the colliding one.
+             * The swap is only performed if it results in a valid grid state with no further collisions in a virtual grid.
+             */
+            if (collidingItems.length === 1 && previousItemPosition != null && !this._options.allowDirectionalGaps) {
+                if (this._tryToSwapWithRightCollidingItemWhenMovingItemRight(item, previousItemPosition, collidingItems[0])){
+                    return true;
+                };
+            }
+
+            const gridClone = new GridList([], this._options);
             const itemIndex = this._getItemIndex(item);
             let leftOfItem;
             let rightOfItem;
             let aboveOfItem;
             let belowOfItem;
 
-            GridList.cloneItems(this.items, _gridList.items);
-            _gridList.generateGrid();
-
-            const itemClone = _gridList.items[itemIndex];
+            GridList.cloneItems(this.items, gridClone.items);
+            gridClone.generateGrid();
 
             for (let i = 0; i < collidingItems.length; i++) {
-                const collidingItem = _gridList.items[collidingItems[i]],
-                    collidingPosition = this._getItemPosition(collidingItem);
+                const itemClone = gridClone.items[itemIndex];
+                const collidingItem = gridClone.items[collidingItems[i]];
+                const collidingPosition = this._getItemPosition(collidingItem);
 
                 /*
                  * Checking if collidingItem is still colliding with item b/c _itemFitsAtPositionWhithShiftingRight
@@ -749,7 +976,7 @@
                  *  1| -- 02 -- 03 03
                  *
                  */
-                const ciCollidingItems = _gridList._getItemsCollidingWithItem(collidingItem);
+                const ciCollidingItems = gridClone._getItemsCollidingWithItem(collidingItem);
                 if (ciCollidingItems.indexOf(itemIndex) == -1) {
                     continue;
                 }
@@ -770,24 +997,24 @@
                 aboveOfItem = [collidingPosition.x, position.y - collidingPosition.h]; // visually left if grid is vertical
                 belowOfItem = [collidingPosition.x, position.y + position.h]; // visually right if grid is vertical
 
-                if (_gridList._itemFitsAtPosition(collidingItem, belowOfItem)) {
-                    _gridList._updateItemPosition(collidingItem, belowOfItem);
-                } else if (_gridList._itemFitsAtPosition(collidingItem, aboveOfItem)) {
-                    _gridList._updateItemPosition(collidingItem, aboveOfItem);
-                } else if (_gridList._itemFitsAtPositionWhithMultipleVerticalShifts(collidingItem, itemClone, 'down')) {
+                if (gridClone._itemFitsAtPosition(collidingItem, belowOfItem)) {
+                    gridClone._updateItemPosition(collidingItem, belowOfItem);
+                } else if (gridClone._itemFitsAtPosition(collidingItem, aboveOfItem)) {
+                    gridClone._updateItemPosition(collidingItem, aboveOfItem);
+                } else if (gridClone._itemFitsAtPositionWhithMultipleVerticalShifts(collidingItem, itemClone, 'down')) {
                     /*
                      *  visually right if grid is vertical
                      * nothing to do here
                      */
-                } else if (_gridList._itemFitsAtPositionWhithMultipleVerticalShifts(collidingItem, itemClone, 'up')) {
+                } else if (gridClone._itemFitsAtPositionWhithMultipleVerticalShifts(collidingItem, itemClone, 'up')) {
                     /*
                      *  visually right if grid is vertical
                      * nothing to do here
                      */
-                } else if (_gridList._itemFitsAtPosition(collidingItem, rightOfItem)) {
-                    _gridList._updateItemPosition(collidingItem, rightOfItem);
-                } else if (_gridList._itemFitsAtPosition(collidingItem, leftOfItem)) {
-                    _gridList._updateItemPosition(collidingItem, leftOfItem);
+                } else if (gridClone._itemFitsAtPosition(collidingItem, rightOfItem)) {
+                    gridClone._updateItemPosition(collidingItem, rightOfItem);
+                } else if (gridClone._itemFitsAtPosition(collidingItem, leftOfItem)) {
+                    gridClone._updateItemPosition(collidingItem, leftOfItem);
                 } else {
                     /*
                      * Collisions failed, we must use the pullItemsToLeft method to arrange
@@ -804,7 +1031,7 @@
              * we accept this scenario and marge the brached-out grid instance into the
              * original one
              */
-            GridList.cloneItems(_gridList.items, this.items);
+            GridList.cloneItems(gridClone.items, this.items);
             this.generateGrid();
             return true;
         },
@@ -813,16 +1040,16 @@
             const verticalMove = direction == 'up' ? -1 : 1;
 
             //Creating a virtual grid in the process
-            const _gridList = new GridList([], this._options);
+            const gridClone = new GridList([], this._options);
 
-            GridList.cloneItems(this.items, _gridList.items);
-            _gridList.generateGrid();
-            const collidingItemClone = _gridList.items[this._getItemIndex(collidingItem)];
-            const justMovedItemClone = _gridList.items[this._getItemIndex(justMovedItem)];
+            GridList.cloneItems(this.items, gridClone.items);
+            gridClone.generateGrid();
+            const collidingItemClone = gridClone.items[this._getItemIndex(collidingItem)];
+            const justMovedItemClone = gridClone.items[this._getItemIndex(justMovedItem)];
 
             // Return true if item is on the set of columns occupied by justMoveItemPosition
             const isItemOnRightPerimeter = function(item) {
-                const position = _gridList._getItemPosition(item);
+                const position = gridClone._getItemPosition(item);
                 return (
                     position.x >= justMoveItemPosition.x &&
           position.x + position.w - 1 <= justMoveItemPosition.x + justMoveItemPosition.w - 1
@@ -831,8 +1058,8 @@
 
             // Return true if shifting item to the right won't create a new line on the grid
             const wontGetOutOfBundaries = function(item) {
-                const position = _gridList._getItemPosition(item);
-                return position.y + position.h - 1 + verticalMove < _gridList._options.lanes && position.y + verticalMove >= 0;
+                const position = gridClone._getItemPosition(item);
+                return position.y + position.h - 1 + verticalMove < gridClone._options.lanes && position.y + verticalMove >= 0;
             };
 
             /*
@@ -848,7 +1075,7 @@
                 }
 
                 //Shifting below
-                const oldPosition = _gridList._getItemPosition(item);
+                const oldPosition = gridClone._getItemPosition(item);
 
                 const newPosition = {
                     x: oldPosition.x,
@@ -857,12 +1084,12 @@
                     h: item.h
                 };
 
-                _gridList._updateItemPosition(item, [newPosition.x, newPosition.y]);
+                gridClone._updateItemPosition(item, [newPosition.x, newPosition.y]);
 
                 //Resolving collisions
-                const collidingItems = _gridList._getItemsCollidingWithItem(item);
+                const collidingItems = gridClone._getItemsCollidingWithItem(item);
                 for (let i = 0; i < collidingItems.length; i++) {
-                    const ci = _gridList.items[collidingItems[i]];
+                    const ci = gridClone.items[collidingItems[i]];
                     if (ci != justMovedItemClone && !shiftItemVertically(ci)) {
                         return false;
                     }
@@ -873,8 +1100,8 @@
             };
 
             //Getting positions
-            const collidingPosition = _gridList._getItemPosition(collidingItemClone);
-            const justMoveItemPosition = _gridList._getItemPosition(justMovedItemClone);
+            const collidingPosition = gridClone._getItemPosition(collidingItemClone);
+            const justMoveItemPosition = gridClone._getItemPosition(justMovedItemClone);
 
             //Shifting to the right as many times as necessary to resolve original conflict
             let nbShiftsNecessary;
@@ -897,7 +1124,7 @@
              * - Not creating new column
              * - Not affecting any item outside of the set of lines occupied by justMovedItem
              */
-            GridList.cloneItems(_gridList.items, this.items);
+            GridList.cloneItems(gridClone.items, this.items);
             this.generateGrid();
             return true;
         },
@@ -970,6 +1197,18 @@
             return tail;
         },
 
+        _findPositionForItemFillingHorizontalGap: function(item, start) {
+            let currentX = start.x;
+            for (let i = currentX; i >= 0; i--) {
+                if (this._itemFitsAtPosition(item, [i, start.y])) {
+                    currentX = i;
+                } else {
+                    break;
+                }
+            }
+            return [currentX, start.y];
+        },
+
         _getItemByAttribute: function(key, value) {
             for (let i = 0; i < this.items.length; i++) {
                 if (this.items[i][key] === value) {
