@@ -99,7 +99,7 @@ class TimeseriesForecastingSearchContext(AbstractContext):
     """
 
     def __init__(
-        self, df, split_handler, model_scorer, preprocessed_external_features, shift_map, fit_before_predict,
+        self, df, split_handler, model_scorer, preprocessed_external_features, shift_map,
         min_timeseries_size_for_training, eval_metrics, metric_sign, trainable_model,
         skip_too_short_timeseries, custom_train_test_intervals,
     ):
@@ -108,7 +108,6 @@ class TimeseriesForecastingSearchContext(AbstractContext):
         self._model_scorer = model_scorer
         self._preprocessed_external_features = preprocessed_external_features
         self._shift_map = shift_map
-        self._fit_before_predict = fit_before_predict
         self._min_timeseries_size_for_training = min_timeseries_size_for_training
         self._eval_metrics = eval_metrics
         self._metric_sign = metric_sign
@@ -148,7 +147,6 @@ class TimeseriesForecastingSearchContext(AbstractContext):
             model_scorer=self._model_scorer,
             preprocessed_external_features=self._preprocessed_external_features,
             shift_map=self._shift_map,
-            fit_before_predict=self._fit_before_predict,
             parameters=parameters,
             eval_metrics=self._eval_metrics,
             metric_sign=self._metric_sign,
@@ -202,7 +200,7 @@ class TimeseriesForecastingSearchContext(AbstractContext):
             start_date = interval[0]
             end_date = interval[1]
             search_interval_df = self._df[(self._df[self._split_handler.time_column] >= start_date) & (
-                    self._df[self._split_handler.time_column] <= end_date)]
+                    self._df[self._split_handler.time_column] < end_date)]
             intervals_dfs.append(search_interval_df)
         return intervals_dfs
 
@@ -277,7 +275,8 @@ class TimeseriesForecastingSearchContext(AbstractContext):
         if custom_train_test_intervals is not None and len(custom_train_test_intervals) > 0:
             return TimeseriesForecastingSearchContext.get_contiguous_search_intervals(custom_train_test_intervals)
         else:
-            return [(min(df[time_column]), max(df[time_column]))]
+            # The search interval is used as [start, end) so we need to add a ms to the interval end to ensure we get all df.
+            return [(min(df[time_column]), max(df[time_column]) + pd.Timedelta(milliseconds=1))]
 
     @staticmethod
     def get_contiguous_search_intervals(intervals):
@@ -317,7 +316,6 @@ class TimeseriesForecastingSearchContext(AbstractContext):
                     train_intervals_union[-1][1] = max(last_end, end)
                 else:
                     train_intervals_union.append([start, end])
-
         # Step 3: Remove test/eval intervals from train intervals union
         train_without_test_intervals = []
         for train_start, train_end in train_intervals_union:
@@ -331,14 +329,10 @@ class TimeseriesForecastingSearchContext(AbstractContext):
                     else:
                         # Keep left side
                         if part_start < test_start:
-                            # do not include overlapping test start date as boundaries are included in the intervals
-                            part_end_from_test_start = test_start - timedelta(milliseconds=1)
-                            next_parts.append((part_start, min(part_end, part_end_from_test_start)))
+                            next_parts.append((part_start, min(part_end, test_start)))
                         # Keep right side
                         if part_end > test_end:
-                            # do not include overlapping test end date as boundaries are included in the intervals
-                            part_start_from_test_end = test_end + timedelta(milliseconds=1)
-                            next_parts.append((max(part_start, part_start_from_test_end), part_end))
+                            next_parts.append((max(part_start, test_end), part_end))
                 current_parts = next_parts
             train_without_test_intervals.extend(current_parts)
 
@@ -346,7 +340,7 @@ class TimeseriesForecastingSearchContext(AbstractContext):
 
 
 def _dku_timeseries_fit_and_score(
-    trainable_model, train_df, test_df, historical_df, model_scorer, preprocessed_external_features, shift_map, fit_before_predict, parameters,
+    trainable_model, train_df, test_df, historical_df, model_scorer, preprocessed_external_features, shift_map, parameters,
     eval_metrics, metric_sign, split_id
 ):
     log_df(logger, train_df, model_scorer.time_variable, fold_id=split_id, prefix="Hyperparameter search train")
@@ -372,7 +366,10 @@ def _dku_timeseries_fit_and_score(
     estimator.fit(train_df, external_features=preprocessed_external_features, shift_map=shift_map)
 
     fit_time = unix_time_millis() - start_time
-    
+
+    # Never refit right after the HP search as the model was already fit for the search
+    fit_before_predict = False
+
     forecasts_by_timeseries = model_scorer.predict_all_test_timesteps(estimator, historical_df, test_df, fit_before_predict)
     _, aggregated_metrics = model_scorer.score(historical_df, test_df, forecasts_by_timeseries, split_id)
 
