@@ -1,13 +1,12 @@
 import logging
 import traceback
+import os
 
 import numpy as np
 import pandas as pd
-from sklearn.calibration import CalibratedClassifierCV
 from sklearn.ensemble import ExtraTreesClassifier
 from sklearn.ensemble import GradientBoostingClassifier
 from sklearn.ensemble import RandomForestClassifier
-from sklearn.linear_model import LogisticRegression
 from sklearn.linear_model import SGDClassifier
 from sklearn.model_selection import train_test_split
 from sklearn.neighbors import KNeighborsClassifier
@@ -24,6 +23,7 @@ from dataiku.doctor.prediction.common import IntegerHyperparameterDimension
 from dataiku.doctor.prediction.common import SVMHyperparametersSpace
 from dataiku.doctor.prediction.common import TrainableModel
 from dataiku.doctor.prediction.common import TreesHyperparametersSpace
+from dataiku.doctor.prediction.common import TabICLHyperparametersSpace
 from dataiku.doctor.prediction.common import dump_pretrain_info
 from dataiku.doctor.prediction.common import get_groups_for_hp_search_cv
 from dataiku.doctor.prediction.common import get_initial_intrinsic_perf_data
@@ -41,12 +41,12 @@ from dataiku.doctor.prediction.lightgbm_prediction import LightGBMClassification
 from dataiku.doctor.prediction.xgboost_trainable_model import XGBoostTrainableModel
 from dataiku.doctor.sparse import prepare_multiframe_as_sparse_if_needed
 from dataiku.doctor.utils import doctor_constants
-from dataiku.doctor.utils.skcompat import gbt_skcompat_actual_params
+from dataiku.doctor.utils.skcompat import gbt_skcompat_actual_params, dku_logistic_regression, dku_calibrated_classifier_cv
 from dataiku.doctor.utils.skcompat import gbt_skcompat_hp_space
 from dataiku.doctor.utils.skcompat import get_base_estimator
 from dataiku.doctor.utils.skcompat import sgd_skcompat_actual_params
 from dataiku.doctor.utils.skcompat import sgd_skcompat_hp_space
-from dataiku.doctor.utils.gpu_execution import get_gpu_config_from_core_params, get_single_gpu_id_from_gpu_device, XGBOOSTGpuCapability
+from dataiku.doctor.utils.gpu_execution import get_gpu_config_from_core_params, get_single_gpu_id_from_gpu_device, XGBOOSTGpuCapability, TabICLGpuCapability
 
 logger = logging.getLogger(__name__)
 
@@ -62,7 +62,6 @@ def get_class_weight_dict(train_y):
 
 CLASSIFICATION_ALGORITHMS = {}
 
-
 def register_classification_algorithm(algorithm):
     CLASSIFICATION_ALGORITHMS[algorithm.algorithm] = algorithm()
 
@@ -142,6 +141,111 @@ class RFClassification(ClassicalPredictionAlgorithm):
 register_classification_algorithm(RFClassification)
 
 
+class TabICLClassification(ClassicalPredictionAlgorithm):
+    algorithm = "TABICL_CLASSIFICATION"
+
+    # IMPORTANT: The constants below (COMMIT_ID, MODEL_NAME, CHECKPOINT_VERSION)
+    # must stay up to date and synchronized with classification.tmpl file and ModelCacheService Java class
+    COMMIT_ID = "d2bb371378702e06a6da989c1b1e358b28e485a4" # Repository commit hash to ensure consistent weight retrieval
+    MODEL_NAME = "DataikuNLP/TabICL-clf@" + COMMIT_ID # HuggingFace repository identifier for the pretrained TabICL model
+    CHECKPOINT_VERSION = "tabicl-classifier-v1.1-0506.ckpt" # Specific checkpoint file to load (current latest best-performing version).
+                                                            # May need to be updated when new weights are released
+    @staticmethod
+    def get_model_path(model_name, checkpoint_version):
+        """
+        Download model weights from HuggingFace (if not already cached)
+        and return the path to the checkpoint file
+        :rtype: str
+        """
+        from dataiku.core.model_provider import get_model_provider
+        model_provider = get_model_provider()
+        model_path = model_provider.get_or_download_model(model_name)
+        return os.path.join(model_path, checkpoint_version)
+
+    @staticmethod
+    def get_device(core_params):
+        """
+        Get the appropriate device (CPU/GPU) for TabICL based on core parameters
+        """
+        gpu_config = get_gpu_config_from_core_params(core_params)
+        return TabICLGpuCapability.get_device(gpu_config)
+
+    @staticmethod
+    def get_norm_methods(input_hp_space):
+        """
+        Extract the list of selected normalization methods from the input hyperparameter space
+        """
+        norm_methods = ["norm_none", "norm_power", "norm_quantile", "norm_quantile_rtdl", "norm_robust"]
+        return [method.removeprefix('norm_') for method in norm_methods if input_hp_space[method]]
+
+    @staticmethod
+    def build_tab_icl_estimator(input_hp_space, device, model_path, norm_methods):
+        """
+        Build a TabICLClassifier with given hyperparameters and checkpoint
+        """
+        try:
+            from dataiku.vendor.tabicl import TabICLClassifier
+        except Exception as e:
+            logger.error("Failed to load tabicl package: {}".format(e))
+            raise ImportError("Could not import tabicl package")
+
+        estimator = TabICLClassifier(
+            norm_methods=norm_methods,
+            batch_size=input_hp_space['batch_size'],
+            device=device,
+            random_state=input_hp_space['random_state'],
+            softmax_temperature=input_hp_space['softmax_temperature'],
+            outlier_threshold=input_hp_space['outlier_threshold'],
+            allow_auto_download=False, # Prevent TabICL from fetching weights automatically; weight management is delegated to DSS.
+            model_path=model_path
+        )
+
+        if device == 'cpu':
+            estimator.set_params(n_jobs=input_hp_space['n_jobs'])
+
+        return estimator
+
+    def model_from_params(self, input_hp_space, modeling_params, core_params):
+        hp_space = TabICLHyperparametersSpace.from_definition(
+            input_hp_space,
+            hp_names_to_dimension_class={
+                "n_estimators": IntegerHyperparameterDimension,
+                "class_shift": CategoricalHyperparameterDimension
+            }
+        )
+
+        device = self.get_device(core_params)
+        model_path = self.get_model_path(self.MODEL_NAME, self.CHECKPOINT_VERSION)
+        norm_methods = self.get_norm_methods(input_hp_space)
+        estimator = self.build_tab_icl_estimator(input_hp_space, device, model_path, norm_methods)
+
+        return TrainableModel(estimator=estimator, hyperparameters_space=hp_space, supports_sample_weights=False)
+
+    def actual_params(self, ret, clf, fit_params):
+
+        model_params = clf.get_params()
+
+        ret["tabicl"] = {
+            "n_estimators": model_params["n_estimators"],
+            "class_shift": model_params["class_shift"],
+            "norm_methods": model_params["norm_methods"],
+            "batch_size": model_params["batch_size"],
+            "random_state": model_params["random_state"],
+            "softmax_temperature": model_params["softmax_temperature"],
+            "outlier_threshold": model_params["outlier_threshold"],
+            "n_jobs": model_params["n_jobs"],
+            "device": model_params["device"],
+            "allow_auto_download": model_params["allow_auto_download"],
+            "model_path": model_params["model_path"]
+        }
+
+        return {
+            "resolved": ret,
+            "other": {}
+        }
+
+register_classification_algorithm(TabICLClassification)
+
 class ExtraTreesClassification(ClassicalPredictionAlgorithm):
     algorithm = "EXTRA_TREES"
 
@@ -290,7 +394,7 @@ class LogisticRegClassification(ClassicalPredictionAlgorithm):
         else:
             solver = "liblinear"
 
-        estimator = LogisticRegression(
+        estimator = dku_logistic_regression(
             multi_class=multi_class,
             solver=solver,
             random_state=1337
@@ -741,7 +845,7 @@ def classification_fit(modeling_params, core_params, transformed_train, transfor
     clf.fit(train_X, train_y, **final_fit_parameters)
 
     if calibrate_proba:
-        calibrated_clf = CalibratedClassifierCV(clf, cv="prefit", method=calibration_method.lower())
+        calibrated_clf = dku_calibrated_classifier_cv(clf, cv="prefit", method=calibration_method.lower())
         calibrated_clf.fit(calib_X, calib_y, sample_weight=calib_w)
         clf = calibrated_clf
 
