@@ -3,6 +3,7 @@ from abc import abstractmethod
 from abc import ABCMeta
 
 import numpy as np
+import scipy
 import pandas as pd
 import six
 
@@ -121,8 +122,8 @@ class DefaultShapleyFrankensteinScoresBuilder(ShapleyFrankensteinScoresBuilder):
         return self._frankenstein_builder.get_size()
 
     def _get_frankenstein_score(self, df, df_score, frankenstein_builder):
-        matching_indices_in_df, frankenstein_arr = frankenstein_builder.build_frankenstein_array(df.values,
-                                                                                                 self.background_rows_df.values)
+        matching_indices_in_df, frankenstein_arr = frankenstein_builder.build_frankenstein_dense_array(df.values,
+                                                                                                       self.background_rows_df.values)
         frankenstein_df = pd.DataFrame(data=frankenstein_arr, columns=df.columns).astype(df.dtypes)
         logger.info("Built frankenstein of shape {}".format(frankenstein_df.shape))
         return self._score_computer(frankenstein_df, df_score, matching_indices_in_df)
@@ -185,19 +186,33 @@ class PreprocessShapleyFrankensteinScoresBuilder(ShapleyFrankensteinScoresBuilde
                                                                        self._frankenstein_features_builder))
         return self._frankenstein_features_builder.get_size()
 
-    def _get_frankenstein_score(self, df_preprocessed, df_score, frankenstein_builder):
-        matching_indices_in_arr, frankenstein_arr = frankenstein_builder.build_frankenstein_array(
+    def _get_frankenstein_score_dense(self, df_preprocessed, df_score, frankenstein_builder):
+        matching_indices_in_arr, frankenstein_arr = frankenstein_builder.build_frankenstein_dense_array(
             df_preprocessed, self._preprocessed_background)
         logger.info("Built frankenstein of shape {}".format(frankenstein_arr.shape))
         return self._score_computer_from_preprocess(frankenstein_arr, df_score, matching_indices_in_arr)
 
+    def _get_frankenstein_score_sparse(self, csr_preprocessed, df_score, frankenstein_builder):
+        matching_indices_in_data, frankenstein_csr = frankenstein_builder.build_frankenstein_sparse_matrix(
+            csr_preprocessed, self._preprocessed_background)
+        logger.info("Built frankenstein of shape {}".format(frankenstein_csr.shape))
+        return self._score_computer_from_preprocess(frankenstein_csr, df_score, matching_indices_in_data)
+
     def build_scores(self, df):
-        df_preprocessed = self._preprocess(df)
-        df_score = self._score_computer_from_preprocess(df_preprocessed)
-        frankenstein_score = self._get_frankenstein_score(
-            df_preprocessed, df_score, self._frankenstein_features_builder)
-        frankenstein_to_compare_score = self._get_frankenstein_score(
-            df_preprocessed, df_score, self._frankenstein_to_compare_features_builder)
+
+        preprocessed_data = self._preprocess(df)
+        score = self._score_computer_from_preprocess(preprocessed_data)
+        if isinstance(preprocessed_data, scipy.sparse.csr_matrix):
+            frankenstein_score = self._get_frankenstein_score_sparse(
+                preprocessed_data, score, self._frankenstein_features_builder)
+            frankenstein_to_compare_score = self._get_frankenstein_score_sparse(
+                preprocessed_data, score, self._frankenstein_to_compare_features_builder)
+        else:
+            # Numpy array
+            frankenstein_score = self._get_frankenstein_score_dense(
+                preprocessed_data, score, self._frankenstein_features_builder)
+            frankenstein_to_compare_score = self._get_frankenstein_score_dense(
+                preprocessed_data, score, self._frankenstein_to_compare_features_builder)
         return frankenstein_score, frankenstein_to_compare_score
 
 
@@ -211,14 +226,14 @@ class ShapleyIndividualExplanationsExtractor(ShapleyExplanationsExtractor):
         :rtype: SimpleExplanationResult
         """
         # Compute marginal contribution of having some column in predictions but not in predictions_to_compare
-        # Shape: (len(columns), background_size, len(observations_df))
+        # Shape: (background_size, len(columns), len(observations_df))
         diff = (baseline_score.score - swapped_columns_score.score).reshape(
             self._background_size, len(self._columns_to_explain), -1, order="F")
-        # Average over the background_size to get Shapley values for each column of interest in each row
-        # Shape: (len(observations_df), len(columns))
 
-        # With overrides declined outcome the model can output nan predictions, therefore we need to ensure
+        # Average over the background_size to get Shapley values for each column of interest in each row
+        # With overrides (declined outcome), the model can output nan predictions, therefore we need to ensure
         # that the mean is computed only on the valid predictions.
+        # Shape: (len(observations_df), len(columns))
         reduced_diff = np.nanmean(diff, axis=0).T
 
         # columns_indices corresponds to the columns Shapley values were computed on
@@ -279,16 +294,16 @@ def build_shapley_frankenstein_masks(total_num_columns, col_indices_to_explain,
     Builds masks required to build Shapley frankenstein arrays.
 
     The Shapley algorithm relies on:
-     * Randomly permuting values of some columns of the data with values taken from _background_ data
+     * Randomly replacing values of some columns of the data by values taken from _background_ data
        in 2 similar but different ways,
      * Then getting the prediction of those newly created rows
-     * Deriving the explanations by comparing the predictions of the 2 ways to permute
+     * Deriving the explanations by comparing the predictions of the 2 ways to replace
     More details in the original paper: https://arxiv.org/abs/1705.07874
 
     In order to efficiently compute those new rows and their prediction, we build _frankenstein_ arrays that are
     concatenations of all those rows for all the columns to explain, and then run the prediction only once on the
     frankenstein. To do so, we first build masks that will tell us, for each cell of the frankenstein, whether it needs
-    to be taken from the original data, or permuted with some cell from the background.
+    to be taken from the original data, or replaced by some cell from the background data.
 
     In order to better rely on the magic of numpy broadcasting feature, we do not build 2D masks, but 3D masks such
     that the values in the 2D mask [i, :, :] represent the mask for column to explain number i.
@@ -301,12 +316,12 @@ def build_shapley_frankenstein_masks(total_num_columns, col_indices_to_explain,
         for i in range(background_size):       => [190,   'j. Doe', 'male', 38.0,   70]
           permutations = get_permutation()     => [3, 0, 2, 1, 4]
           col_index = get_col_index()          => 2
-          # swap all cols in permutations prior to col_index
+          # replace all cols in permutations strictly prior to col_index
           c_to_swap = get_c_to_swap()          => [3, 0]
           n_row = do_swap(row, c_to_swap,      => [190, 'Mr Smith', 'male', 38.0, 7.25]
                           back_rows[i])             ^                        ^
           frankenstein.add(n_row)
-          # for comparison, swap all cols in permutations including col_index
+          # for comparison, replace all cols in permutations prior or equal to col_index (1 additional replacement)
           c_to_swap_c = get_c_to_swap_c()    => [3, 0, 2]
           n_row_c = do_swap(row, c_to_swap_c,=> [190, 'Mr Smith', 'male', 38.0, 7.25]
                             back_rows[i])         ^                  ^      ^
@@ -327,8 +342,8 @@ def build_shapley_frankenstein_masks(total_num_columns, col_indices_to_explain,
     """
     assert permutations.shape == (background_size, total_num_columns)
     num_columns_to_explain = len(col_indices_to_explain)
-    fk_permutations_mask = np.zeros((num_columns_to_explain, background_size, total_num_columns), dtype=bool)
-    fk_compare_permutations_mask = fk_permutations_mask.copy()
+    fk_replacements_mask = np.zeros((num_columns_to_explain, background_size, total_num_columns), dtype=bool)
+    fk_compare_replacements_mask = fk_replacements_mask.copy()
 
     # For loop over column length done once, which should be negligible compared to the other dimensions, so ok
     for col_index, col_index_in_all_cols in enumerate(col_indices_to_explain):
@@ -363,7 +378,7 @@ def build_shapley_frankenstein_masks(total_num_columns, col_indices_to_explain,
         #           [0, 0, 1, 1],
         #           [0, 1, 1, 1]])
         #
-        # # Getting the logical opposite puts True where, in the permutations we need to swap
+        # # Getting the logical opposite puts True where, in the permutations, we need to swap
         # np.logical_not((permutations == col_index_in_all_cols).cumsum(axis=1))
         # => array([[ True,  True,  True, False],
         #           [ True,  True,  True, False],
@@ -375,12 +390,12 @@ def build_shapley_frankenstein_masks(total_num_columns, col_indices_to_explain,
         #
         #
         # # Retrieving the indices in the permutations
-        # indices_to_permute_col = np.where(np.logical_not((permutations == col_index_in_all_cols).cumsum(axis=1)))
+        # indices_to_replace_col = np.where(np.logical_not((permutations == col_index_in_all_cols).cumsum(axis=1)))
         # => (array([0, 0, 0, 1, 1, 1, 3, 3, 3, 5, 5, 6]),
         #     array([0, 1, 2, 0, 1, 2, 0, 1, 2, 0, 1, 0]))
-        indices_to_permute_col = np.where(np.logical_not((permutations == col_index_in_all_cols).cumsum(axis=1)))
+        indices_to_replace_col = np.where(np.logical_not((permutations == col_index_in_all_cols).cumsum(axis=1)))
         # # Now that we have the indices in the permutations, we can project them on the real mask
-        # per_col_mask[indices_to_permute_col[0], permutations[indices_to_permute_col]] = True
+        # per_col_mask[indices_to_replace_col[0], permutations[indices_to_replace_col]] = True
         # per_col_mask
         # => array([[ True,  True, False,  True],
         #           [ True,  True, False,  True],
@@ -389,15 +404,15 @@ def build_shapley_frankenstein_masks(total_num_columns, col_indices_to_explain,
         #           [False, False, False, False],
         #           [ True,  True, False, False],
         #           [False,  True, False, False]])
-        fk_permutations_mask[col_index, :, :][indices_to_permute_col[0], permutations[indices_to_permute_col]] = True
+        fk_replacements_mask[col_index, :, :][indices_to_replace_col[0], permutations[indices_to_replace_col]] = True
 
         # Build 2nd frankenstein mask filled with the rows of 1st frankenstein
-        # except column being explained is also permuted with background rows
-        fk_compare_permutations_mask[col_index, :, :] = fk_permutations_mask[col_index, :, :]
-        fk_compare_permutations_mask[col_index, :, :][:, col_index_in_all_cols] = True
+        # except column being explained is also replaced with background data
+        fk_compare_replacements_mask[col_index, :, :] = fk_replacements_mask[col_index, :, :]
+        fk_compare_replacements_mask[col_index, :, :][:, col_index_in_all_cols] = True
 
-    return (ShapleyFrankensteinArrayBuilder(fk_permutations_mask),
-            ShapleyFrankensteinArrayBuilder(fk_compare_permutations_mask))
+    return (ShapleyFrankensteinArrayBuilder(fk_replacements_mask),
+            ShapleyFrankensteinArrayBuilder(fk_compare_replacements_mask))
 
 
 class ShapleyFrankensteinArrayBuilder(object):
@@ -418,7 +433,7 @@ class ShapleyFrankensteinArrayBuilder(object):
                 "background_size={}, num_cols_to_explain={})".format(self._total_num_columns,
                                                                      self._background_size, self._num_cols_to_explain))
 
-    def build_frankenstein_array(self, data, background_rows):
+    def build_frankenstein_dense_array(self, data, background_rows):
         """
         Build frankenstein array and return it, along with the matching indices in the data it was built from
         :type data: np.ndarray
@@ -438,3 +453,38 @@ class ShapleyFrankensteinArrayBuilder(object):
                          background_rows[np.newaxis, np.newaxis, :, :],
                          data[:, np.newaxis, np.newaxis, :])
                   .reshape(-1, self._total_num_columns))
+
+    def build_frankenstein_sparse_matrix(self, data_csr, background_rows_csr):
+        """
+        Build frankenstein sparse matrix and return it, along with the matching indices in the data it was built from
+        :type data_csr: scipy.sparse.csr_matrix
+        :type background_rows_csr: scipy.sparse.csr_matrix
+        :return the matching row indices in the original data, and the frankenstein sparse matrix
+        :rtype: (np.ndarray, scipy.sparse.csr_matrix)
+        """
+        assert data_csr.shape[1] == self._total_num_columns, "Wrong number of columns"
+        assert background_rows_csr.shape[1] == self._total_num_columns, "Wrong number of columns"
+        assert background_rows_csr.shape[0] == self._background_size, "Wrong number of rows in background"
+        matching_indices_in_array = np.repeat(np.arange(data_csr.shape[0]), self._background_size * self._num_cols_to_explain)
+        to_stack_masks = []
+        for data_row_index in range(data_csr.shape[0]):
+            to_stack_masks.append(self.mask.reshape(self.mask.shape[0] * self.mask.shape[1], -1))
+        # mask_expanded.shape = (number of columns to explain) * (number of background rows) * (number of rows in the data to explain), (total number of columns)
+        # NB: this is the same size as the dense array returned by build_frankenstein_dense_array
+        masks_expanded = np.vstack(to_stack_masks)
+        data_expanded = scipy.sparse.kron(data_csr, np.ones(background_rows_csr.shape[0] * self.mask.shape[0], dtype=bool)[::,np.newaxis])
+        background_rows_expanded = scipy.sparse.kron(np.ones(data_csr.shape[0] * self.mask.shape[0], dtype=bool)[::,np.newaxis], background_rows_csr)
+        fk = sparse_where(masks_expanded, background_rows_expanded, data_expanded)
+        return matching_indices_in_array, fk
+
+def sparse_where(mask, m1, m2):
+    """
+    :param mask: 2d ndarray, dtype bool
+    :param m1: csr_matrix
+    :param m2: csr_matrix
+    :return: csr_matrix
+    """
+    assert mask.ndim == 2, "Wrong shape for sparse_where mask: expected 2d array"
+    assert mask.shape == m1.shape, "Incompatible shapes between mask {} and matrix {} in `sparse_where`".format(mask.shape, m1.shape)
+    assert mask.shape == m2.shape, "Incompatible shapes between mask {} and matrix {} in `sparse_where`".format(mask.shape, m2.shape)
+    return m1.multiply(mask) + m2.multiply(~mask)
