@@ -58,14 +58,14 @@ class TimeseriesEvaluationHandler(object):
         self.custom_metrics = metrics_params["customMetrics"]
 
         self.external_features = (
-            get_external_features(self.preprocessing_params, self.modeling_params.get("isShiftWindowsCompatible", False)) if self.algorithm.SUPPORTS_EXTERNAL_FEATURES else None
+            get_external_features(self.preprocessing_params, self.algorithm) if self.algorithm.EXTERNAL_FEATURES_COMPATIBILITY.supports_external_features() else None
         )
 
         context = DiagOnlyContext(diagnostics_folder_context)
         self.listener = ProgressListener(context=context)
 
         self.timeseries_preprocessing = TimeseriesPreprocessing(
-            preprocessing_folder_context, core_params, preprocessing_params, modeling_params, self.listener, self.algorithm.SUPPORTS_EXTERNAL_FEATURES
+            preprocessing_folder_context, core_params, preprocessing_params, modeling_params, self.listener, self.algorithm.EXTERNAL_FEATURES_COMPATIBILITY.supports_external_features()
         )
         self.has_model_evaluation_store = model_evaluation_store_folder_context is not None
         self.model_evaluation_store_folder_context = model_evaluation_store_folder_context
@@ -102,7 +102,7 @@ class TimeseriesEvaluationHandler(object):
                 self.core_params,
                 self.preprocessing_params,
                 self.external_features,
-                self.modeling_params.get("isShiftWindowsCompatible", False),
+                self.algorithm.EXTERNAL_FEATURES_COMPATIBILITY.supports_past_only_external_features(),
                 separate_external_features=False
             )
 
@@ -111,7 +111,7 @@ class TimeseriesEvaluationHandler(object):
             resampled_df = add_rolling_windows_for_scoring(resampled_df, self.core_params, windows_list, self.preprocessing_params,  self.preprocessing_folder_context)
 
         if self.external_features:
-            preprocess_on_full_df = self.algorithm.USE_GLUON_TS
+            preprocess_on_full_df = self.algorithm.ONE_MODEL_FOR_MULTIPLE_TS
 
             self.timeseries_preprocessing.create_timeseries_preprocessing_handlers(
                 resampled_df, preprocess_on_full_df, use_saved_resources=True
@@ -148,7 +148,7 @@ class TimeseriesEvaluationHandler(object):
             for timeseries_identifier, df_of_timeseries_identifier in timeseries_iterator(preprocessed_df, self.timeseries_identifier_columns):
                 timeseries_length = len(df_of_timeseries_identifier.index)
 
-                if not self.algorithm.USE_GLUON_TS and timeseries_identifier not in self.clf.trained_models:
+                if not self.algorithm.ONE_MODEL_FOR_MULTIPLE_TS and timeseries_identifier not in self.clf.trained_models:
                     unseen_timeseries_identifiers.append(timeseries_identifier)
                 elif timeseries_length < min_required_length_for_evaluation:
                     too_short_timeseries_identifiers.append(timeseries_identifier)
