@@ -3,10 +3,13 @@ import math
 import sys
 
 import numpy as np
+from enum import Enum
 
+from dataiku.base.utils import package_is_at_least
 from dataiku.core import doctor_constants
 from dataiku.doctor.crossval.search_runner import TimeseriesForecastingSearchRunner
-from dataiku.doctor.prediction.common import get_input_hyperparameter_space, ETSHyperparametersSpace
+from dataiku.doctor.prediction.common import get_input_hyperparameter_space, ETSHyperparametersSpace, \
+    CrostonHyperparametersSpace, TFTHyperparameterSpace
 from dataiku.doctor.prediction.common import TrainableModel
 from dataiku.doctor.prediction.common import TabularPredictionAlgorithm
 from dataiku.doctor.prediction.common import HyperparametersSpace
@@ -18,16 +21,32 @@ from dataiku.doctor.prediction.common import FloatHyperparameterDimension
 from dataiku.doctor.prediction.common import CategoricalHyperparameterDimension
 from dataiku.doctor.prediction.common import TreesHyperparametersSpace, get_max_features_dimension, replace_value_by_empty
 from dataiku.doctor.prediction.custom_scoring import get_custom_evaluation_metric
+from dataiku.doctor.prediction.lightgbm_prediction import LightGBMHyperparametersSpace
 from dataiku.doctor.prediction.metric import TIMESERIES_METRICS_NAME_TO_FIELD_NAME
 from dataiku.doctor.prediction.metric import CUSTOM
 from dataiku.doctor.timeseries.preparation.resampling.utils import get_frequency, get_monthly_day_alignment
-from dataiku.doctor.timeseries.utils import prefix_custom_metric_name
+from dataiku.doctor.timeseries.utils import prefix_custom_metric_name, NEURALFORECAST_N_WINDOWS
 
 from dataiku.core.doctor_constants import TIMESERIES_FORECAST
-from dataiku.doctor.utils.gpu_execution import get_gpu_config_from_core_params, GluonTSMXNetGPUCapability, get_default_gpu_params, GluonTSTorchGPUCapability
+from dataiku.doctor.utils.gpu_execution import get_gpu_config_from_core_params, GluonTSMXNetGPUCapability, \
+    get_default_gpu_params, GluonTSTorchGPUCapability, NeuralforecastGPUCapability
 
 logger = logging.getLogger(__name__)
 
+class ExternalFeaturesCompatibility(Enum):
+    NO_SUPPORT = 1
+    KNOWN_IN_ADVANCE = 2
+    KNOWN_IN_ADVANCE_AND_PAST_ONLY = 3
+
+    def supports_external_features(self):
+        return self in {
+            ExternalFeaturesCompatibility.KNOWN_IN_ADVANCE,
+            ExternalFeaturesCompatibility.KNOWN_IN_ADVANCE_AND_PAST_ONLY
+        }
+
+    def supports_past_only_external_features(self):
+        return self == ExternalFeaturesCompatibility.KNOWN_IN_ADVANCE_AND_PAST_ONLY
+
 
 class TimeseriesForecastingAlgorithm(TabularPredictionAlgorithm):
     class ModelRefit:
@@ -35,15 +54,15 @@ class TimeseriesForecastingAlgorithm(TabularPredictionAlgorithm):
         SUPPORTED = "SUPPORTED"
         NOT_SUPPORTED = "NOT_SUPPORTED"
 
+    ONE_MODEL_FOR_MULTIPLE_TS = False
     REFIT_FOR_SCORING = ModelRefit.NOT_SUPPORTED
-    USE_GLUON_TS = False
-    SUPPORTS_EXTERNAL_FEATURES = False
+    EXTERNAL_FEATURES_COMPATIBILITY = ExternalFeaturesCompatibility.NO_SUPPORT
     REQUIRES_EXTERNAL_FEATURES = False
     SUPPORTS_MODEL_COEFFICIENTS = False
     SUPPORTS_INFORMATION_CRITERIA = False
     SUPPORTS_QUANTILES = True
-    # SUPPORTS_EXTERNAL_FEATURES to be synced with:
-    # platypus -> static/dataiku/js/analysis/prediction/timeseries/settings.js L.1678
+    # EXTERNAL_FEATURES_COMPATIBILITY to be synced with:
+    # platypus -> static/dataiku/js/analysis/prediction/timeseries/settings.js L.2958
     # java -> com/dataiku/dip/analysis/model/prediction/TimeseriesForecastingModelDetails.java L.43
 
     @staticmethod
@@ -100,7 +119,6 @@ class TimeseriesForecastingAlgorithm(TabularPredictionAlgorithm):
             trainable_model=trainable_model,
             search_settings=search_settings,
             split_handler=split_handler,
-            fit_before_predict=self.should_fit_before_predict(),
             min_timeseries_size_for_training=self.get_min_size_for_training(modeling_params, preprocessing_params, core_params[doctor_constants.PREDICTION_LENGTH]),
             model_folder_context=model_folder_context,
             evaluation_metric=evaluation_metric_name,
@@ -124,30 +142,55 @@ class TimeseriesForecastingMLAlgorithm(TimeseriesForecastingAlgorithm):
     SUPPORTS_QUANTILES = False
 
     def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
-        shifts = preprocessing_params["feature_generation"]["shifts"]
-        windows = preprocessing_params["feature_generation"]["windows"]
-        return self.get_minimum_size_for_x_number_of_rows(shifts, windows)
+        return self.get_minimum_size_for_x_number_of_rows(preprocessing_params)
 
     def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
-        shifts = preprocessing_params["feature_generation"]["shifts"]
-        windows = preprocessing_params["feature_generation"]["windows"]
-        return self.get_minimum_size_for_x_number_of_rows(shifts, windows, prediction_length)
+        return self.get_minimum_size_for_x_number_of_rows(preprocessing_params, prediction_length)
 
     @staticmethod
-    def get_minimum_size_for_x_number_of_rows(shifts, windows, prediction_length=None):
-        furthest_shift_from_horizon, furthest_shift_from_forecast, min_window_shift = sys.maxsize, sys.maxsize, sys.maxsize
+    def get_minimum_size_for_x_number_of_rows(preprocessing_params, prediction_length=None):
+        """
+        <-- Past ------------|------------ Future -->
+        ... -k, ..., -2, -1, 0, 1, 2, 3, 4, 5 ..., prediction_length
+                             |  |__________________________________| Forecast Horizon
+                    Forecast origin (t=0)
+        """
+        from dataiku.doctor.timeseries.preparation.preprocessing import is_shift_compatible
+        furthest, furthest_shift, min_window_shift = sys.maxsize, sys.maxsize, sys.maxsize
 
-        if len(shifts) == 0 and len(windows) == 0:
+        if len(preprocessing_params["feature_generation"]["shifts"]) == 0 and len(preprocessing_params["feature_generation"]["windows"]) == 0:
             return prediction_length if prediction_length else 1
 
-        if len(shifts) > 0:
-            furthest_shift_from_horizon = 1 + min([min(shift["from_horizon"]) if len(shift["from_horizon"]) > 0 else 0 for shift in shifts.values()])
-            furthest_shift_from_forecast = min([min(shift["from_forecast"]) if len(shift["from_forecast"]) > 0 else 0 for shift in shifts.values()])
+        for column in list(preprocessing_params["per_feature"].keys()):
+            role = preprocessing_params["per_feature"][column]["role"]
+            if not is_shift_compatible(role):
+                continue
 
-        if len(windows) > 0:
-            min_window_shift = min([-window["length"] + window["shift"] - (1 if window["is_from_forecast"] else 0) for window in windows])
+            shifts = preprocessing_params["feature_generation"].get("shifts", {}).get(column, None)
+            windows = preprocessing_params["feature_generation"].get("windows", [])
+            if shifts is not None:
+                if len(shifts["from_forecast"]) > 0:
+                    furthest_column_shift_from_forecast = -1 + min(shifts["from_forecast"])
+                else:
+                    furthest_column_shift_from_forecast = 0
 
-        furthest = min(furthest_shift_from_forecast, furthest_shift_from_horizon, min_window_shift)
+                if shifts["from_horizon_mode"] == 'FIXED' and len(shifts["from_horizon"]) > 0:
+                    furthest_column_shift_from_horizon = min(shifts["from_horizon"])
+                elif shifts["from_horizon_mode"] == 'AUTO' and len(shifts["from_horizon_auto"]) > 0:
+                    furthest_column_shift_from_horizon = min(shifts["from_horizon_auto"])
+                else:
+                    furthest_column_shift_from_horizon = 0
+
+                furthest_shift = min(furthest_shift, furthest_column_shift_from_forecast, furthest_column_shift_from_horizon)
+
+            if len(windows) > 0:
+                for window in windows:
+                    if column in window["operations_map"].keys() and any(operation.get("enabled", False) for operation in window["operations_map"][column]):
+                        current_window_min_shift = -window["length"] + window["shift"] - (1 if window["is_from_forecast"] else 0)
+                        min_window_shift = min(min_window_shift, current_window_min_shift)
+
+        # Only consider negative shifts
+        furthest = min(0, furthest_shift, min_window_shift)
 
         if prediction_length:
             furthest -= prediction_length
@@ -157,7 +200,7 @@ class TimeseriesForecastingMLAlgorithm(TimeseriesForecastingAlgorithm):
 
 class RandomForestForecastingAlgorithm(TimeseriesForecastingMLAlgorithm):
 
-    SUPPORTS_EXTERNAL_FEATURES = True
+    EXTERNAL_FEATURES_COMPATIBILITY = ExternalFeaturesCompatibility.KNOWN_IN_ADVANCE_AND_PAST_ONLY
     REQUIRES_EXTERNAL_FEATURES = True
 
     def model_from_params(self, input_hp_space, modeling_params, core_params):
@@ -216,8 +259,10 @@ class RandomForestForecastingAlgorithm(TimeseriesForecastingMLAlgorithm):
 
 class RidgeForecastingAlgorithm(TimeseriesForecastingMLAlgorithm):
 
-    SUPPORTS_EXTERNAL_FEATURES = True
+    EXTERNAL_FEATURES_COMPATIBILITY = ExternalFeaturesCompatibility.KNOWN_IN_ADVANCE_AND_PAST_ONLY
     REQUIRES_EXTERNAL_FEATURES = True
+    import sklearn
+    SUPPORTS_MODEL_COEFFICIENTS = package_is_at_least(sklearn, "1.0")
 
     def model_from_params(self, input_hp_space, modeling_params, core_params):
         from dataiku.doctor.timeseries.models.ml_estimator import  DkuTimeSeriesRidgeEstimator
@@ -247,15 +292,19 @@ class RidgeForecastingAlgorithm(TimeseriesForecastingMLAlgorithm):
         amp = {"resolved": ret, "other": {}}
         params = model.get_params()
 
-        ret["ridge"] = {
-            "alpha": params["alpha"],
-        }
+        ret["ridge"] = {}
+
+        if params.get("alpha_mode", "") == "AUTO":
+            ret["ridge"]["alpha"] = 0.0
+        else:
+            ret["ridge"]["alpha"] = params["alpha"]
 
         return amp
 
+
 class XGBoostForecastingAlgorithm(TimeseriesForecastingMLAlgorithm):
 
-    SUPPORTS_EXTERNAL_FEATURES = True
+    EXTERNAL_FEATURES_COMPATIBILITY = ExternalFeaturesCompatibility.KNOWN_IN_ADVANCE_AND_PAST_ONLY
     REQUIRES_EXTERNAL_FEATURES = True
 
     def model_from_params(self, input_hp_space, modeling_params, core_params):
@@ -346,9 +395,77 @@ class XGBoostForecastingAlgorithm(TimeseriesForecastingMLAlgorithm):
         }
         return amp
 
+class LightGBMForecastingAlgorithm(TimeseriesForecastingMLAlgorithm):
+    EXTERNAL_FEATURES_COMPATIBILITY = ExternalFeaturesCompatibility.KNOWN_IN_ADVANCE_AND_PAST_ONLY
+    REQUIRES_EXTERNAL_FEATURES = True
+
+    def model_from_params(self, input_hp_space, modeling_params, core_params):
+        from dataiku.doctor.timeseries.models.ml_estimator import DkuTimeSeriesLightGBMEstimator
+        from dataiku.doctor.prediction.common import safe_positive_int
+
+        estimator = DkuTimeSeriesLightGBMEstimator(
+            frequency=get_frequency(core_params),
+            prediction_length=core_params[doctor_constants.PREDICTION_LENGTH],
+            time_variable=core_params[doctor_constants.TIME_VARIABLE],
+            target_variable=core_params[doctor_constants.TARGET_VARIABLE],
+            timeseries_identifiers=core_params[doctor_constants.TIMESERIES_IDENTIFIER_COLUMNS],
+            monthly_day_alignment=get_monthly_day_alignment(core_params),
+            n_jobs=safe_positive_int(input_hp_space['n_jobs']),
+            random_state=input_hp_space['random_state'],
+            max_depth=input_hp_space["max_depth"]
+        )
+
+        hyperparameters_space = LightGBMHyperparametersSpace.from_definition(
+            input_hp_space,
+            hp_names_to_dimension_class={
+                "boosting_type": CategoricalHyperparameterDimension,
+                "num_leaves": IntegerHyperparameterDimension,
+                "learning_rate": FloatHyperparameterDimension,
+                "n_estimators": IntegerHyperparameterDimension,
+                "min_split_gain": FloatHyperparameterDimension,
+                "min_child_weight": FloatHyperparameterDimension,
+                "min_child_samples": IntegerHyperparameterDimension,
+                "colsample_bytree": FloatHyperparameterDimension,
+                "reg_alpha": FloatHyperparameterDimension,
+                "reg_lambda": FloatHyperparameterDimension
+            },
+            constructor_args={
+                "use_bagging": input_hp_space["use_bagging"],
+                "subsample": input_hp_space["subsample"],
+                "subsample_freq": input_hp_space["subsample_freq"]
+            }
+        )
+
+        return TrainableModel(estimator, hyperparameters_space=hyperparameters_space)
+
+    def actual_params(self, ret, model, fit_params):
+        model_params = model.get_params()
+        amp = {"resolved": ret, "other": {}}
+
+        ret["lightgbm"] = {
+            "boosting_type": model_params["boosting_type"],
+            "num_leaves": model_params["num_leaves"],
+            "max_depth": model_params["max_depth"],
+            "learning_rate": model_params["learning_rate"],
+            "n_estimators": model_params["n_estimators"],
+            "subsample_for_bin": model_params["subsample_for_bin"],
+            "objective": model_params["objective"],
+            "min_split_gain": model_params["min_split_gain"],
+            "min_child_weight": model_params["min_child_weight"],
+            "min_child_samples": model_params["min_child_samples"],
+            "subsample": model_params["subsample"],
+            "subsample_freq": model_params["subsample_freq"],
+            "colsample_bytree": model_params["colsample_bytree"],
+            "reg_alpha": model_params["reg_alpha"],
+            "reg_lambda": model_params["reg_lambda"],
+            "random_state": model_params["random_state"],
+            "n_jobs": model_params["n_jobs"]
+        }
+
+        return amp
 
 class GluonTSTrivialIdentity(TimeseriesForecastingAlgorithm):
-    USE_GLUON_TS = True
+    ONE_MODEL_FOR_MULTIPLE_TS = True
 
     def model_from_params(self, input_hp_space, modeling_params, core_params):
         from dataiku.doctor.timeseries.models.gluonts.trivial_identity import DkuTrivialIdentityEstimator
@@ -378,7 +495,7 @@ class GluonTSTrivialIdentity(TimeseriesForecastingAlgorithm):
 
 
 class GluonTSSeasonalNaive(TimeseriesForecastingAlgorithm):
-    USE_GLUON_TS = True
+    ONE_MODEL_FOR_MULTIPLE_TS = True
 
     def model_from_params(self, input_hp_space, modeling_params, core_params):
         from dataiku.doctor.timeseries.models.gluonts.seasonal_naive import DkuSeasonalNaiveSEstimator
@@ -415,8 +532,8 @@ class GluonTSSeasonalNaive(TimeseriesForecastingAlgorithm):
 
 
 class GluonTSNPTSForecaster(TimeseriesForecastingAlgorithm):
-    USE_GLUON_TS = True
-    SUPPORTS_EXTERNAL_FEATURES = True
+    ONE_MODEL_FOR_MULTIPLE_TS = True
+    EXTERNAL_FEATURES_COMPATIBILITY = ExternalFeaturesCompatibility.KNOWN_IN_ADVANCE
 
     def model_from_params(self, input_hp_space, modeling_params, core_params):
         from dataiku.doctor.timeseries.models.gluonts.npts import DkuNPTSEstimator
@@ -474,7 +591,7 @@ class GluonTSNPTSForecaster(TimeseriesForecastingAlgorithm):
 
 
 class GluonTSTorchSimpleFeedForward(TimeseriesForecastingAlgorithm):
-    USE_GLUON_TS = True
+    ONE_MODEL_FOR_MULTIPLE_TS = True
 
     def model_from_params(self, input_hp_space, modeling_params, core_params):
         gpu_config = get_gpu_config_from_core_params(core_params)
@@ -550,8 +667,8 @@ class GluonTSTorchSimpleFeedForward(TimeseriesForecastingAlgorithm):
 
 
 class GluonTSTorchDeepAR(TimeseriesForecastingAlgorithm):
-    USE_GLUON_TS = True
-    SUPPORTS_EXTERNAL_FEATURES = True
+    ONE_MODEL_FOR_MULTIPLE_TS = True
+    EXTERNAL_FEATURES_COMPATIBILITY = ExternalFeaturesCompatibility.KNOWN_IN_ADVANCE
 
     def model_from_params(self, input_hp_space, modeling_params, core_params):
         gpu_config = get_gpu_config_from_core_params(core_params)
@@ -643,7 +760,7 @@ class GluonTSTorchDeepAR(TimeseriesForecastingAlgorithm):
 
 
 class GluonTSSimpleFeedForward(TimeseriesForecastingAlgorithm):
-    USE_GLUON_TS = True
+    ONE_MODEL_FOR_MULTIPLE_TS = True
 
     def model_from_params(self, input_hp_space, modeling_params, core_params):
         gpu_config = get_gpu_config_from_core_params(core_params)
@@ -714,8 +831,8 @@ class GluonTSSimpleFeedForward(TimeseriesForecastingAlgorithm):
 
 
 class GluonTSDeepAR(TimeseriesForecastingAlgorithm):
-    USE_GLUON_TS = True
-    SUPPORTS_EXTERNAL_FEATURES = True
+    ONE_MODEL_FOR_MULTIPLE_TS = True
+    EXTERNAL_FEATURES_COMPATIBILITY = ExternalFeaturesCompatibility.KNOWN_IN_ADVANCE
 
     def model_from_params(self, input_hp_space, modeling_params, core_params):
         gpu_config = get_gpu_config_from_core_params(core_params)
@@ -804,11 +921,11 @@ class GluonTSDeepAR(TimeseriesForecastingAlgorithm):
         from gluonts.time_feature import get_lags_for_frequency
         return self.get_min_size_for_scoring(modeling_params, preprocessing_params,
                                              prediction_length) + max(get_lags_for_frequency(freq_str=frequency))
-        
+
 
 class GluonTSTransformer(TimeseriesForecastingAlgorithm):
-    USE_GLUON_TS = True
-    SUPPORTS_EXTERNAL_FEATURES = True
+    ONE_MODEL_FOR_MULTIPLE_TS = True
+    EXTERNAL_FEATURES_COMPATIBILITY = ExternalFeaturesCompatibility.KNOWN_IN_ADVANCE
 
     def model_from_params(self, input_hp_space, modeling_params, core_params):
         gpu_config = get_gpu_config_from_core_params(core_params)
@@ -891,8 +1008,8 @@ class GluonTSTransformer(TimeseriesForecastingAlgorithm):
 
 
 class GluonTSMQCNN(TimeseriesForecastingAlgorithm):
-    USE_GLUON_TS = True
-    SUPPORTS_EXTERNAL_FEATURES = True
+    ONE_MODEL_FOR_MULTIPLE_TS = True
+    EXTERNAL_FEATURES_COMPATIBILITY = ExternalFeaturesCompatibility.KNOWN_IN_ADVANCE
 
     def model_from_params(self, input_hp_space, modeling_params, core_params):
         gpu_config = get_gpu_config_from_core_params(core_params)
@@ -969,7 +1086,7 @@ class GluonTSMQCNN(TimeseriesForecastingAlgorithm):
 
 class AutoArima(TimeseriesForecastingAlgorithm):
     REFIT_FOR_SCORING = TimeseriesForecastingAlgorithm.ModelRefit.SUPPORTED
-    SUPPORTS_EXTERNAL_FEATURES = True
+    EXTERNAL_FEATURES_COMPATIBILITY = ExternalFeaturesCompatibility.KNOWN_IN_ADVANCE
     SUPPORTS_MODEL_COEFFICIENTS = True
     SUPPORTS_INFORMATION_CRITERIA = True
 
@@ -1049,7 +1166,7 @@ class AutoArima(TimeseriesForecastingAlgorithm):
 
 class Arima(TimeseriesForecastingAlgorithm):
     REFIT_FOR_SCORING = TimeseriesForecastingAlgorithm.ModelRefit.SUPPORTED
-    SUPPORTS_EXTERNAL_FEATURES = True
+    EXTERNAL_FEATURES_COMPATIBILITY = ExternalFeaturesCompatibility.KNOWN_IN_ADVANCE
     SUPPORTS_MODEL_COEFFICIENTS = True
 
     def model_from_params(self, input_hp_space, modeling_params, core_params):
@@ -1176,6 +1293,7 @@ class ETS(TimeseriesForecastingAlgorithm):
     REFIT_FOR_SCORING = TimeseriesForecastingAlgorithm.ModelRefit.SUPPORTED
     SUPPORTS_MODEL_COEFFICIENTS = True
     SUPPORTS_INFORMATION_CRITERIA = True
+    EXTERNAL_FEATURES_COMPATIBILITY = ExternalFeaturesCompatibility.NO_SUPPORT
 
     def model_from_params(self, input_hp_space, modeling_params, core_params):
         from dataiku.doctor.timeseries.models.statistical.ets import DkuETSEstimator
@@ -1226,10 +1344,126 @@ class ETS(TimeseriesForecastingAlgorithm):
         # Heuristic method needs at least 10 observations
         return max(prediction_length, modeling_params["ets_timeseries_grid"]["seasonal_periods"], 10)
 
+class NeuralForecastAlgorithm(TimeseriesForecastingAlgorithm):
+    EXTERNAL_FEATURES_COMPATIBILITY = ExternalFeaturesCompatibility.KNOWN_IN_ADVANCE_AND_PAST_ONLY
+    ONE_MODEL_FOR_MULTIPLE_TS = True
+
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
+        # requirement to compute quantiles
+        return max((NEURALFORECAST_N_WINDOWS + 1) * prediction_length, get_parameter_max_value(modeling_params, "context_length"))
+
+    @staticmethod
+    def get_gpu_config(core_params):
+        gpu_config = get_gpu_config_from_core_params(core_params)
+        if not NeuralforecastGPUCapability.should_use_gpu(gpu_config):
+            NeuralforecastGPUCapability.disable_all_cuda_devices()
+            gpu_config["params"]["useGpu"] = False
+        return gpu_config
+
+class TFT(NeuralForecastAlgorithm):
+    def model_from_params(self, input_hp_space, modeling_params, core_params):
+        from dataiku.doctor.timeseries.models.neuralforecast.tft import DKUTFTEstimator
+        estimator = DKUTFTEstimator(
+            frequency=get_frequency(core_params),
+            prediction_length=core_params[doctor_constants.PREDICTION_LENGTH],
+            time_variable=core_params[doctor_constants.TIME_VARIABLE],
+            target_variable=core_params[doctor_constants.TARGET_VARIABLE],
+            timeseries_identifiers=core_params[doctor_constants.TIMESERIES_IDENTIFIER_COLUMNS],
+            monthly_day_alignment=get_monthly_day_alignment(core_params),
+            patience=input_hp_space["patience"],
+            batch_size=input_hp_space["batch_size"],
+            random_state=input_hp_space["random_state"],
+            max_steps=input_hp_space["max_steps"],
+            quantiles=core_params[doctor_constants.QUANTILES], # quantiles have to be available at fit time,
+            gpu_config=self.get_gpu_config(core_params)
+        )
+
+        hp_names_to_dimension_class = {
+            "context_length": IntegerHyperparameterDimension,
+            "learning_rate": FloatHyperparameterDimension,
+            "hidden_size_factor": IntegerHyperparameterDimension,
+            "n_rnn_layers": IntegerHyperparameterDimension,
+            "n_head": IntegerHyperparameterDimension
+        }
+
+        hyperparameters_space = TFTHyperparameterSpace.from_definition(
+            input_hp_space,
+            hp_names_to_dimension_class=hp_names_to_dimension_class,
+            limit_hidden_size=input_hp_space["limit_hidden_size"],
+            max_hidden_size=input_hp_space["max_hidden_size"]
+        )
+
+        return TrainableModel(estimator, hyperparameters_space=hyperparameters_space)
+
+    def actual_params(self, ret, model, fit_params):
+        params = model.get_params()
+        ret["tft_params"] = {
+            "context_length": params["context_length"],
+            "learning_rate": params["learning_rate"],
+            "hidden_size": params["hidden_size"],
+            "hidden_size_factor": params["hidden_size_factor"],
+            "n_rnn_layers": params["n_rnn_layers"],
+            "n_head": params["n_head"],
+            "max_steps": params["max_steps"],
+            "patience": params["patience"],
+            "batch_size": params["batch_size"],
+            "random_state": params["random_state"]
+        }
+        amp = {"resolved": ret}
+        return amp
+
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
+        return modeling_params["tft_params"]["context_length"]
+
+class NHITS(NeuralForecastAlgorithm):
+    def model_from_params(self, input_hp_space, modeling_params, core_params):
+        from dataiku.doctor.timeseries.models.neuralforecast.nhits import DKUNHITSEstimator
+        estimator = DKUNHITSEstimator(
+            frequency=get_frequency(core_params),
+            prediction_length=core_params[doctor_constants.PREDICTION_LENGTH],
+            time_variable=core_params[doctor_constants.TIME_VARIABLE],
+            target_variable=core_params[doctor_constants.TARGET_VARIABLE],
+            timeseries_identifiers=core_params[doctor_constants.TIMESERIES_IDENTIFIER_COLUMNS],
+            monthly_day_alignment=get_monthly_day_alignment(core_params),
+            patience=input_hp_space["patience"],
+            batch_size=input_hp_space["batch_size"],
+            random_state=input_hp_space["random_state"],
+            max_steps=input_hp_space["max_steps"],
+            quantiles=core_params[doctor_constants.QUANTILES], # quantiles have to be available at fit time,
+            gpu_config=self.get_gpu_config(core_params)
+        )
+
+        hp_names_to_dimension_class = {
+            "context_length": IntegerHyperparameterDimension,
+            "learning_rate": FloatHyperparameterDimension,
+        }
+
+        hyperparameters_space = HyperparametersSpace.from_definition(
+            input_hp_space,
+            hp_names_to_dimension_class=hp_names_to_dimension_class,
+        )
+
+        return TrainableModel(estimator, hyperparameters_space=hyperparameters_space)
+
+    def actual_params(self, ret, model, fit_params):
+        params = model.get_params()
+        ret["nhits_params"] = {
+            "context_length": params["context_length"],
+            "learning_rate": params["learning_rate"],
+            "max_steps": params["max_steps"],
+            "patience": params["patience"],
+            "batch_size": params["batch_size"],
+            "random_state": params["random_state"]
+        }
+        amp = {"resolved": ret}
+        return amp
+
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
+        return modeling_params["nhits_params"]["context_length"]
 
 class Prophet(TimeseriesForecastingAlgorithm):
     REFIT_FOR_SCORING = TimeseriesForecastingAlgorithm.ModelRefit.SUPPORTED
-    SUPPORTS_EXTERNAL_FEATURES = True
+    EXTERNAL_FEATURES_COMPATIBILITY = ExternalFeaturesCompatibility.KNOWN_IN_ADVANCE
     SUPPORTS_MODEL_COEFFICIENTS = True
 
     def model_from_params(self, input_hp_space, modeling_params, core_params):
@@ -1297,6 +1531,55 @@ class Prophet(TimeseriesForecastingAlgorithm):
         return 2
 
 
+class Croston(TimeseriesForecastingAlgorithm):
+    SUPPORTS_QUANTILES = False
+    REFIT_FOR_SCORING = TimeseriesForecastingAlgorithm.ModelRefit.MANDATORY
+
+    def model_from_params(self, input_hp_space, modeling_params, core_params):
+        from dataiku.doctor.timeseries.models.statistical.croston import DkuCrostonEstimator
+        estimator = DkuCrostonEstimator(
+            frequency=get_frequency(core_params),
+            time_variable=core_params[doctor_constants.TIME_VARIABLE],
+            prediction_length=core_params[doctor_constants.PREDICTION_LENGTH],
+            target_variable=core_params[doctor_constants.TARGET_VARIABLE],
+            timeseries_identifier_columns=core_params[doctor_constants.TIMESERIES_IDENTIFIER_COLUMNS],
+            monthly_day_alignment=get_monthly_day_alignment(core_params),
+        )
+
+        hp_names_to_dimension_class = {
+            "variant": CategoricalHyperparameterDimension,
+        }
+
+        if input_hp_space["variant"]["values"]["TSB"]["enabled"]:
+            hp_names_to_dimension_class["alpha_d"] = FloatHyperparameterDimension
+            hp_names_to_dimension_class["alpha_p"] = FloatHyperparameterDimension
+
+        hyperparameters_space = CrostonHyperparametersSpace.from_definition(
+            input_hp_space,
+            hp_names_to_dimension_class=hp_names_to_dimension_class,
+        )
+
+        return TrainableModel(estimator, hyperparameters_space=hyperparameters_space)
+
+    def actual_params(self, ret, model, fit_params):
+        params = model.get_params()
+        ret["croston_timeseries_params"] = {
+            "variant": params["variant"],
+        }
+        if params["variant"] == 'TSB':
+            ret["croston_timeseries_params"]["alpha_d"] = params["alpha_d"]
+            ret["croston_timeseries_params"]["alpha_p"] = params["alpha_p"]
+        amp = {"resolved": ret}
+        return amp
+
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
+        # Refit before scoring is required
+        return self.get_min_size_for_training(modeling_params, preprocessing_params, prediction_length)
+
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
+        # Required for the ConformalIntervals calculation
+        return prediction_length * 2 + 1
+
 def get_parameter_max_value(modeling_params, parameter_name):
     input_hp_space = get_input_hyperparameter_space(modeling_params, modeling_params["algorithm"])
     parameter = input_hp_space[parameter_name]
@@ -1339,11 +1622,15 @@ TIMESERIES_FORECASTING_ALGORITHMS_MAP = {
     "GLUONTS_MQCNN": GluonTSMQCNN(),
     "AUTO_ARIMA": AutoArima(),
     "ARIMA": Arima(),
+    "CROSTON": Croston(),
     "ETS": ETS(),
+    "NHITS": NHITS(),
     "SEASONAL_LOESS": SeasonalTrendLoess(),
+    "TFT": TFT(),
     "PROPHET": Prophet(),
     # ML-based forecasting algorithms
     "RANDOM_FOREST_REGRESSION": RandomForestForecastingAlgorithm(),
     "XGBOOST_REGRESSION": XGBoostForecastingAlgorithm(),
     "RIDGE_REGRESSION": RidgeForecastingAlgorithm(),
+    "LIGHTGBM_REGRESSION": LightGBMForecastingAlgorithm(),
 }
