@@ -415,10 +415,10 @@ class KFoldTimeseriesSplitHandler(AbstractTimeseriesSplitHandler):
         helpers.append("the season length (if applicable)")
         return " Try to decrease {}. Or select 'Skip too short time series'.".format(" and/or ".join(helpers))
 
-class TimeseriesInteractiveScoringSplitHandler(KFoldTimeseriesSplitHandler):
+class TimeseriesSingleHorizonDefaultSplitHandler(KFoldTimeseriesSplitHandler):
     """
-    Extend TimeseriesSplitHandler to return only the last horizon as the test set for interactive scoring scenario creation.
+    Extend TimeseriesSplitHandler to return only the last horizon as the test set for interactive scoring scenario creation and permutation importance
     """
     def __init__(self, core_params):
-        super().__init__(1, core_params)
+        super(TimeseriesSingleHorizonDefaultSplitHandler, self).__init__(1, core_params)
         self.test_size = self.prediction_length
