@@ -7,7 +7,7 @@ import traceback
 from concurrent.futures import ThreadPoolExecutor
 from dataclasses import dataclass, asdict
 from pathlib import Path
-from typing import List, Awaitable, Optional
+from typing import List, Optional, Tuple
 
 import torch
 from docling.backend.docling_parse_v4_backend import DoclingParseV4DocumentBackend
@@ -35,9 +35,10 @@ from pydantic import AnyUrl
 
 from dataiku.base.batcher import Batcher
 from dataiku.doctor.utils.gpu_execution import TorchGpuCapability
+from dataiku.llm.docextraction import build_message_log_for_document, AbstractDocumentNode, ImageNode, TextNode, TableNode, SectionNode, SlideNode, RootNode
 from dataiku.llm.docextraction.ocr import get_ocr_config, process_images_with_ocr
 
-logger = logging.getLogger("docling_extraction")
+logger = logging.getLogger(__name__)
 
 
 @dataclass
@@ -50,6 +51,7 @@ class DoclingRequest:
     vlm_annotate: bool
     save_imgs: bool
 
+
 @dataclass
 class DoclingResponse:
     ok: bool
@@ -59,112 +61,12 @@ class DoclingResponse:
     def to_dict(self):
         return asdict(self)
 
-class DocumentNode:
-    """
-    A tree node structure to map docling converted structure to Dataiku structured extraction API: com.dataiku.dip.docextraction.StructuredContent
-    """
-    node_type = None
-    level: int
-
-    def __init__(self, node_id: str, children: List, label, content: str = "", level: int = None, page_provenance: List[ProvenanceItem] = None):
-        self.node_id = node_id
-        self.children = children  # List of children DocumentNode
-        self.label = label  # docling label see DocItemLabel, GroupLabel
-        self.content = content  # Content, text or base64 for images
-        self.level = level  # Level within the hierarchy
-        self.page_provenance = page_provenance  # page provenance from the original document
-
-    def to_dict(self):
-        res = {
-            "type": self.node_type,
-            "content": [child.to_dict() for child in self.children],
-        }
-        if self.page_provenance:
-            res["pages"] = [page.page_no for page in self.page_provenance]
-            if res["pages"]:
-                res["pageRange"] = {
-                    "start": min(res["pages"]),
-                    "end": max(res["pages"])
-                }
-        return res
-
-class ImageNode(DocumentNode):
-    node_type = "image"
-
-    def __init__(self, node_id, children, label, highest_confidence_class: Optional[PictureClassificationPrediction], content="", level=None, page_provenance=None,
-                 image_base64=None, image: ImageRef = None, caption=None):
-        super().__init__(node_id, children, label, content, level, page_provenance)
-        self.highest_confidence_class = highest_confidence_class
-        self.mime_type = image.mimetype if image is not None else None
-        self.height = image.size.height if image is not None else None
-        self.width = image.size.width if image is not None else None
-        self.resolution = image.dpi if image is not None else None
-        self.caption = caption
-        self.image_base64 = image_base64
-
-    def to_dict(self):
-        res = super().to_dict()
-        res["mimeType"] = self.mime_type
-        res["imageID"] = self.node_id
-        if self.height:
-            res["height"] = self.height
-        if self.width:
-            res["width"] = self.width
-        if self.resolution:
-            res["resolution"] = self.resolution
-        if self.content:
-            res["description"] = self.content
-        if self.image_base64:
-            res["imageRef"] = {
-                "type": "inline",
-                "content": self.image_base64,
-                "mimeType": self.mime_type
-            }
-        if self.caption:
-            res["caption"] = self.caption
-        if self.highest_confidence_class:
-            res["classificationData"] = {"className": self.highest_confidence_class.class_name}
-            if self.highest_confidence_class.confidence is not None:
-                res["classificationData"]["confidence"] = self.highest_confidence_class.confidence
-
-        return res
-
-class TextNode(DocumentNode):
-    node_type = "text"
-
-    def to_dict(self):
-        res = super().to_dict()
-        res["text"] = self.content
-        return res
-
-class TableNode(DocumentNode):
-    node_type = "table"
-
-    def to_dict(self):
-        res = super().to_dict()
-        res["text"] = self.content
-        return res
-
-class SectionNode(DocumentNode):
-    node_type = "section"
-
-    def to_dict(self):
-        res = super().to_dict()
-        res["title"] = self.content
-        res["level"] = self.level
-        return res
-
-class SlideNode(SectionNode):
-    node_type = "slide"
-
-class RootNode(DocumentNode):
-    node_type = "document"
 
 class DoclingExtractorPipeline:
-    batcher: Batcher[DoclingRequest, List[dict]]
+    batcher: Batcher[DoclingRequest, dict]
     image_formats: List[str] = ["png", "jpg", "jpeg"]
     supported_formats: List[str] = ["pdf", "docx", "pptx", "html"] + image_formats
-    documentBatchSize: int = 4
+    document_batch_size: int = 4
     config_optimization_mode: str = "AUTO"  # AUTO, FAST, ACCURACY
     device: AcceleratorDevice
     table_structure_options_mode: TableFormerMode
@@ -177,8 +79,8 @@ class DoclingExtractorPipeline:
         self.executor = ThreadPoolExecutor()
 
         if "documentBatchSize" in kernel_settings:
-            self.documentBatchSize = kernel_settings.get("documentBatchSize")
-            settings.perf.doc_batch_size = self.documentBatchSize
+            self.document_batch_size = kernel_settings.get("documentBatchSize")
+            settings.perf.doc_batch_size = self.document_batch_size
         if "pageBatchSize" in kernel_settings:
             settings.perf.page_batch_size = kernel_settings.get("pageBatchSize")
         if "configOptimizationMode" in kernel_settings:
@@ -192,7 +94,7 @@ class DoclingExtractorPipeline:
         self.table_structure_options_mode = self.get_docling_pdf_table_mode(self.config_optimization_mode)
         logger.info("Table structure mode used: %s", self.table_structure_options_mode)
 
-        # Pdf extraction with docling requires those IBM models tableformer and layout:  https://huggingface.co/ds4sd/docling-models
+        # PDF extraction with docling requires those IBM models tableformer and layout:  https://huggingface.co/ds4sd/docling-models
         # We'll check if the models are available within the resource folder. If this is the case we'll use them for extraction. If not we'll continue regular
         # execution (check hf cache check + download if not present)
         self.layout_model, self.artifacts_path = self.find_docling_pdf_models_in_resources(self.table_structure_options_mode,
@@ -201,8 +103,8 @@ class DoclingExtractorPipeline:
         # Layout Model
         logger.info("Layer model used: %s", self.layout_model.name)
 
-        self.batcher = Batcher[DoclingRequest, List[dict]](
-            batch_size=self.documentBatchSize,
+        self.batcher = Batcher[DoclingRequest, dict](
+            batch_size=self.document_batch_size,
             timeout=1,
             process_batch=self._process_batch_async,
             group_by=lambda request: hash((os.path.splitext(request.file_name)[1], request.do_ocr, tuple(request.lang)))
@@ -214,7 +116,7 @@ class DoclingExtractorPipeline:
         logger.info("Processing a batch of %s document extraction requests" % len(requests))
         return self.docling_batch_structured_extract(requests)
 
-    async def _process_batch_async(self, requests: List[DoclingRequest]) -> Awaitable[List[dict]]:
+    async def _process_batch_async(self, requests: List[DoclingRequest]) -> List[dict]:
         return await asyncio.get_running_loop().run_in_executor(self.executor, self._run_batch_sync, requests)
 
     async def process_document(self, process_document_command):
@@ -225,7 +127,8 @@ class DoclingExtractorPipeline:
                                                          process_document_command.get("imageHandlingMode", "IGNORE") == "VLM_ANNOTATE",
                                                          process_document_command.get("saveImgs", False)))
 
-    def find_docling_pdf_models_in_resources(self, table_mode: TableFormerMode, do_image_classification: bool) -> (LayoutModelConfig, Optional[Path]):
+    def find_docling_pdf_models_in_resources(self, table_mode: TableFormerMode, do_image_classification: bool) -> Optional[
+        Tuple[LayoutModelConfig, Optional[Path]]]:
         """
         Look in the resources folder whether the docling PDF models are available. Returns the path to the directory containing the models (that should be
         passed to docling) or None if one or more models is missing
@@ -381,10 +284,16 @@ class DoclingExtractorPipeline:
             text_results = process_images_with_ocr(ocr_options, [(idx, request.document_content) for idx, request in enumerate(requests)])
             for idx, request in enumerate(requests):
                 if idx in text_results:
-                    res.append(DoclingResponse(True, ImageNode(request.file_name, [], "image", None, text_results[idx], 0).to_dict(), None).to_dict())
+                    res.append(DoclingResponse(True, ImageNode(node_id=request.file_name,
+                                                               children=[],
+                                                               label="image",
+                                                               highest_confidence_class_name=None,
+                                                               highest_confidence_class_confidence=None,
+                                                               content=text_results[idx],
+                                                               level=0).to_dict(), None).to_dict())
                 else:
                     # something went wrong with the OCR processing
-                    res.append(DoclingResponse(False, None, f"Error processing image {request.file_name} with OCR").to_dict())
+                    res.append(DoclingResponse(False, {}, f"Error processing image {request.file_name} with OCR").to_dict())
             return res
         # handle PDFs
         elif extension == "pdf":
@@ -463,21 +372,24 @@ class DoclingExtractorPipeline:
                     ConversionStatus.SUCCESS,
                     ConversionStatus.PARTIAL_SUCCESS,
                 }:
-                    logger.error(build_message_log_for_document(conversion_result.document.name, f"An error occurred during docling processing : {conversion_result.errors}"))
-                    res.append(DoclingResponse(False, None, f"Conversion failed for: {conversion_result.document.name} with : {conversion_result.errors}").to_dict())
+                    logger.error(build_message_log_for_document(conversion_result.document.name,
+                                                                f"An error occurred during docling processing : {conversion_result.errors}"))
+                    res.append(
+                        DoclingResponse(False, {}, f"Conversion failed for: {conversion_result.document.name} with : {conversion_result.errors}").to_dict())
                 else:
                     res.append(
                         DoclingResponse(True, build_tree_with_outline(conversion_result.document, needs_ocr,
-                                                                   apply_additional_ocr_on_generated_images, ocr_options,
-                                                                   vlm_annotate, with_inline_images=with_inline_images).to_dict(), None).to_dict())
+                                                                      apply_additional_ocr_on_generated_images, ocr_options,
+                                                                      vlm_annotate, with_inline_images=with_inline_images).to_dict(), None).to_dict())
                     logger.info(build_message_log_for_document(conversion_result.document.name, "Done processing document with docling"))
             except Exception as e:
                 logger.exception("An error occurred during docling processing")
-                res.append(DoclingResponse(False, None, ''.join(traceback.format_exception(type(e), e, e.__traceback__))).to_dict())
+                res.append(DoclingResponse(False, {}, ''.join(traceback.format_exception(type(e), e, e.__traceback__))).to_dict())
         return res
 
+
 def build_tree_with_outline(document: DoclingDocument, needs_ocr, apply_additional_ocr_on_generated_images=False,
-                             ocr_options=None, vlm_annotate=False, with_inline_images=False) -> RootNode:
+                            ocr_options=None, vlm_annotate=False, with_inline_images=False) -> RootNode:
     """
     Given a docling conversion result. We build a tree that represent the structure of the documents. The nodes can either be sections, texts, images or tables.
 
@@ -493,7 +405,7 @@ def build_tree_with_outline(document: DoclingDocument, needs_ocr, apply_addition
     # We use a stack to keep track of the current title hierarchy. If a deeper section is encountered, it is added to the stack. This will ensure deeper text
     # have the whole outline. If a higher section is encountered, it is removed from the stack because it means we entered into a new section and the current
     # outline must be updated.
-    stack = [root]
+    stack: List[AbstractDocumentNode] = [root]
     # current_text holds the current text nodes that have not been yet added to the tree. Docling creates multiple paragraphs that we want to merge.
     # If we encounter a new section, an image, or a table, we need to merge the previous paragraphs and add them to the tree. To do so we use the append_pending_nodes_to_tree function.
     current_text: List[TextNode] = []
@@ -506,57 +418,60 @@ def build_tree_with_outline(document: DoclingDocument, needs_ocr, apply_addition
         images_text = apply_ocr_on_images_for_non_pdf_documents(document, ocr_options)
     # Captions for images are returned as part of ImageNode when doing vlm annotation, we keep track of them to avoid duplicates. Because document.iterate_items() will also return them
     # as text, and we don't want them to be considered as separate text nodes.
-    captions_cref_list = []
+    captions_cref_list: List[str] = []
     for node_item, level in document.iterate_items(with_groups=True):
         if not stack:
-            logger.error(build_message_log_for_document(document.source_file_path, "Stack is empty while building document tree for document. This should not happen."))
+            logger.error(
+                build_message_log_for_document(document.source_file_path, "Stack is empty while building document tree for document. This should not happen."))
             stack.append(root)
         if hasattr(node_item, "label"):
             # To build the tree structure we rely on the labeled nodes of docling. Docling identifies titles/section headers and groups (chapters, sections, slides) that we use to build the outline.
             # Groups do not have any content, but we still identify some as sections because they represent a logical section in the document for pptx files
             # In pptx documents, we need to rely on groups to recover the slide structure.
-            if node_item.label == DocItemLabel.TITLE or node_item.label == DocItemLabel.SECTION_HEADER :
+            if node_item.label == DocItemLabel.TITLE or node_item.label == DocItemLabel.SECTION_HEADER:
                 current_section = SectionNode(node_item.self_ref, [], node_item.label, node_item.text if hasattr(node_item, "text") else "", level,
-                                              node_item.prov if hasattr(node_item, "prov") else None)
+                                              [prov.page_no for prov in node_item.prov] if hasattr(node_item, "prov") else [])
                 handle_section_item(stack, current_section, level, current_text)
 
             elif type(node_item) == GroupItem and node_item.label in [GroupLabel.CHAPTER, GroupLabel.SLIDE]:
                 # For now docling identifies slide as chapter see https://github.com/docling-project/docling/blob/main/docling/backend/mspowerpoint_backend.py#L331-L335
                 # We should pay attention to any changes for next docling bump, for the moment chapter is only used in mspowerpoint_backend.py
-                current_section = SlideNode(node_item.self_ref, [], node_item.label, increment_slide(node_item.name) if hasattr(node_item, "name") else "", level, None)
+                current_section = SlideNode(node_item.self_ref, [], node_item.label, increment_slide(node_item.name) if hasattr(node_item, "name") else "",
+                                            level, [])
                 handle_section_item(stack, current_section, level, current_text)
 
 
             elif isinstance(node_item, ListItem) and node_item.text:
-                if current_text and not current_text[-1].label == DocItemLabel.LIST_ITEM:
+                if current_text and current_text[-1].label != DocItemLabel.LIST_ITEM:
                     # We have a new list item but the current_text is not a list item, so we need to flush the current_text to the tree first. And then start a new list item list.
                     append_pending_nodes_to_tree(stack, current_text)
                 current_text.append(TextNode(node_item.self_ref, [], DocItemLabel.LIST_ITEM, node_item.text, level,
-                                                node_item.prov if hasattr(node_item, "prov") else None))
+                                             [prov.page_no for prov in node_item.prov] if hasattr(node_item, "prov") else []))
 
             elif isinstance(node_item, PictureItem):
                 append_pending_nodes_to_tree(stack, current_text)
-                handle_image_item(document, node_item, needs_ocr, vlm_annotate, with_inline_images, apply_additional_ocr_on_generated_images, images_text, captions_cref_list, stack, image_idx, level)
+                handle_image_item(document, node_item, needs_ocr, vlm_annotate, with_inline_images, apply_additional_ocr_on_generated_images, images_text,
+                                  captions_cref_list, stack, image_idx, level)
                 image_idx += 1
             elif isinstance(node_item, TableItem):
-                    append_pending_nodes_to_tree(stack, current_text)
-                    stack[-1].children.append(TableNode(node_item.self_ref, [], node_item.label, node_item.export_to_markdown(document), level,
-                                                        node_item.prov if hasattr(node_item, "prov") else None))
+                append_pending_nodes_to_tree(stack, current_text)
+                stack[-1].children.append(TableNode(node_item.self_ref, [], node_item.label, node_item.export_to_markdown(document), level,
+                                                    [prov.page_no for prov in node_item.prov] if hasattr(node_item, "prov") else []))
             else:
                 # Make sure we don't add empty paragraphs
-                if hasattr(node_item, "text") and node_item.text and not node_item.self_ref in captions_cref_list:
+                if hasattr(node_item, "text") and node_item.text and node_item.self_ref not in captions_cref_list:
                     if current_text and current_text[-1].label == DocItemLabel.LIST_ITEM:
                         # We had a list item before, so we need to flush it to the tree first.
                         append_pending_nodes_to_tree(stack, current_text)
                     current_text.append(TextNode(node_item.self_ref, [], node_item.label, node_item.text, level,
-                                                 node_item.prov if hasattr(node_item, "prov") else None))
+                                                 [prov.page_no for prov in node_item.prov] if hasattr(node_item, "prov") else []))
     # Add last current_text
     if stack and current_text:
         append_pending_nodes_to_tree(stack, current_text)
     return root
 
+
 def apply_ocr_on_images_for_non_pdf_documents(document: DoclingDocument, ocr_options) -> dict:
-    images_text = {}
     image_refs = []
     for picture in document.pictures:
         # Check that docling correctly included the base64 image in the document. If so apply OCR on it.
@@ -564,22 +479,26 @@ def apply_ocr_on_images_for_non_pdf_documents(document: DoclingDocument, ocr_opt
                 isinstance(picture.image, ImageRef)
                 and isinstance(picture.image.uri, AnyUrl)
                 and picture.image.uri.scheme == "data"
+                and picture.image.uri.path is not None
         ):
             image_refs.append((picture.self_ref, picture.image.uri.path.split(",")[1]))
     images_text = process_images_with_ocr(ocr_options, image_refs)
     return images_text
 
-def handle_section_item(stack: List[DocumentNode], current_section: SectionNode, level: int, current_text: List[TextNode]):
+
+def handle_section_item(stack: List[AbstractDocumentNode], current_section: SectionNode, level: int, current_text: List[TextNode]):
     append_pending_nodes_to_tree(stack, current_text)
     # The stack carries the current hierarchy. If the current section is of a lower level (means higher in the hierarchy), then
-    # we need to remove the deeper levels from the stack. Level 0 is the root, level 6 is the max depth for markdown.
-    while stack and stack[-1].level >= level:
+    # we need to remove the deeper levels from the stack. Level 0 is the root, level 6 is the max depth for Markdown.
+    while stack and stack[-1].level is not None and stack[-1].level >= level:
         stack.pop()
     if stack:
         stack[-1].children.append(current_section)
     stack.append(current_section)
 
-def handle_image_item(document, image_item: PictureItem, needs_ocr, vlm_annotate, with_inline_images, apply_additional_ocr_on_generated_images, images_text, captions_cref_list, stack, image_idx, level):
+
+def handle_image_item(document, image_item: PictureItem, needs_ocr, vlm_annotate, with_inline_images, apply_additional_ocr_on_generated_images, images_text,
+                      captions_cref_list, stack, image_idx, level):
     """
     Handle an image item from docling conversion. Depending on the settings, it can extract OCR text from the image, or retrieve its base64 representation for VLM annotation (later in Java).
     :param document: docling document
@@ -587,7 +506,7 @@ def handle_image_item(document, image_item: PictureItem, needs_ocr, vlm_annotate
     :param needs_ocr: whether OCR is needed
     :param vlm_annotate: whether VLM annotation is needed
     :param with_inline_images: whether the base64 should be returned
-    :param apply_additional_ocr_on_generated_images: Docling only supports OCR on pdfs. For docx/pptx, docling does not apply OCR on images, if needed, we do it manually on the generated images.
+    :param apply_additional_ocr_on_generated_images: Docling only supports OCR on PDFs. For docx/pptx, docling does not apply OCR on images, if needed, we do it manually on the generated images.
     :param images_text: A dictionary of image cref to OCR text for images that needed additional OCR
     :param captions_cref_list:     # Captions for images are returned as part of ImageNode when doing vlm annotation, we keep track of them to avoid duplicates. Because document.iterate_items() will also return them
     :param stack: stack of current document hierarchy
@@ -600,6 +519,7 @@ def handle_image_item(document, image_item: PictureItem, needs_ocr, vlm_annotate
             isinstance(image_item.image, ImageRef)
             and isinstance(image_item.image.uri, AnyUrl)
             and image_item.image.uri.scheme == "data"
+            and image_item.image.uri.path is not None
     ) and with_inline_images:
         image_base64 = image_item.image.uri.path.split(",")[1]
     else:
@@ -611,7 +531,7 @@ def handle_image_item(document, image_item: PictureItem, needs_ocr, vlm_annotate
         # In docling 2.61.1, annotations are deprecated but meta field is not yet filled... We migrate them to avoid any future issues when annotations are removed.
         image_item._migrate_annotations_to_meta()
     if image_item.meta and image_item.meta.classification:
-        highest_confidence_class: PictureClassificationPrediction = image_item.meta.classification.get_main_prediction()
+        highest_confidence_class = image_item.meta.classification.get_main_prediction()
 
     if needs_ocr:
         if not apply_additional_ocr_on_generated_images:
@@ -630,20 +550,28 @@ def handle_image_item(document, image_item: PictureItem, needs_ocr, vlm_annotate
             if image_item.self_ref in images_text:
                 # Retrieve the OCR text for the image that we computed at the beginning of the function
                 content = images_text[image_item.self_ref]
-    elif vlm_annotate:
-        if image_base64 is not None:
-            # We look for caption to enrich the image Node
-            # This caption will also be returned on the iterate_items, so we need to keep track of them in order not to add them twice
-            if image_item.captions:
-                logger.info("trying to find caption for image:")
-                caption = find_captions_for_image(document, image_item)
-                captions_cref_list.extend([ref_item.cref for ref_item in image_item.captions])
+    elif vlm_annotate and image_base64 is not None:
+        # We look for caption to enrich the image Node
+        # This caption will also be returned on the iterate_items, so we need to keep track of them in order not to add them twice
+        if image_item.captions:
+            logger.info("trying to find caption for image:")
+            caption = find_captions_for_image(document, image_item)
+            captions_cref_list.extend([ref_item.cref for ref_item in image_item.captions])
+    highest_confidence_class_name = None
+    highest_confidence_class_confidence = None
+    if highest_confidence_class is not None:
+        highest_confidence_class_name = highest_confidence_class.class_name
+        highest_confidence_class_confidence = highest_confidence_class.confidence
     stack[-1].children.append(
-        ImageNode("image_" + str(image_idx), [], image_item.label, highest_confidence_class, content, level,
-                  image_item.prov if hasattr(image_item, "prov") else None,
+        ImageNode(node_id="image_" + str(image_idx), children=[],
+                  label=image_item.label, highest_confidence_class_name=highest_confidence_class_name,
+                  highest_confidence_class_confidence=highest_confidence_class_confidence,
+                  content=content, level=level,
+                  page_provenance=[prov.page_no for prov in image_item.prov] if hasattr(image_item, "prov") else [],
                   image_base64=image_base64,
                   image=image_item.image if hasattr(image_item, "image") else None, caption=caption))
 
+
 def find_captions_for_image(document: DoclingDocument, image_node: PictureItem) -> (Optional[str]):
     """
     Look if the image has any caption. Docling will have the caption as an image child if it's identified as such
@@ -660,14 +588,15 @@ def find_captions_for_image(document: DoclingDocument, image_node: PictureItem)
                 continue
         if caption:
             return caption
-        else:
-            return None
+    return None
 
-def append_pending_nodes_to_tree(stack: List[DocumentNode], current_text: List[TextNode]):
+
+def append_pending_nodes_to_tree(stack: List[AbstractDocumentNode], current_text: List[TextNode]):
     if stack and current_text:
         stack[-1].children.append(merge_paragraphs(current_text))
     current_text.clear()
 
+
 def merge_paragraphs(current_text: List[TextNode]) -> TextNode:
     node_id_parts, children, content_parts, provenance = [], [], [], []
     for node in current_text:
@@ -679,10 +608,8 @@ def merge_paragraphs(current_text: List[TextNode]) -> TextNode:
     content = "\n".join(content_parts)
     return TextNode(node_id, children, "merged_text", content, current_text[0].level if current_text else None, provenance)
 
-def build_message_log_for_document(source_file_path, message):
-    return f"[{source_file_path}] - {message}"
 
-def increment_slide(slide_name:str) -> str:
+def increment_slide(slide_name: str) -> str:
     # Docling slide index start from 0 which introduces a mismatch with the pageRange
     # We increment the slide-xx, to start at slide-1
     split_name = slide_name.split('-')
