@@ -1,16 +1,29 @@
+from typing import Optional
+
 import pandas as pd
-from langchain_core.callbacks import BaseCallbackHandler
-from langchain_core.outputs import LLMResult
 
 from dataiku.llm.types import CompletionSettings
-from dataikuapi.dss.llm_tracing import SpanBuilder
 
 
-def mean_or_none(pd_series):
+def mean_or_none(pd_series) -> Optional[float]:
     mean = pd_series.mean(skipna=True)
     return None if pd.isna(mean) else mean
 
 
+def median_or_none(pd_series) -> Optional[float]:
+    median = pd_series.median(skipna=True)
+    return None if pd.isna(median) else median
+
+
+def p95_or_none(pd_series: pd.Series) -> Optional[float]:
+    if pd_series.dropna().empty:  # dropna includes both NaN and None
+        # quantile will be nan in this case- we'd rather have None for serialization reasons
+        return None
+
+    # quantile will ignore NaN/None, so long as there is at least one non-NaN/None value
+    return float(pd_series.quantile(0.95))
+
+
 def get_llm_args(completion_settings: CompletionSettings):
     llm_args = {}
     if "temperature" in completion_settings:
@@ -23,13 +36,3 @@ def get_llm_args(completion_settings: CompletionSettings):
         llm_args["top_p"] = completion_settings["topP"]
     return llm_args
 
-
-class CompletionTraceHandler(BaseCallbackHandler):
-    span: SpanBuilder
-
-    def __init__(self, span: SpanBuilder):
-        self.span = span
-
-    def on_llm_end(self, response: LLMResult, **kwargs) -> None:
-        if response.llm_output is not None and "lastTrace" in response.llm_output:
-            self.span.append_trace(response.llm_output["lastTrace"])
