@@ -360,6 +360,12 @@ if __name__ == "__main__":
     # Start watchdog only after fork() to avoid thread-related funnyness in fork()
     watch_stdin(proc)
 
+    # catch sigterm that you get when DSS stops
+    def propagate_sigterm(s, f):
+        logging.warning("[Wrapper] propagate SIGTERM")
+        proc.kill(signal.SIGKILL)
+    signal.signal(signal.SIGTERM, propagate_sigterm)
+
     # Wait for process to exit
     status = proc.wait()
 
